
/* 
 * BruiView.m -- Implementation file for the BruiView class 
 * 
 * You may freely copy, distribute, and reuse the code in this example. 
 * NeXT disclaims any warranty of any kind, expressed or implied, as to its 
 * fitness for any particular use, nor even that it has a use. 
 * 
 */ 

#import  "BruiView.h"
#import  "drawingFuncs.h"
#import  "nxlcau23.h"
// #import  "nxardn23.h"
// #import  "nxrijn23.h"
#import  <math.h> 
#import  <stdlib.h>
#import  <string.h>
#include <stdio.h>
#include <time.h>

int    bruiBord;
int    bruiConst;
int    bruiBound;
int    bruiGen;
int    bruiMenu;
int    bruiPerm;
int    bruiShift;
int    bruiType;
int    bruiWidth;

extern int    co, dn;		/* connectivity, dimension */
extern int    no, lk;		/* # nodes, links 	   */

extern NXColor colst[KK];

extern int  ww[QWN][QWN];	/* connection matrix for de Bruijn nucleus  */
extern int  tox[QWN];		/* bit pattern of the top of the node	*/
extern int  box[QWN];		/* pattern of the bottom of the node	*/

extern int    qbno;
extern int    qnmax;
extern int    ndst, xdst, nloo, xloo;
extern int    noloo;		/* not part of any loop			*/
extern int    unlat;		/* unilaterally connected only		*/
extern int    nolat;		/* disconnected = different components	*/
extern double qcntr[QDN][2];

void   asctobin();
int    bottm(int n);
int    closeto(double x, double y);
void   dbdraw(int s, int n, int z, int f, int  t);
void   edijn(int g, int s, int p, int r);
void   edrui(int i);
int    fflip(int i, int w);
void   howdown(int *sud);
int    inlinks(int n);
void   inpull(int n);
int    level (int n);
void   makemap();
int    oulinks(int n);
void   printrail(char* ch, int i, int w, int l);
int    proximate(double ex, double wy);
void   prxtrmd(int *min, int *max);
void   prxtrml(int *min, int *max);
int    shflev(int g, int s);
void   shortevolve(int m, int n);
int    topof(int n);
void   upthere(int *sud);
void   wrru(char *c, char ru[KK][KK][KK][KK]);


@implementation BruiView

/* - - - - - - - -   F I L E   A N D   D I R E C T O R Y   - - - - - - - - */

- setFilename:(const char *)aFilename {
if (filename) free(filename);
filename = malloc(strlen(aFilename)+1);
strcpy(filename, aFilename);
[window setTitleAsFilename:aFilename];
return self;
}

- openBrui:sender {
  char     *types[2] = {"bru",0};
  int      i, j, k, t, b, q, fd;
  double   x, y;
  NXStream *theStream;

  [[OpenPanel new] allowMultipleFiles: NO];
  [[OpenPanel new] chooseDirectories: NO];

  if ([ [OpenPanel new] runModalForTypes:types]) {
    [self setFilename: [[OpenPanel new] filename]];
    fd = open(filename, O_RDONLY, 06666);
    theStream = NXOpenFile(fd, NX_READONLY);
//      NXScanf(theStream,"%s",cstr);
  [self showBrui: sender];
printf("openingBrui\n");
  NXScanf(theStream,"shift%d%d\n",&i,&j);
printf("shift %d %d\n",i,j);
  [sh0 selectCellAt: i: j];
  if (i!=-1) [self bruiShift: sh0];
  NXScanf(theStream,"shift%d%d\n",&i,&j);
printf("shift %d %d\n",i,j);
  [sh1 selectCellAt: i: j];
  if (i!=-1) [self bruiShift: sh1];
  NXScanf(theStream,"shift%d%d\n",&i,&j);
printf("shift %d %d\n",i,j);
  [sh2 selectCellAt: i: j];
  if (i!=-1) [self bruiShift: sh2];
  NXScanf(theStream,"const%d%d\n",&i,&j);
printf("const %d %d\n",i,j);
  [whichConstant selectCellAt: i: j];  
  if (i!=-1) [self bruiConst: whichConstant];
  NXScanf(theStream,"width%d%d\n",&i,&j);
printf("width %d %d\n",i,j);
  [whichWidth selectCellAt: i: j];
  [self runBrui: whichWidth];
//  [whichWidth setEnabled: YES];
    do {
	q=NXScanf(theStream,"%d, %lf, %lf\n",&i,&x,&y);
	qcntr[i][0]=x;
	qcntr[i][1]=y;
        } while (q!=EOF);
    qbno=0;
    NXClose(theStream);
    close(fd);
    [window setTitleAsFilename: filename];
    [window setDocEdited:NO];
    }
  [self redrawBrui: self];
  return self;
}

- saveBruiAs:sender {
  id panel;
  const char *dir;
  char *file;

/* prompt user for the file name and save to that file */

  if (filename==0) {
    /* no filename; set up defaults */ 
    dir  = NXHomeDirectory();
    file = (char *)[window title];
    } else {
    file=rindex(filename,'/');
    if (file) {
      dir   = filename;
      *file = 0;
      file++;
      } else {
      dir  = filename;
      file = (char *)[window title];
      }
    }
  panel = [SavePanel new]; 
  [panel setRequiredFileType: ""];
  if ([panel runModalForDirectory: dir file: file]) {
    [self setFilename: [panel filename]];
    return [self saveBrui: sender];
    }
    return nil;  /* didn't save */
  }
    
- saveBrui:sender {
  int      i, j, fd;
  NXStream *theStream;

  if (filename==0) return [self saveBruiAs: sender];
  [window setTitle: "Saving ..."];

  fd = open(filename, O_WRONLY | O_CREAT | O_TRUNC, 06666);
  if (fd < 0) {
    NXRunAlertPanel(0, "Cannot save file: %s",0,0,0,strerror(errno));
    return self;
    }
  theStream = NXOpenFile(fd, NX_WRITEONLY);
  NXPrintf(theStream,"shift %2d %2d\n",
	[sh0 selectedRow],[sh0 selectedCol]);
  NXPrintf(theStream,"shift %2d %2d\n",
	[sh1 selectedRow],[sh1 selectedCol]);
  NXPrintf(theStream,"shift %2d %2d\n",
	[sh2 selectedRow],[sh2 selectedCol]);
  NXPrintf(theStream,"const %2d %2d\n",
	[whichConstant selectedRow],[whichConstant selectedCol]);
  NXPrintf(theStream,"width %2d %2d\n",
	[whichWidth selectedRow],[whichWidth selectedCol]);
  for (i=0; i<qnmax; i++) NXPrintf(theStream,"%3d, %4.2f, %4.2f\n",
	i, qcntr[i][0], qcntr[i][1]);
  NXClose(theStream);
  close(fd);
//  [window setTitleAsFilename: filename];
  [wholePanel setDocEdited:NO];
  printf("saveBrui ended\n");
  return self;
}

- close:sender {const char *fname;
int         q;
  if ([wholePanel isDocEdited]) {
//    fname=filename ? filename : [sender title];
//    if (rindex(fname,'/')) fname = rindex(fname,'/') + 1;
    q = NXRunAlertPanel(
      "Save",
      "Save changes to %s?",
      "Save",
      "Don't Save",
      "Cancel",
      filename);
    }
    if (q==1) {if (![self saveBrui:nil]) return nil;}
    if(q==-1) return nil;  /* cancel the closing */
//  [sender setDelegate: nil];
//  [proc free];
//  [self free];
  return self;
}

- windowWillClose:sender {
printf("Brui windowWillClose\n");
  [self close: self];
//  [sender setDelegate: nil];
printf("yes, really\n");
return self;
}

/* -------------------------------- B R U I -------------------------------- */

- showBrui:sender {
printf("dbWindow is about to load %d\n",dn);
    if (db2Window==nil) {
      if (![NXApp loadNibSection: "brui.nib" owner: self withNames: NO]) 
	{return nil;}
//    rootViews = [[List alloc] init];
//    rootWindows = [[List alloc] init];
//    [cyWidth setEmptySelectionEnabled: YES];
//    [cyWidth selectCellAt: -1: 0];
//    [browser setEmptySelectionEnabled: YES];
//    [browser setMultipleSelectionEnabled: NO];
//    [browser getTitleFromPreviousColumn: NO];
//    [browser setTitle: "components" ofColumn: 0];
//    [self setGraphData];
    }
 	      [db2Window makeKeyAndOrderFront: sender];
//	      [db3Window performClose: sender];
//  wru(ch);
//  [ruleNumber setStringValue: ch];
//   [self bruiReset: self];
   [self bruiReset: self];

  [nodeNumber setIntValue: 0];
//  [self ediOpt: self];
printf("dbWindow is loaded\n");
  return self;
}

- bruiShift:sender {
       if (sender==sh0) bruiGen=0; 
  else if (sender==sh1) bruiGen=1;
  else if (sender==sh2) bruiGen=2;
  bruiShift=[sender selectedCol];
  [self bruiReset: self];
       if (bruiGen==0) [sh0 selectCellAt: 0: bruiShift];
  else if (bruiGen==1) [sh1 selectCellAt: 0: bruiShift];
  else if (bruiGen==2) [sh2 selectCellAt: 0: bruiShift];
  [whichWidth setEnabled: YES];
  bruiType=3*bruiGen+bruiShift;
  edrui(bruiType);
return self;
}

- bruiConst:sender {
  bruiGen=0;
  bruiShift=[sender selectedCol];
  [self bruiReset: self];
  [whichConstant selectCellAt: 0: bruiShift];
  [whichWidth setEnabled: YES];
  bruiType=bruiShift+'e';
  edrui(bruiType);
return self;
}

- bruiReset:sender {
  [sh0 setEmptySelectionEnabled: YES];
  [sh0 selectCellAt: -1: 0];
  [sh1 setEmptySelectionEnabled: YES];
  [sh1 selectCellAt: -1: 0];
  [sh2 setEmptySelectionEnabled: YES];
  [sh2 selectCellAt: -1: 0];
  [whichConstant setEmptySelectionEnabled: YES];
  [whichConstant selectCellAt: -1: 0];
  [whichWidth setEmptySelectionEnabled: YES];
  [whichWidth selectCellAt: -1: 0];
  [whichWidth setEnabled: NO];
return self;
}

- runBrui:sender {int i, j;
  bruiWidth=[sender selectedCol];
  bruiBound=[sender selectedRow];
  bruiBord=bruiWidth+2;
  bruiMenu=bruiWidth+'A';
  if (bruiBound==1) {bruiMenu += 'M'-'A'; bruiBord-=2;}
  edrui(bruiMenu);
  [[trivGnrl cellAt: 0: 0] setIntValue: no];
  [[trivGnrl cellAt: 1: 0] setIntValue: lk];
  makemap();
  [[trivGnrl cellAt: 2: 0] setIntValue: nloo];
  [[trivGnrl cellAt: 3: 0] setIntValue: xloo];
  [[trivGnrl cellAt: 4: 0] setIntValue: ndst];
  [[trivGnrl cellAt: 5: 0] setIntValue: xdst];
  [[trivGnrl cellAt: 6: 0] setIntValue: noloo];
  [[trivGnrl cellAt: 7: 0] setIntValue: unlat];
  [[trivGnrl cellAt: 8: 0] setIntValue: nolat];
  qcntr[0][0]=0.0;
  qcntr[0][1]=0.0;
  for (i=1; i<no; i++) {
    qcntr[i][0]=(double)(rand()%400)-200.0;
    qcntr[i][1]=(double)(rand()%400)-200.0;
    j=fflip(i,bruiBord+1);
    if (i==j) {
      qcntr[j][0]=0.0;
      qcntr[j][1]=-qcntr[i][1];
      } else {
      qcntr[j][0]=-qcntr[i][0];
      qcntr[j][1]=-qcntr[i][1];
      }
    }
  [self redrawBrui: self];
return self;
}

- printFirst:sender {edrui('8'); return self;}
// - printFirst:sender {edrui('='); return self;}

- redrawBrui:sender {
  [wholePanel makeKeyAndOrderFront: self];
  [guphView display];
return self;
}

- printBrui:sender {
  [wholePanel smartPrintPSCode:self];
return self;
}

- locAte:sender {char *c; int i;
  i=[nodeNumber intValue];
  c=[[dbNode selectedCell] title];
  if (i<no) {
    [nodeNumber setIntValue: qbno];
    if (strcmp(c,"self")==0) qbno=i;
    if (strcmp(c,"flip")==0) qbno=fflip(i,bruiBord+1);
    if (strcmp(c,"none")==0) qbno=qnmax;
//    if (strcmp(c,"root")==0) qbno=cyroot[i];
    [nodeNumber setIntValue: qbno];
    } 
  else [nodeNumber setIntValue: qnmax];
  [self setRuTriv];
  [self redrawBrui: self];
return self;
}

- setRuTriv {char ch[13]; int i, j, k, m;
	[[trivNode cellAt: 0: 0] setIntValue: qbno];
	[[trivNode cellAt: 1: 0] setIntValue: inlinks(qbno)];
	[[trivNode cellAt: 2: 0] setIntValue: oulinks(qbno)];
	printrail(ch,topof(qbno),bruiBord-1,NO);
	[[trivNode cellAt: 3: 0] setStringValue: ch];
	printrail(ch,bottm(qbno),bruiBord-1,NO);
	[[trivNode cellAt: 4: 0] setStringValue: ch];
  [nodeNumber setIntValue: qbno];
  return self;
}

- bruiMove:sender 
{char *c; int i, n; double p, th, x, y, t, x0, y0;
  c=[[sender selectedCell] title];
         if (strcmp(c,"dilate")==0) {
	  for (i=0; i<QDN; i++) {qcntr[i][0]*=1.125; qcntr[i][1]*=1.125;}
  } else if (strcmp(c,"shrink")==0) {
	  for (i=0; i<QDN; i++) {qcntr[i][0]/=1.1667; qcntr[i][1]/=1.1667;}
  } else if (strcmp(c,"save")==0) {
for (i=0; i<QDN; i++) {qcntr[i][2]=qcntr[i][0]; qcntr[i][3]=qcntr[i][1];}
  } else if (strcmp(c,"restore")==0) {
for (i=0; i<QDN; i++) {qcntr[i][0]=qcntr[i][2]; qcntr[i][1]=qcntr[i][3];}
  } else if (strcmp(c,"left")==0) {
	  for (i=0; i<QDN; i++) qcntr[i][0]-=15.0;
  } else if (strcmp(c,"right")==0) {
	  for (i=0; i<QDN; i++) qcntr[i][0]+=15.0;
  } else if (strcmp(c,"up")==0) {
	  for (i=0; i<QDN; i++) qcntr[i][1]-=15.0;
  } else if (strcmp(c,"down")==0) {
	  for (i=0; i<QDN; i++) qcntr[i][1]+=15.0;
  } else if (strcmp(c,"rotate")==0) {
	  th=0.314;
	  for (i=0; i<QDN; i++) {
	    x=qcntr[i][0]; y=qcntr[i][1];
	    qcntr[i][0]= x*cos(th)+y*sin(th);
	    qcntr[i][1]=-x*sin(th)+y*cos(th);
	    }
  } else if (strcmp(c,"horiz")==0) {
	  for (i=0; i<QDN; i++) qcntr[i][0]=-qcntr[i][0];
  } else if (strcmp(c,"vert")==0) {
	  for (i=0; i<QDN; i++) qcntr[i][1]=-qcntr[i][1];
  }
  [self redrawBrui: self];
  return self;
}

- showTrail:sender {int tl;
  [trailWindow makeKeyAndOrderFront: sender];
  tl=[trailText textLength];
  [trailText setSel: tl: tl];
  [trailText replaceSel: "\n"];
  [trailText scrollSelToVisible]; 
  [trailText display];
return self;
}

/* -------------------------------- C Y C L -------------------------------- */

/* the cycle programs have been moved to cycle.nib and associated files */

/* -------------------------------- P A I R -------------------------------- */

/* the pair programs have been moved to pair.nib and associated files */
/* even so, they require too much space and time to be used in lcau23 */

/* -----------------------------  S U B S E T  ---------------------------- */

/* the subset programs have been moved to suset.nib and associated files  */
/* even so, they require far too much space and time to be used in lcau23 */


/* - - - - - - - - - - - - -    R U I N V I E W   - - - - - - - - - - - - - */


- initFrame:(const NXRect *)frameRect
/*
 * Initializes the new BruiView object.  First, an initFrame: message is sent 
 * to super to initialize BruiView as a View.  Next, the BruiView sets its own 
 * state -- that it is opaque and that the origin of its coordinate system lies 
 * in the center of its area.  It then creates and initializes its associated 
 * objects, a Storage object, an NXCursor, and a Cell.  Finally, it loads into 
 * the Window Server some PostScript procedures that it will use in drawing 
 * itself. 
 */
{
//    NXPoint	spot;
	
    [super initFrame: frameRect];
    [self translate: floor(frame.size.width/2): floor(frame.size.height/2)];
    [self setOpaque:YES];
//    crossCursor = 
//	[NXCursor newFromImage: [NXImage newFromSection: "circle.tiff"]];
//    spot.x = spot.y = 7.0;
//    [crossCursor setHotSpot:&spot];

    [self setFlipped: YES];

    return self;

printf("initFrame from BruiView\n");
}


- setDelegate:anObject
/*
 * Sets the BruiView's delegate instance variable to the supplied object.
 */
{
	delegate = anObject;
	return self;
}


- drawSelf:(const NXRect *)rects :(int)rectCount
/*
 * Draws the BruiView's background and axes.  If there are any points,
 * these are drawn too.
 *
 * (Note:  For simplicity, although BruiView only repaints the background 
 * of the update region, it redraws the entire axes.  For better performance, 
 * only those parts of the axes that fall within the update region should 
 * be redrawn.)   
 */
{
        char             c[WNML], textline[80], *tshf, *tbou;
	unsigned int	 i, j, k;
        double           p, q, s;
  struct tm  *timeptr;
  time_t  secsnow;

int     i0, i1, ss;
	
	if (rects == NULL) return self;

	/* 
	 * If we're printing, we need to load drawing procedures to the 
	 * printing context.  This simple test is OK for a View that prints a 
	 * single page.  If there were multiple pages, the procedures would be 
	 * loaded multiple times.  Although this is inefficient, it causes no 
	 * error.  Views that print multiple pages should load procedures such 
	 * as these in an endPrologue method (see the View spec sheet). 
	 */

	/* paint visible area white then draw axes */
	if ([window canStoreColor]) PSsetgray(NX_LTGRAY); 
		else PSsetgray(NX_WHITE);
	NXRectFill(&rects[0]);
	PSsetgray(NX_DKGRAY);

   time(&secsnow);
   timeptr = localtime(&secsnow);
   sprintf(textline,"%d-%d-19%02d\n",
	((timeptr->tm_mon) + 1), 
	timeptr->tm_mday,
        timeptr->tm_year);

  	[[Font newFont: "Courier" size: 12.0 matrix: NX_FLIPPEDMATRIX] set];
	PSsetgray(NX_BLACK);
		dbdraw( [treeShow selectedRow],
			qbno,
			[zeroLinks state],
			[selfLinks state],
			[labelStyle selectedRow] );
		PSstroke();

  	[[Font newFont: "Times-Bold" size: 18.0 matrix: NX_FLIPPEDMATRIX] set];
		PSsetgray(NX_BLACK); 
	PSmoveto(bounds.origin.x+10.0,bounds.origin.y+21.0);
	switch (bruiType) {
	  case 'e': tshf="Zeroes"; break;
	  case 'f': tshf="Ones"; break;
	  case  4:  tshf="Still Life"; break;
	  case  5:  tshf="Longitudinal"; break;
	  case  2:  tshf="Diagonal"; break;
	  case  1:  tshf="Transversal"; break;
	  default:  tshf=""; break;}
	switch (bruiBound) {
	  case 0:  tbou="Cyclic"; break;
	  case 1:  tbou="Freestanding"; break;
	  default: tbou=""; break;}
	sprintf(textline,"%s %s Width %2d",tbou,tshf,bruiWidth+1);
	PSshow(textline);

	return self;
}

- sizeTo:(NXCoord)width :(NXCoord)height
/*
 * Ensures that whenever the BruiView is resized, the origin of its 
 * coordinate system is repositioned to the center of its area.
 */
{
 	[self setFlipped: YES];
	[super sizeTo:width :height]; 
	[self setDrawOrigin:-floor(width/2) : -floor(height/2)]; 
   [self display];
	return self;
}

- mouseDown:(NXEvent *) theEvent
/*
 * Responds to a message the system sends whenever the user presses the mouse
 * button when the cursor is over the BruiView.  The BruiView changes the 
 * cursor to a cross-hairs image and then starts asking for mouse-dragged or 
 * mouse-up events.  As it receives mouse-dragged events, the BruiView updates 
 * the readOut text Cell with the cursor's coordinates.  If the user releases 
 * the mouse button while the cursor is over the BruiView, the BruiView 
 * registers the point and then sends a message to its delegate notifying it of 
 * the new point.
 */
{
	char    ch[13];
	int	looping = YES, oldMask;
	NXPoint	aPoint;

         double  x, y, ex, wy;
	 int     i, j, k, tl;

	[crossCursor set];
	oldMask = [window addToEventMask:NX_MOUSEDRAGGEDMASK];

/* for the 2-D de Bruijn  diagram --- - - - - - - - - - - - - - - - - - - */

 			[window setDocEdited:YES];
		        aPoint = theEvent->location;
		        [self convertPoint:&aPoint fromView:nil];
                        ex=aPoint.x;
                        wy=aPoint.y;
			qbno=proximate(ex,wy);
			[nodeNumber setIntValue: qbno];

		[self setRuTriv];
		if ([leaveTrail state]) {
			printrail(ch,topof(qbno),bruiBord-1,YES);
			printf("%s",ch);
			  tl=[trailText textLength];
			  [trailText setSel: tl: tl];
			  [trailText replaceSel:  ch];
			  [trailText scrollSelToVisible]; 
			  [trailText display];
			}

       if (theEvent->data.mouse.click==3) {} else 
       if (theEvent->data.mouse.click==2) {
		inpull(qbno);
		} else {
	[self lockFocus];
        PSsetinstance(YES);
	do {
 		PSnewinstance();
		aPoint = theEvent->location;
		[self convertPoint:&aPoint fromView:nil];
                ex=aPoint.x;
                wy=aPoint.y;
			PSsetgray(NX_BLACK);
			PSmoveto(ex+12.0,wy);
			PSarc(ex, wy, 12.0, 0.0, 360.0);
			PSstroke();
			qcntr[qbno][0]=ex;
			qcntr[qbno][1]=wy;
	if (strcmp([[dragStyle selectedCell] title],"hpair")==0) {
			    j=fflip(qbno,bruiBord+1);
			    if (j!=qbno) {
				qcntr[j][0]=-ex;
				qcntr[j][1]= wy;
				}
		}
	if (strcmp([[dragStyle selectedCell] title],"ipair")==0) {
			    j=fflip(qbno,bruiBord+1);
			    if (j!=qbno) {
				qcntr[j][0]=-ex;
				qcntr[j][1]=-wy;
				}
		}

		[window flushWindow];
		if (theEvent->type == NX_MOUSEUP) {
	    	/* on mouse-up, clean up state */
			[window flushWindow];
			looping = NO;
			}
   	} while (looping && (theEvent= 
		[NXApp getNextEvent: NX_MOUSEUPMASK | NX_MOUSEDRAGGEDMASK]));
	[self unlockFocus];
	}

   [self display];

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */ 

	[window setEventMask:oldMask];
	[NXArrow set];
	return self;
}

@end

/* end BruiView.m */
