
/* 
 * HEXLView.m -- Implementation file for the HEXLView class 
 *
 * You may freely copy, distribute, and reuse the code in this example.
 * NeXT disclaims any warranty of any kind, expressed or implied, as to 
 * its fitness for any particular use, nor even that it has a use.
 *
 */

#import "HEXLView.h"
#import "drawingFuncs.h"
#import "nxlcau33.h"
#import <math.h> 
#import <stdlib.h>
#import <string.h>

#define MB 12				/* maximum binomial coefficient */
#define NG 13				/* number of Golay Classes	*/

extern char ascrule[KK][KK][KK][KK][KK][KK][KK];
extern char auxrule[KK][KK][KK][KK][KK][KK][KK];
extern char golRule[];			/* evolutionary rule */
extern char sqrule[];			/* evolutionary rule */
extern char ttrule[];			/* evolutionary rule */

extern char    arr1[], arr2[];
extern double  bp[];			/* Bernstein coefficients    */
extern int     tc[];			/* totalistic key array      */
extern int     hxtus[];			/* totalistic use stats      */
extern int     lf[];			/* number of live cells	     */
       int     llf[KK];			/* save number of live cells */

extern NXColor colst[KK];    		/* Array for state colors    */

int    hxCheckBoard=NO;
int    drawHXBack=YES;
int    hxDrawBnstein=NO;
int    hxDrawMosaic=NO;
int    hxDrawFreqs=NO;
int    hxDrawPlane=NO;
int    hxDrawRturn=NO;
int    hxDrawSmall=NO;
int    hxDrawTrajec=NO;
int    hxDrawQuency=NO;
int    hxSlowFreqs=NO;
int    hxSmallSmall=NO;
int    hxSmallMosaic=NO;

int    hxLastPlane=0;			/* 0 = none, 1 = evol, 2 = mosaic */
int    hxLastTraj=0;			/* 0 = none, 1 = traj, 2 = return */
int    hxMoTiles=3;
int    phexDots=NO;
int    hxSkipThree=NO;
int    hxTrajLen=10;
int    hxRepPlEv;
int    hxRturnNum=1;

int     hxtotpmut[TS] = {0,1,2,3,4,5,6,7};
int     hxgtb[NG] = {0,1,2,2,2,3,3,3,4,4,4,5,6};
double  hxttb[TS] = {1.0,7.0,21.0,35.0,35.0,21.0,7.0,1.0};
NXColor hxtotcolst[TS];    		/* Array for totalistic colors    */

extern int    drawview;
extern int    co, dn;

void    hxgfram(int n, int l);
void    hxgdiag(int n, int l);
void    asctobin();
void	ascintohex();
void	asczero();
void    bhxgraf(int m, int n);
void    hxbdot(int i, int j, int l);
int     istot();
int     ix2(int i, int j);
int     ix3(int i, int j, int k);
int     nearest(double x, double y);
void    onegen(int n);
void    onehexgen(char *aa, char *bb, int m, int n); 
void    ranblin(double p1, double p2);
void    renewarry();
void    savearry();
void    screen(char *aa, int m, int n);
void    hxstatecount(char *aa, int m, int n);
void    hxSuggrule(int i);
void    videodot(int i, int j, int l);
void    wrru(char *c, char ru[KK][KK][KK][KK][KK][KK][KK]);
void    whxsscrrul(char *c, char ru[KK][KK][KK][KK][KK][KK][KK]);
void    zerline();

@implementation HEXLView
 
/* -------------------------------- S Q A R -------------------------------- */

/* put up the principal window for the PairView. Since pair.nib is a 	  */
/* secondary .nib file, it is not loaded until there is a demand, which   */
/* would usually arise from the "Pair" option in the main menu.	   	  */

- loadHexl:sender {
  if (hexlWindow==nil) {
    if (![NXApp loadNibSection: "hexl.nib" owner: self withNames: NO]) 
	{return nil;}
    }
  dn=2;
  [HexlWindow makeKeyAndOrderFront: sender];
  [hexlWindow orderFront: nil];
  [GolayPanel orderFront: nil];
  [self copyRule: sender];
return self;
}

void hxDrawsquare() {int i, j;
  for (i=0; i<128; i++) for (j=0; j<128; j++) videodot(j,i,arr1[i*128+j]+1);
}

void hxDrawcube() {int i, j, k;
  for (i=0; i<32; i++) for (j=0; j<32; j++) for (k=0; k<16; k++) 
    videodot(j+32*(k/4),i+32*(k%4),arr1[ix3(i,j,k)]+1);
}

void hxManysquare(i,j) int i, j; {int k, l;
  for (k=0; k<128; k++) for (l=0; l<128; l++)  
    videodot(k+128*i,l+128*j,arr1[ix2(k,l)]+1);
}

void hxDrawgrid() {int i;
	  PSsetgray(NX_BLACK); 
	  PSsetlinewidth(0.1);
  for (i=0; i<5; i++) {
  PSmoveto((double)(128.0*i),0.0); PSlineto((double)(128.0*i),512.0);}
  for (i=0; i<5; i++) {
  PSmoveto(0.0,(double)(128.0*i)); PSlineto(512.0,(double)(128.0*i));}
  PSstroke();
}

- printHEXL:sender {
   if (keyWindow==HexlWindow) 
	{[HexlWindow smartPrintPSCode: self];}
   else if (keyWindow==planeWindow) {
    switch (hxLastPlane) {
      case 0: break;
      case 1: hxDrawPlane=YES;   break;
      case 2: hxDrawMosaic=YES;  break;
      case 3: hxDrawSmall=YES;   break;
      case 4: hxSmallMosaic=YES; break;
      default: break;}
    [planeWindow smartPrintPSCode: self];
    switch (hxLastPlane) {
      case 0: break;
      case 1: hxDrawPlane=NO;   break;
      case 2: hxDrawMosaic=NO;  break;
      case 3: hxDrawSmall=NO;   break;
      case 4: hxSmallMosaic=NO; break;
      default: break;}
    }
return self;
}

- windowDidBecomeKey:sender {
	keyWindow=sender;
	[menuCopy setTarget: self];
	[menuCopy setAction: @selector(copyHEXL:)];
	[menuCopy setEnabled: YES];
	[menuPrint setTarget: self];
	[menuPrint setAction: @selector(printHEXL:)];
	[menuPrint setEnabled: YES];
return self;
}

- windowDidResignKey:sender {
	[menuCopy setEnabled: NO];
	[menuPrint setEnabled: NO];
return self;
}

- copyHexlView:sender {
NXRect zbounds;
id     pBoard;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [HexlView getBounds: &zbounds];
  [HexlView writePSCodeInside: &zbounds to: pBoard]; 
return self;
}

- copyGolayPanel:sender {
NXRect zbounds;
id     pBoard;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [GolayView getBounds: &zbounds];
  [GolayView writePSCodeInside: &zbounds to: pBoard]; 
return self;
}

- copyQuency:sender {
NXRect zbounds;
id     pBoard;
  hxDrawQuency = YES;
  drawview = 14;
  drawHXBack = NO;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  if ([justTheView state]) {
    [freqView getBounds: &zbounds];
    [freqView writePSCodeInside: &zbounds to: pBoard]; 
    } else {
    [[freqWindow contentView] getBounds: &zbounds];
    [[freqWindow contentView] writePSCodeInside: &zbounds to: pBoard]; 
    }
  drawHXBack = YES;
  hxDrawQuency = NO;
return self;
}

- copyWhoosit: (int *) flag {
NXRect zbounds;
id     pBoard;
  *flag = YES;
  drawview=13;
  drawHXBack=NO;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [trajView getBounds: &zbounds];
  [trajView writePSCodeInside: &zbounds to: pBoard]; 
  drawHXBack=YES;
  *flag = NO;
return self;
}

- copyWhatsit: (int *) flag {
NXRect zbounds;
id     pBoard;
  *flag = YES;
  drawHXBack=NO;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [planeView getBounds: &zbounds];
  [planeView writePSCodeInside: &zbounds to: pBoard]; 
  drawHXBack=YES;
  *flag = NO;
return self;
}

- copyHEXL:sender {
   if (keyWindow==HexlWindow) {[self copyHexlView: self];}
   else if (keyWindow==GolayPanel) {[self copyGolayPanel: self];}
   else if (keyWindow==trajWindow) {
   	savearry();
	switch (hxLastTraj) {
          case 0: break;
          case 1: [self copyWhoosit: &hxDrawTrajec]; break;
          case 2: [self copyWhoosit: &hxDrawRturn];  break;
          case 3: [self copyWhoosit: &hxDrawFreqs];  break;
          case 4: [self copyWhoosit: &hxSlowFreqs];  break;
          default: break;}
        renewarry();}
   else if (keyWindow==planeWindow) switch (hxLastPlane) {
     case 0: break;
     case 1: savearry(); [self copyWhatsit: &hxDrawPlane]; renewarry(); break;
     case 2: savearry(); [self copyWhatsit: &hxDrawMosaic]; renewarry(); break;
     case 3: drawview=7;  [self copyWhatsit: &hxDrawSmall];   break;
     case 4: savearry(); drawview=11; [self copyWhatsit: &hxSmallMosaic]; 
		renewarry(); break;
     default: break;}
   else if (keyWindow==freqWindow) [self copyQuency: self];
return self;
}

- drawPlEvol:sender {int i;
  hxRepPlEv=[[sender selectedCell] tag];
  drawview=7;
  hxDrawPlane=YES;
  asctobin();
  if (hxRepPlEv==0) [self display]; else for (i=0; i<hxRepPlEv; i++) {
     [genEv setIntValue: i];
     onegen(AL);
     [self display];
     }
  [genEv setIntValue: hxRepPlEv];
  hxDrawPlane=NO;
  hxLastPlane=1;
return self;
}

- printPlEvol:sender {
    hxRepPlEv=0;
    drawview=7;
    hxDrawPlane=YES;
    [keyWindow smartPrintPSCode: self];
    hxDrawPlane=NO;
return self;
}

- drawMosaic:sender {
    drawview=11;
    hxDrawMosaic=YES;
    [self display];
    hxDrawMosaic=NO;
    hxLastPlane=2;
return self;
}

- printMosaic:sender {
    drawview=11;
    hxDrawMosaic=YES;
    [keyWindow smartPrintPSCode: self];
    hxDrawMosaic=NO;
return self;
}

- showProbability:sender {int i; double pp, p[KK];
  for (i=0; i<KK; i++) p[i]=[[sender cellAt: i: 0] doubleValue];
  for (pp=0, i=0; i<KK; i++) pp+=p[i];
  for (i=0; i<KK; i++) {
    [[pSliders cellAt: i: 0] setDoubleValue: p[i]/pp];
    [[pWindows cellAt: i: 0] setDoubleValue: p[i]/pp];
    }
return self;
}

- randomLine:sender {double p[KK]; int i;
  for (i=0; i<KK; i++) p[i]=[[pSliders cellAt:i:0] doubleValue];
  ranblin(p[0],p[1]);
return self;
}

- sCellLine:sender {
  hxSmallSmall=YES;
  [planeView display];
  hxSmallSmall=NO;
return self;
}

- uniformLine:sender {ranblin(0.5,0.5); return self;}

- checkerBoard:sender {
  hxCheckBoard=YES;
  [planeView display];
  hxCheckBoard=NO;
return self;
}

- chmann:sender {nutoto(33); totoasc(); [self anyrule: self]; return self;}
- golife:sender {conlif(); [self anyrule: self]; return self;}

- earule:sender {hxSuggrule(0); [self anyrule: self]; return self;}
- nerule:sender {hxSuggrule(1); [self anyrule: self]; return self;}
- nwrule:sender {hxSuggrule(2); [self anyrule: self]; return self;}
- werule:sender {hxSuggrule(3); [self anyrule: self]; return self;}
- swrule:sender {hxSuggrule(4); [self anyrule: self]; return self;}
- serule:sender {hxSuggrule(5); [self anyrule: self]; return self;}
- idrule:sender {hxSuggrule(6); [self anyrule: self]; return self;}
- mjrule:sender {hxSuggrule(7); [self anyrule: self]; return self;}

- anyrule:sender {
   [rulePanel setAllRules: self];
   [self copyRule: self];
return self;}

void hxSuggrule(i) int i; {int j, k[KK], i0, i1, i2, i3, i4, i5, i6;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
  for (i4=0; i4<KK; i4++)
  for (i5=0; i5<KK; i5++)
  for (i6=0; i6<KK; i6++) {
    switch (i) {
      case 0: j=i6; break;
      case 1: j=i5; break;
      case 2: j=i4; break;
      case 3: j=i3; break;
      case 4: j=i2; break;
      case 5: j=i1; break;
      case 6: j=i0; break;
      case 7: j=0; if (i0+i1+i2+i3+i4+i5+i6>3) j=1; break;
      default: j=0; break;}
    ascrule[i0][i1][i2][i3][i4][i5][i6] = '0'+j;
    }
}

/* --------------------------- E V O L U T I O N --------------------------- */

- drawSmEvol:sender {int k;
    drawview=7;
    hxDrawSmall=YES;
    ascintohex();
    if ([reStore state]) savearry();
    sscanf([[sender selectedCell] title],"%d",&hxRepPlEv);
    [self display];
    if (hxRepPlEv>0) for (k=0; k<hxRepPlEv; k++) [self display];
    if ([reStore state]) renewarry();
    hxDrawSmall=NO;
    hxLastPlane=3;
return self;
}

- printSmEvol:sender {
    drawview=7;
    hxDrawSmall=YES;
    hxRepPlEv=0;
    [keyWindow smartPrintPSCode: self];
    hxDrawSmall=NO;
return self;
}

- drawSmMosaic:sender {
    hxMoTiles=[moTies selectedRow]+2;
    drawview=11;
    hxSmallMosaic=YES;
    ascintohex();
    savearry();
    [self display];
    renewarry();
    hxSmallMosaic=NO;
    hxLastPlane=4;
return self;
}

- printSmMosaic:sender {
    drawview=11;
    hxSmallMosaic=YES;
    savearry();
    [keyWindow smartPrintPSCode: self];
    renewarry();
    hxSmallMosaic=NO;
return self;
}

- drawTraj:sender {
    sscanf([[sender selectedCell] title],"%d",&hxTrajLen);
    phexDots=[probDots state];
    drawview=13;
    hxDrawTrajec=YES;
    ascintohex();
    savearry();
    [trajView display];
    renewarry();
    hxDrawTrajec=NO;
    hxLastTraj=1;
return self;
}

- printTraj:sender {
    drawview=13;
    hxDrawTrajec=YES;
    savearry();
    [keyWindow smartPrintPSCode: self];
    renewarry();
    hxDrawTrajec=NO;
return self;
}

- drawRtrn:sender {
    sscanf([[sender selectedCell] title],"%d",&hxRturnNum);
    drawview=13;
    hxDrawRturn=YES;
    ascintohex();
    savearry();
    [trajView display];
    renewarry();
    hxDrawRturn=NO;
    hxLastTraj=2;
return self;
}

- printRtrn:sender {
    drawview=13;
    hxDrawRturn=YES;
    savearry();
    [keyWindow smartPrintPSCode: self];
    renewarry();
    hxDrawRturn=NO;
return self;
}

- drawFreq:sender {
//    sscanf([[sender selectedCell] title],"%d",&hxRturnNum);
    drawview=13;
    hxDrawFreqs=YES;
    ascintohex();
    savearry();
    [trajView display];
    renewarry();
    hxDrawFreqs=NO;
    hxLastTraj=3;
return self;
}

- printFreq:sender {
    drawview=13;
    hxDrawFreqs=YES;
    savearry();
    [keyWindow smartPrintPSCode: self];
    renewarry();
    hxDrawFreqs=NO;
return self;
}

- slowFreq:sender {
//    sscanf([[sender selectedCell] title],"%d",&hxRturnNum);
    drawview=13;
    hxSlowFreqs=YES;
    ascintohex();
    savearry();
    [trajView display];
    renewarry();
    hxSlowFreqs=NO;
    hxLastTraj=4;
return self;
}

- printSlowFreq:sender {
    drawview=13;
    hxSlowFreqs=YES;
    savearry();
    [keyWindow smartPrintPSCode: self];
    renewarry();
    hxSlowFreqs=NO;
return self;
}

- drawFrequency:sender {
    hxSkipThree=[eachAndAll state];
    hxtotpmut[0]=[freqPerm0 selectedCol];
    hxtotpmut[1]=[freqPerm1 selectedCol];
    hxtotpmut[2]=[freqPerm2 selectedCol];
    hxtotpmut[3]=[freqPerm3 selectedCol];
    hxtotpmut[4]=[freqPerm4 selectedCol];
    hxtotpmut[5]=[freqPerm5 selectedCol];
    hxtotpmut[6]=[freqPerm6 selectedCol];
    hxtotpmut[7]=[freqPerm7 selectedCol];
    hxtotcolst[0]=[freqColor0 color];
    hxtotcolst[1]=[freqColor1 color];
    hxtotcolst[2]=[freqColor2 color];
    hxtotcolst[3]=[freqColor3 color];
    hxtotcolst[4]=[freqColor4 color];
    hxtotcolst[5]=[freqColor5 color];
    hxtotcolst[6]=[freqColor6 color];
    hxtotcolst[7]=[freqColor7 color];
    drawview=14;
    hxDrawQuency=YES;
    asctoqar();
    savearry();
    [freqView display];
    renewarry();
    hxDrawQuency=NO;
    hxLastTraj=5;
return self;
}

- printFrequency:sender {
    drawview=14;
    hxDrawQuency=YES;
    savearry();
    [keyWindow smartPrintPSCode: self];
    renewarry();
    hxDrawQuency=NO;
return self;
}

/* draw rectangle (i,j) in a mosaic of (u,v) */
/* whose individual rectangles measure (m,n) */
void hxManyrect(i,j,u,v,m,n) int i, j, u, v, m, n; {int k, l;
  ascintohex();
  for (k=0; k<m; k++) for (l=0; l<n; l++) videodot(i*m+k,j*n+l,arr1[k*n+l]);
  onehexgen(arr1,arr2,m,n);
}

/* m = high/hxMoTiles, n = wide/hxMoTiles */
void hxSmallgrid(m,n) int m, n; {int i; double em, en, mo;
  em=(double)m;
  en=(double)n;
  mo=(double)hxMoTiles;
	  PSsetgray(NX_BLACK); 
	  PSsetlinewidth(0.1);
  for (i=0; i<hxMoTiles+1; i++) {
  PSmoveto((double)(en*i),0.0); PSlineto((double)(en*i),mo*em);}
  for (i=0; i<hxMoTiles+1; i++) {
  PSmoveto(0.0,(double)(em*i)); PSlineto(mo*en,(double)(em*i));}
  PSstroke();
}

/* -----------------------  D E F I N E   R U L E S  ----------------------- */

/* - - - - - - - -   B E R N S T E I N   H I S T O G R A M S   - - - - - - */

- drawBern:sender {
    drawview=12;
    hxDrawBnstein=YES;
    [bernView display];
    hxDrawBnstein=NO;
    [window setDocEdited:YES];
return self;
}

- selfBern:sender {
    drawview=12;
    hxDrawBnstein=YES;
    [self display];
    hxDrawBnstein=NO;
    [window setDocEdited:YES];
return self;
}

- printRule:sender {
   drawview=12;
   hxDrawBnstein=YES;
   [keyWindow printPSCode:self];
   hxDrawBnstein=NO;
return self;
}

- enterRule:sender {id cell; char *c, ch[WNML]; int i, j;
char    tiempo[30], fecha[30];
struct  tm  *timeptr;
time_t  secsnow;

   time(&secsnow);
   timeptr = localtime(&secsnow);
   sprintf(tiempo,"%02d:%02d:%02d\n",
	timeptr->tm_hour, 
	timeptr->tm_min,
        timeptr->tm_sec);
   [tiTime setStringValue: tiempo];
   sprintf(fecha,"%d-%d-19%02d\n",
	((timeptr->tm_mon) + 1), 
	timeptr->tm_mday,
        timeptr->tm_year);
   [tiDat setStringValue: fecha];

    for (i=0; i<NG; i++) for (j=0; j<MB; j++) {
      cell=[theZMatrix cellAt: j: i];
      if ([cell isEnabled]) {
	c=[cell title];
        ascrule[c[0]-'0']
		[c[1]-'0']
		[c[2]-'0']
		[c[3]-'0']
		[c[4]-'0']
		[c[5]-'0']
		[0]         =[cell intValue]+'0';
	}
      cell=[theOMatrix cellAt: j: i];
      if ([cell isEnabled]) {
	c=[cell title];
        ascrule[c[0]-'0']
		[c[1]-'0']
		[c[2]-'0']
		[c[3]-'0']
		[c[4]-'0']
		[c[5]-'0']
		[1]         =[cell intValue]+'0';
	}
      }
      wrru(ch,ascrule);
      [ruleNumber setStringValue: ch];
      if (istot()) {
		[tRuleNumber setIntValue: totonu(0)];
		for (i=0; i<TS; i++) 
		  [[theToMatrix cellAt: 0: i] setIntValue: ttrule[i]-'0'];
		}
      else [tRuleNumber setStringValue: "enter totalistic #"];
      [HexlWindow setDocEdited: NO];
      [rulePanel setAllRules: self];
      [bernView selfBern: nil];
return self;
}

- resetRule:sender {int i, j; 
    for (i=0; i<NG; i++) for (j=0; j<MB; j++) {
	[[theOMatrix cellAt: j: i] setIntValue: NO];
	[[theZMatrix cellAt: j: i] setIntValue: NO];
	}
return self;
}

- copyRule:sender {id cell; char *c, ch[WNML]; int i, j;
    for (i=0; i<NG; i++) for (j=0; j<MB; j++) {
      cell=[theZMatrix cellAt: j: i];
      if ([cell isEnabled]) {
	c=[cell title];
        [cell setIntValue: ascrule[c[0]-'0']
				[c[1]-'0']
				[c[2]-'0']
				[c[3]-'0']
				[c[4]-'0']
				[c[5]-'0']
				[0]-'0'        ];
	}
      cell=[theOMatrix cellAt: j: i];
      if ([cell isEnabled]) {
	c=[cell title];
        [cell setIntValue: ascrule[c[0]-'0']
				[c[1]-'0']
				[c[2]-'0']
				[c[3]-'0']
				[c[4]-'0']
				[c[5]-'0']
				[1]-'0'        ];
	}
      }
      wrru(ch,ascrule);
      [ruleNumber setStringValue: ch];
      whxsscrrul(ch,ascrule);
      [xsRuleNumber setStringValue: ch];
      if (istot()) {
	 [tRuleNumber setIntValue: totonu(0)];
	 for (i=0; i<TS; i++) 
	   [[theToMatrix cellAt: 0: i] setIntValue: ttrule[i]-'0'];
	 }
      else [tRuleNumber setStringValue: "enter totalistic #"];
      [HexlWindow setDocEdited: NO];
      [bernView selfBern: nil];
return self;
}

- readRule:sender 
    {[rulePanel readWRule: sender]; [self copyRule: self]; return self;}

- readTRule:sender 
    {[rulePanel readTRule: sender]; [self copyRule: self]; return self;}

- enterTRule:sender {char ch[WNML];
char    tiempo[30], fecha[30];
struct  tm  *timeptr;
time_t  secsnow;

   time(&secsnow);
   timeptr = localtime(&secsnow);
   sprintf(tiempo,"%02d:%02d:%02d\n",
	timeptr->tm_hour, 
	timeptr->tm_min,
        timeptr->tm_sec);
   [tiTime setStringValue: tiempo];
   sprintf(fecha,"%d-%d-19%02d\n",
	((timeptr->tm_mon) + 1), 
	timeptr->tm_mday,
        timeptr->tm_year);
   [tiDat setStringValue: fecha];

     ttrule[[sender selectedCol]] = [[sender selectedCell] state]+'0';
     totoasc();
     wrru(ch,ascrule);
     [ruleNumber setStringValue: ch];
     [rulePanel setAllRules: self];
     [HexlWindow setDocEdited: NO];
     if (istot()) [tRuleNumber setIntValue: totonu(0)];
       else [tRuleNumber setStringValue: "enter totalistic #"];
return self;
}

- copyTRule:sender {char ch[WNML]; int i;
     wrru(ch,ascrule);
     [ruleNumber setStringValue: ch];
     if (istot()) {
		[tRuleNumber setIntValue: totonu(0)];
		for (i=0; i<TS; i++) 
		  [[theToMatrix cellAt: 0: i] setIntValue: ttrule[i]-'0'];
		}
       else [tRuleNumber setStringValue: "enter totalistic #"];
     [HexlWindow setDocEdited: NO];
     [bernView selfBern: nil];
return self;
}

/* -------------------------------- * * * * -------------------------------- */

- initFrame:(const NXRect *)frameRect
/*
 * Initializes the new HEXLView object.  First, an initFrame: message is sent 
 * to super to initialize HEXLView as a View.  Next, the HEXLView sets its own 
 * state -- that it is opaque and that the origin of its coordinate system lies 
 * in the center of its area.  It then creates and initializes its associated 
 * objects, a Storage object, an NXCursor, and a Cell.  Finally, it loads into 
 * the Window Server some PostScript procedures that it will use in drawing 
 * itself. 
 */
{
    NXPoint	spot;
	
    [super initFrame:frameRect];
/*  [self translate:floor(frame.size.width/2) :floor(frame.size.height/2)]; */
    [self setOpaque:YES];
    crossCursor=[NXCursor newFromImage:[NXImage newFromSection:"cross.tiff"]];
    spot.x = spot.y = 7.0;
    [crossCursor setHotSpot:&spot];

  PSsetgray(NX_BLACK);
  PSselectfont("Times-Roman",16.0);

  [self setFlipped: YES];
  drawview=7;			      /* prefer origin in upper left corner */
	return self;
}

- setDelegate:anObject
/*
 * Sets the HEXLView's delegate instance variable to the supplied object.
 */
{
	delegate = anObject;
	return self;
}

- drawSelf:(const NXRect *)rects :(int)rectCount
/*
 * Draws the HEXLView's background and axes.  If there are any points,
 * these are drawn too.
 *
 * (Note:  For simplicity, although HEXLView only repaints the background 
 * of the update region, it redraws the entire axes.  For better performance, 
 * only those parts of the axes that fall within the update region should 
 * be redrawn.)   
 */
{
	int		 i, j, k, l, n, jj;
        double           p, q, r, s, t, mo;
	NXRect		 plotRect;

	if (rects == NULL) return self;

	/* paint visible area white then draw axes */
	PSsetgray(NX_WHITE);
	if (drawHXBack) NXRectFill(&rects[0]);
	PSsetgray(NX_DKGRAY);

	if (hxDrawBnstein==YES) {
	  for (i=0; i<TS; i++) bp[i]=0.0;
	  for (i=0; i<NG; i++) for (j=0; j<MB; j++) {
		if ([[theZMatrix cellAt: j: i] state]) bp[hxgtb[i]]+=1.0;
		if ([[theOMatrix cellAt: j: i] state]) bp[hxgtb[i]+1]+=1.0;
		}
	  hxgfram(50,3);
	  hxgdiag(50,3);
	  bhxgraf(TS-1,50);
	  }

	if (hxDrawTrajec==YES) {
	  hxgfram(75,3);
	  hxgdiag(75,3);
	  bhxgraf(TS-1,75);
	  hxstatecount(arr1,64,64);
	  for (j=0; j<KK; j++) llf[j]=lf[j];
	  onehexgen(arr1,arr2,64,64);
	  if (phexDots) hxbdot((75*llf[1])/4096,(75*lf[1])/4096,3);
	    else hxbline(0.034*((double)llf[1]),0.034*((double)lf[1]),0);
	  for (i=0; i<hxTrajLen; i++) {
	    for (j=0; j<KK; j++) llf[j]=lf[j];
	    onehexgen(arr1,arr2,64,64);
	    if (phexDots) hxbdot((75*llf[1])/4096,(75*lf[1])/4096,3);
	  else hxbline(0.034*((double)llf[1]),0.034*((double)lf[1]),1);
	    }
	  PSstroke();
	  }

	if (hxDrawRturn==YES) {
	  hxgfram(75,3);
	  hxgdiag(75,3);
	  bhxgraf(TS-1,75);
	  for (i=0; i<75; i++) {
	    p=((double)i)/75.0; q=1.0-p;
	    ranblin(q,p);
	    for(j=0; j<hxRturnNum; j++) onehexgen(arr1,arr2,64,64);
	    hxbdot(i,(75*lf[1])/4096,3);
	    }
	  }

	if (hxDrawFreqs==YES) {
	  hxgfram(75,3);
	  for (i=0; i<75; i++) {
	    onehexgen(arr1,arr2,64,64);
	    for (j=0, k=0; j<TS; j++)
    if (hxtus[j]>57) for (l=0; l<hxtus[j]; l+=58) hxbdot(i,k++,j%2);
	    hxbdot(i,(75*lf[1])/4096,3);
	    }
// printf("teecees:");
// for (i=0; i<DS; i++) printf("%5d ",tc[i]);
// printf("\n");
// printf("tussies: ");
// for (i=0; i<TS; i++) printf("%5d ",hxtus[i]);
// printf("\n");
	  }

	if (hxSlowFreqs==YES) {
	  hxgfram(75,3);
	  for (i=0; i<75; i+=3) {
	    onehexgen(arr1,arr2,64,64);
	    for (j=0, k=0; j<TS; j++)
		if (hxtus[j]>57) for (l=0; l<hxtus[j]; l+=58) {
		  hxbdot(i,k,j%3);
		  hxbdot(i+1,k,j%3);
		  hxbdot(i+2,k,j%3);
		  k++;
		  }
	    hxbdot(i,(75*lf[1])/4096,3);
	    hxbdot(i+1,(75*lf[1])/4096,3);
	    hxbdot(i+2,(75*lf[1])/4096,3);
	    }
	  }

	if (hxDrawQuency==YES) {
	  hxgfram(125,3);
	  for (i=0; i<125; i++) {
	    onehexgen(arr1,arr2,64,64);
	    for (j=0, k=0; j<TS; j++) {
		jj=hxtotpmut[j];
		NXSetColor(hxtotcolst[jj]);
		if (hxtus[jj]>32) 
		    for (l=0; l<hxtus[jj]; l+=33) hxbdot(i,k++,5);
		}
	    hxbdot(i,(125*lf[1])/4096,3);
	    if (hxSkipThree) {
	      onehexgen(arr1,arr2,64,64);
	      onehexgen(arr1,arr2,64,64);
	      }
	    }
// printf("teecees:"); for (i=0; i<DS; i++) printf("%5d ",tc[i]); printf("\n");
printf("tussies: "); for (i=0; i<TS; i++) printf("%5d ",hxtus[i]); printf("\n");
	  }

        if (hxCheckBoard==YES) {
   	  [self getBounds: &plotRect];
	  mo=(double)hxMoTiles;
	  j=(int)(plotRect.size.width/mo);
	  i=(int)(plotRect.size.height/mo);
	  zerline();
          for (k=0; k<i/2; k++) {
	    for (l=0; l<j/2; l++) arr1[k*j+l]=0;
            for (l=j/2; l<j; l++) arr1[k*j+l]=1;
            }
          for (k=i/2; k<i; k++) {
	    for (l=0; l<j/2; l++) arr1[k*j+l]=1;
            for (l=j/2; l<j; l++) arr1[k*j+l]=0;
            }
	  }

        if (hxSmallSmall==YES) {
	  mo=(double)hxMoTiles;
   	  [self getBounds: &plotRect];
	  j=(int)(plotRect.size.width/mo);
	  i=(int)(plotRect.size.height/mo);
	  zerline();
          for (k=i/3; k<2*(i/3); k++) for (l=j/3; l<2*(j/3); l++) 
		arr1[k*j+l]=random()%KK;
	  }

        if (hxDrawSmall==YES) {
	  [window setTitle: "evolution"];
   	  [self getBounds: &plotRect];
	  j=(int)(plotRect.size.width/4.0);           /* 4.0 for drawview=7 */
	  i=(int)(plotRect.size.height/4.0);
	  screen(arr1,i,j);
	  if(hxRepPlEv>0) onehexgen(arr1,arr2,i,j);
	  }

        if (hxSmallMosaic==YES) {
	  mo=(double)hxMoTiles;
	  [window setTitle: "mosaic"];
   	  [self getBounds:&plotRect];
	  j=(int)(plotRect.size.width/mo);
	  i=(int)(plotRect.size.height/mo);
	  for (k=0; k<hxMoTiles; k++) for (l=0; l<hxMoTiles; l++) {
	    hxManyrect(k,l,hxMoTiles,hxMoTiles,i,j);
	    [window flushWindow];
	    }
	  hxSmallgrid(i,j);
	  }

        if (hxDrawPlane==YES) {
   switch (dn) {
     case 2: 
	[window setTitle: "von Neumann (2,1) Plane Evolution"];
	hxDrawsquare();
	break;
     case 3: 
	[window setTitle: "3-D Solid Tetrahedron Evolution"];
	hxDrawcube();
	hxDrawgrid();
	break;
     default: [window setTitle: "HEXL22 Plane Evolution"];break;}
	}

        if (hxDrawMosaic==YES) {
   switch (dn) {
     case 2: 
	[window setTitle: "von Neumann (2,1) hxManysquare Plane Evolution"];
	for (i=0; i<4; i++) for (j=0; j<4; j++) {
	  hxManysquare(i,j);
	  onegen(AL);
	  [window flushWindow];
	  }
	hxDrawgrid();
	break;
     default: [window setTitle: "HEXL22 Plane Evolution"];break;}
	}

	return self;
}

- sizeTo:(NXCoord)width :(NXCoord)height
/*
 * Ensures that whenever the HEXLView is resized, the origin of its 
 * coordinate system is repositioned to the center of its area.
 */
{
//	  [self setFlipped: YES];
	[super sizeTo: width: height];
	if (drawview==4||drawview==6||drawview==7||drawview>=10) 
		{[self setDrawOrigin:0 :0]; return self;}
	else
		[self setDrawOrigin: -floor(width/2) : -floor(height/2)]; 
	return self;
}

- mouseDown:(NXEvent *) theEvent
/*
 * Responds to a message the system sends whenever the user presses the mouse
 * button when the cursor is over the HEXLView.  The HEXLView changes the 
 * cursor to a cross-hairs image and then starts asking for mouse-dragged or 
 * mouse-up events.  As it receives mouse-dragged events, the HEXLView updates 
 * the readOut text Cell with the cursor's coordinates.  If the user releases 
 * the mouse button while the cursor is over the HEXLView, the HEXLView 
 * registers the point and then sends a message to its delegate notifying it 
 * of the new point.
 */
{
	int	looping = YES, oldMask;
	NXPoint	aPoint;
	NXRect	plotRect;

      /*   double  x, y,  eks, wye; */
	 double ex, wy;
	 int k;

	[crossCursor set];
	oldMask = [window addToEventMask:NX_MOUSEDRAGGEDMASK];
 				[window setDocEdited:YES];
		                aPoint = theEvent->location;
				aPoint.y = aPoint.y;
		                [self convertPoint:&aPoint fromView:nil];
                                ex=aPoint.x;
                                wy=aPoint.y;
				k=nearest(ex,wy);

//       if (theEvent->data.mouse.click==3) {} else 
//       if (theEvent->data.mouse.click==2) {} else 
	{
	[self lockFocus];
//         PSsetinstance(YES);
	do {
// 		PSnewinstance();
		aPoint = theEvent->location;
		[self convertPoint:&aPoint fromView:nil];
		aPoint.y= (plotRect.size.height-1)-aPoint.y;
                ex=aPoint.x;
                wy=aPoint.y;
				PSsetgray(NX_BLACK);
		[window flushWindow];
		if (theEvent->type == NX_MOUSEUP) {
	/* on mouse-up, register point, inform delegate, and clean up state */
			[window flushWindow];
			looping = NO;
		}	
   	} while (looping && (theEvent = 
		[NXApp getNextEvent:NX_MOUSEUPMASK|NX_MOUSEDRAGGEDMASK]));
	[self unlockFocus];

	}

	[window setEventMask:oldMask];
	[NXArrow set];
	return self;
}

@end

/* end HEXLView.m */
