
/* 
 * OverAll.m -- Implementation file for the OverAll class 
 *
 * You may freely copy, distribute, and reuse the code in this example.
 * NeXT disclaims any warranty of any kind, expressed or implied, as to 
 * its fitness for any particular use, nor even that it has a use.
 *
 */

#import "OverAll.h"
#import "nxlcau33.h"
#import "rec.h"
#import <math.h> 
#import <stdlib.h>
#import <string.h>
#import <time.h>

int  co, dn;			/* connectivity, dimension */

char dfclr[KK][12] = {"1.0,0.0,0.0","0.0,1.0,0.0","0.0,0.0,1.0"};

NXColor colst[KK];
NXDefaultsVector defaults={
			   {"well0", "1.0,0.0,0.0"},
			   {"well1", "0.0,1.0,0.0"},
			   {"well2", "0.0,0.0,1.0"},
			   {NULL,NULL}
			  };

char colorwells[KK][6] = {"well0","well1","well2"};

extern Inst   rprg[384];			/* rec program array */
extern char   cstr[CLEN];			/* console rec program */
extern char   cprg[CLEN];			/* edit stored program */
extern Inst   xprg[10][XLEN];

extern struct fptbl dtbl[];
extern char idef[][30];

void asctobin();
void rule33();
void ttzero();
void gfzero();

@implementation OverAll

/* ---------------  A P P   I N I T I A L I Z A T I O N   ----------------- */

- appWillInit:sender {printf("appWillInit\n"); return self;}

- appDidInit:sender {
char   c[20], ch[WNML], *df;
int    i, o;
double red, green, blue;
struct tm  *timeptr;
time_t  secsnow;
  printf("appDidInit = nxlcau33\n");
  co=1;		/* normal connectivity */
  dn=1;		/* one-dimensional     */
  rule33();     /* printf("rule33\n"); */
  asctobin();
  ttzero();
//  stzero();
  gfzero();
  [rulePanel setAllRules: self];
  asctobin();
  golta();
  totta();

   time(&secsnow);
   timeptr = localtime(&secsnow);
   sprintf(c,"%d-%d-19%02d\n",
	((timeptr->tm_mon) + 1), 
	timeptr->tm_mday,
        timeptr->tm_year);
   [tiDat setStringValue: c];

   NXRegisterDefaults("NXLCAU33Defaults", defaults);
   NXUpdateDefaults();
    for(i=0; i <KK; i++) {
     df=NXGetDefaultValue("NXLCAU33Defaults",colorwells[i]);
     if (df==NULL) {
	NXWriteDefault("NXLCAU33Defaults",colorwells[i],dfclr[i]);
	df=&dfclr[i];
	}
     sscanf(df,"%6lf,%6lf,%6lf",&red,&green,&blue); /* don't use %F */
     colst[i]=NXConvertRGBToColor(red,green,blue);
     printf("%s: %s\n",colorwells[i],df);
     }

   [[NXColorPanel sharedInstance: YES] setContinuous: YES];
   [self updateColors];

   /*  initialize r_vst with selected standard procedures */
   rec_c(idef[0], xprg[0], XLEN, dtbl);
   r_vst['a']=&xprg[0];

   [browser getTitleFromPreviousColumn: YES];
   [browser setDoubleAction: @selector(copyAndRunREC:)];
   [browser setPath: "/demonstrations"];

return self;
}
 
- appWillTerminate:sender {printf("appWillTerminate\n"); return self;}

/* ----------------------------- G E N E R A L ----------------------------- */

- saveColors:sender {char c[32]; int i;
    colst[0]=[inCol0 color];
    colst[1]=[inCol1 color];
    colst[2]=[inCol2 color];
  for (i=0; i<KK; i++) {
        sprintf(c,"%6.3f, %6.3f, %6.3f",
		NXRedComponent(colst[i]),
		NXGreenComponent(colst[i]),
		NXBlueComponent(colst[i]));
	NXWriteDefault("NXLCAU33Defaults",colorwells[i],c);
	};
return self;
}

- revertColors:sender
{char *df;  int i;  double red, green, blue; 
  for (i=0; i<KK; i++) {
    df=NXGetDefaultValue("NXLCAU33Defaults",colorwells[i]);
       if (df==NULL) {printf("null df = %s\n",df); return self;}
       sscanf(df,"%6lf,%6lf,%6lf",&red,&green,&blue); 
    colst[i]=NXConvertRGBToColor(red,green,blue);
    printf("%s: %s\n",colorwells[i],df);
    }
  [self updateColors];
return self;
}

- updateColors {
	[inCol0 setColor: colst[0]];
	[inCol1 setColor: colst[1]];
	[inCol2 setColor: colst[2]];
return self;
}

- windowDidBecomeKey:sender {
	[menuCopy setTarget: self];
	[menuCopy setAction: @selector(copyWells:)];
	[menuCopy setEnabled: YES];
return self;
}

- windowDidResignKey:sender {[menuCopy setEnabled: NO]; return self;}

- copyWells:sender {
NXRect zbounds;
id     pBoard;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [theWellBox getBounds: &zbounds];
  [theWellBox writePSCodeInside: &zbounds to: pBoard]; 
return self;
}

/* - - - - - - - -  R E C   L I N E   C O N F I G U R A T I O N  - - - - - */
 
- useRECLine:sender {
  [RECPanel makeKeyAndOrderFront: sender];
  [RECValue setStringValue: "*"];
 return self;
}

- runRECLine:sender; {
  [RECValue setStringValue: " "];
  if (rec([sender stringValue])) {
     [RECValue setStringValue: "T"];
     [rulePanel setAllRules: self];
     }
  else
    [RECValue setStringValue: "F"];
  return self;
}

rec(z) char *z; {if (rec_c(z,rprg,PLEN,dtbl)==1) rec_x(rprg);}

/* << S E T - D I M E N S I O N - A N D - L O A D - A C C O R D I N G L Y >> */

- loadOverAll:sender {
   if (ardnPanel==nil) {
      if (![NXApp loadNibSection: "overall.nib" owner: self withNames: NO]) 
	{return nil;}
    }
  if ([sender selectedCell]==menuArdn) 
	[ardnPanel makeKeyAndOrderFront: sender];
  if ([sender selectedCell]==menuColor) 
	[colorPanel makeKeyAndOrderFront: sender];
  if ([sender selectedCell]==menuPair)  
	[pairPanel makeKeyAndOrderFront: sender];
  if ([sender selectedCell]==menuSset)  
	[ssetPanel makeKeyAndOrderFront: sender];
  return self;
}

- dimenChoice:sender {
   if (dimenPrefs==nil) {
      if (![NXApp loadNibSection: "overall.nib" owner: self withNames: NO]) 
	{return nil;}
      [dimenMenu setEmptySelectionEnabled: YES];
      [dimenMenu selectCellAt: -1: 0];
    }
  [dimenPrefs makeKeyAndOrderFront: sender];
  return self;
}

- setDimension:sender {dn = [sender selectedRow]+1; return self;}

- dbWindow:sender {
printf("ruinWindow is about to load %d\n",dn);
   switch (dn) {
     case 1:  [rijnView loadRijn: sender];
//	      [db2Window performClose: sender];
//	      [db3Window performClose: sender];
	      break;
     case 2:  [ruinView showRuin: sender];
//	      [db2Window performClose: sender];
//	      [db3Window performClose: sender];
	      break;
     default: break;}
printf("ruinWindow is loaded\n");
  return self;
}

@end

/* end OverAll.m */
