
/* 
 * ProbView.m -- Implementation file for the ProbView class 
 *
 * You may freely copy, distribute, and reuse the code in this example.
 * NeXT disclaims any warranty of any kind, expressed or implied, as to 
 * its fitness for any particular use, nor even that it has a use.
 *
 */

#import "ProbView.h"
#import "drawingFuncs.h"
#import "nxlcau33.h"
#import "nxprob33.h"
#import <math.h> 
#import <dpsclient/psops.h>
#import <dpsclient/wraps.h>
#import <stdlib.h>
#import <string.h>

int    probBack=YES;
int    drawBasin=NO;
int    drawEntpy=NO;
int    drawMonc=NO;
int    drawEProb=NO;
int    drawMeanF=NO;
int    drawPlaid=NO;
int    drawXBlock=NO;
int    drawNBlk=NO;
int    drawStat=NO;

extern int    drawview;

int    srule[2][27];
int    cu=12;

double cubic[25][25][25];

extern NXColor colst[KK];

extern double blpr[KK];			/* probability from blocks	*/

extern char   arr1[], arr2[];

int    plaidline[2];			/* center for plaid radius contours */
extern double stt, sta[5][K3+1];


int      ix3(int i, int j, int k);
void     shortevolve(int m, int n);
void     tcolordot(double *p, double *c);
double   tdist(int i, int j, int n);
void     wru(char *c);

/* from conth.c */
void   conth(double z[25][25][25], double c, int i, int j, int k, int vv);
void   cudem(double f[25][25][25], int l, int m, int n, int v);

/* from lcau.c */
void asctobin();
void evolve();
void onegen(int n);
void pgrid();
void pprob();
void ranblin(double p, double q, double r);
void ranlin();
void renewarry();
void savearry();
void xblnk();

#define MAXNUMLENGTH 50
#define DEFAULTRADIUS 1.0

@implementation ProbView

/* - - - - - - - - - - - - -   P R O B V I E W   - - - - - - - - - - - - - */

- loadProb:sender {id prbMen;
  if (sender==rulePanel) {
       if (trivWindow!=nil && [trivWindow isVisible]) [self nxasfreq: sender];}
  else if (probWindow==nil) {
       if (![NXApp loadNibSection: "prob.nib" owner: self withNames: NO]) 
	 {return nil;}
       prbMen = [[Menu alloc] initTitle: "Probability"];

    [[prbMen 
      addItem: "Basins" 
      action: @selector(makeKeyAndOrderFront:) keyEquivalent: '\0']
        setTarget: basinWindow];
    [[prbMen 
      addItem: "Entropy" 
      action: @selector(makeKeyAndOrderFront:) keyEquivalent: '\0']
        setTarget: entropyWindow];
    [[prbMen 
      addItem: "Mean Field" 
      action: @selector(makeKeyAndOrderFront:) keyEquivalent: '\0']
        setTarget: probWindow];
    [[prbMen 
      addItem: "Extended M-F" 
      action: @selector(makeKeyAndOrderFront:) keyEquivalent: '\0']
        setTarget: eProbWindow];
    [[prbMen 
      addItem: "N-Block" 
      action: @selector(makeKeyAndOrderFront:) keyEquivalent: '\0']
        setTarget: blockWindow];

    [[sender window] setSubmenu: prbMen forItem: [sender selectedCell]];
    [[sender selectedCell] performClick: sender];
    }
  return self;
  }

- printAProbWindow:sender {
    if (keyWindow==basinWindow)   [self printBasin:   sender];
    if (keyWindow==blockWindow)   [self printNBlock:  sender];
    if (keyWindow==entropyWindow) [self printEntropy: sender];
    if (keyWindow==eProbWindow)   [self printEProb:   sender];
    if (keyWindow==plaidWindow)   [self printPlaid:   sender];
    if (keyWindow==probWindow)    [self printProb:    sender];
    if (keyWindow==statWindow)    [self printStat:    sender];
return self;
}

- windowDidBecomeKey:sender {
	keyWindow=sender;
	[menuCopy setTarget: self];
	[menuCopy setAction: @selector(copyAProbView:)];
	[menuCopy setEnabled: YES];
	[menuPrint setTarget: self];
	[menuPrint setAction: @selector(printAProbWindow:)];
	[menuPrint setEnabled: YES];
return self;
}

- windowDidResignKey:sender {
	[menuCopy setEnabled: NO];
	[menuPrint setEnabled: NO];
return self;
}

- copyBasin:sender {
NXRect zbounds;
id     pBoard;
  drawview=9;
  probBack=NO;
  drawBasin=YES;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [theBasinView getBounds: &zbounds];
  [theBasinView writePSCodeInside: &zbounds to: pBoard]; 
  drawBasin=NO;
  probBack=YES;
return self;
}

- copyEnpy:sender {
NXRect zbounds;
id     pBoard;
  drawview=9;
  probBack=NO;
  drawEntpy=YES;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [theEntropyView getBounds: &zbounds];
  [theEntropyView writePSCodeInside: &zbounds to: pBoard]; 
  drawEntpy=NO;
  probBack=YES;
return self;
}

- copyProb:sender {
NXRect zbounds;
id     pBoard;
  drawview=9;
  probBack=NO;
  drawMeanF=YES;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [theProbView getBounds: &zbounds];
  [theProbView writePSCodeInside: &zbounds to: pBoard]; 
  drawMeanF=NO;
  probBack=YES;
return self;
}

- copyEProb:sender {
NXRect zbounds;
id     pBoard;
  drawview=8;
  probBack=NO;
  drawEProb=YES;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [theEProbView getBounds: &zbounds];
  [theEProbView writePSCodeInside: &zbounds to: pBoard]; 
  drawEProb=NO;
  probBack=YES;
return self;
}

- copyNBlock:sender {
NXRect zbounds;
id     pBoard;
  drawview=4;
  drawNBlk=YES;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [theBlockView getBounds: &zbounds];
  [theBlockView writePSCodeInside: &zbounds to: pBoard]; 
  drawNBlk=NO;
return self;
}

- copyAProbView:sender {
    if (keyWindow==basinWindow)   [self copyBasin:  sender];
    if (keyWindow==blockWindow)   [self copyNBlock: sender];
    if (keyWindow==entropyWindow) [self copyEnpy:   sender];
    if (keyWindow==probWindow)    [self copyProb:   sender];
    if (keyWindow==eProbWindow)   [self copyEProb:  sender];
    if (keyWindow==plaidWindow)   [self copyPlaid:  sender];
return self;
}

- showPlaid:sender {
  drawview=5;
  [self setFlipped: NO];
  drawPlaid=YES;
  [self display];
  drawPlaid=NO;
  [self setFlipped: YES];
	needsClearing = YES;
return self;
}

- printPlaid:sender {
  drawview=5;
  [self setFlipped: NO];
  drawPlaid=YES;
    [keyWindow smartPrintPSCode: self];
  drawPlaid=NO;
  [self setFlipped: YES];
	needsClearing = YES;

return self;
}

double tdist(i,j,n) int i, j, n; {int a[10], b[10], l, m; double kay, s, t, u;
  if (n>10) return 0.0; 
  kay=1.0/((double)KK);
  for (l=0; l<n; l++) a[l]=b[l]=0;
  for (m=i, l=0; l<n; m/=KK, l++) a[n-l-1]=m%KK;
  for (m=j, l=0; l<n; m/=KK, l++) b[n-l-1]=m%KK;
  for (s=0.0, t=1.0, l=0; l<n; t*=kay, l++) {u=a[l]-b[l]; s+=t*((u<0.0)?-u:u);}
  return s*kay;
}


/* -------------------------------- L C A U -------------------------------- */

/* -------------------------------- R I J N -------------------------------- */

/* -------------------------------- P R O B -------------------------------- */

- showBasin:sender {
    drawBasin=YES;
    [self display];
    drawBasin=NO;
	needsClearing = YES;
return self;
}

- printBasin:sender {
    drawBasin=YES;
    [keyWindow smartPrintPSCode: self];
    drawBasin=NO;
	needsClearing = YES;
return self;
}

- showProb:sender {
    drawview=9;
    drawMeanF=YES;
    [self display];
    drawMeanF=NO;
	needsClearing = YES;
return self;
}

- printProb:sender {
   drawview=9;
   drawMeanF=YES;
   [keyWindow smartPrintPSCode: self];
   drawMeanF=NO;
return self;
}

- showEntropy:sender {
    drawview=9;
    drawEntpy=YES;
    [self display];
    drawEntpy=NO;
	needsClearing = YES;
return self;
}

- printEntropy:sender {
   drawview=9;
   drawEntpy=YES;
   [keyWindow smartPrintPSCode: self];
   drawEntpy=NO;
return self;
}

- showEProb:sender {
    drawview=8;
    drawEProb=YES;
    [self display];
    drawEProb=NO;
return self;
}

- printEProb:sender {
   drawview=8;
   drawEProb=YES;
   [keyWindow smartPrintPSCode: self];
   drawEProb=NO;
return self;
}

- showNBlock:sender {
    drawview=4;
    drawNBlk=YES;
    [self display];
    drawNBlk=NO;
	needsClearing = YES;
return self;
}

- printNBlock:sender {
   drawview=4;
   drawNBlk=YES;
   [keyWindow smartPrintPSCode: self];
   drawNBlk=NO;
return self;
}

- showXBlock:sender {
    drawview=4;
    drawXBlock=YES;
    [self display];
    drawXBlock=NO;
	needsClearing = YES;
return self;
}

- printXBlock:sender {
   drawview=4;
   drawXBlock=YES;
   [keyWindow smartPrintPSCode: self];
   drawXBlock=NO;
return self;
}

- showStat:sender {
    drawview=5;
    drawStat=YES;
    [self display];
    drawStat=NO;
	needsClearing = YES;
return self;
}

- printStat:sender {
   drawview=5;
   drawStat=YES;
   [keyWindow smartPrintPSCode: self];
   drawStat=NO;
return self;
}

- nxmoncar: (int) l {
int i, j, k, n, b[KK], bb[KK][KK];
double bf[KK], bbf[KK][KK];
double a[KK], aa[KK], d[KK], el, en, t, tt, s;

// nblclr();
n=[[statLines selectedCell] tag];
asctobin();
el=1.0/((double)AL);
en=1.0/((double)n);
for (i=0; i<KK; i++) {a[i]=0.0; aa[i]=0.0;}
for (k=0; k<n; k++) {
  onegen(AL);
  for (i=0; i<KK; i++) b[i]=0;
  for (i=0; i<AL; i++) b[arr1[i]]++;
  for (i=0; i<KK; i++) bf[i]=((double)(b[i]))*el;
  for (i=0; i<KK; i++) {a[i]+=bf[i]; aa[i]+=bf[i]*bf[i];}
  for (i=0; i<KK; i++) for (j=0; j<KK; j++) bb[i][j]=0;
  for (i=1; i<AL; i++) bb[arr1[i-1]][arr1[i]]++;
  bb[arr1[AL-1]][arr1[0]]++; 
  for (i=0; i<KK; i++) for (j=0; j<KK; j++) 
    bbf[i][j]=((double)(bb[i][j]))*el;
//  videodot(199-(int)(100.0*bbf[1][1]),BORG+(int)(100.0*bf[1]),l);
  }
for (i=0; i<KK; i++) {a[i]*=en; aa[i]*=en;}
s=1.0/log((double)KK);
t=0.0; tt=0.0;
for (i=0; i<KK; i++) t+=bf[i]*log(bf[i]);
for (i=0; i<KK; i++) d[i]=sqrt(aa[i]-a[i]*a[i]);
for (i=0; i<KK; i++) for (j=0; j<KK; j++) tt+=bbf[i][j]*log(bbf[i][j]);
  [statLamb setDoubleValue: 1.0-bf[0]];
  [statEntro setDoubleValue: -t*s];
  [statEnpai setDoubleValue: -0.5*tt*s];
 printf("average      = %5.3f, %5.3f, %5.3f\n",a[0],a[1],a[2]);
 printf("deviation    = %5.3f, %5.3f, %5.3f\n",d[0],d[1],d[2]);
	[statSDevi setDoubleValue: (d[0]+d[1]+d[2])/3.0];
  for (i=0; i<KK; i++) [[statState cellAt: 0: i] setDoubleValue: bf[i]];
  for (i=0; i<KK; i++) for (j=0; j<KK; j++) {
	[[statPair cellAt: i: j] setDoubleValue: bbf[i][j]];
	[[statProd cellAt: i: j] setDoubleValue: bf[i]*bf[j]];
	}
return self;
}


/* - - - - - - - - - - -   P r o b V i e w   - - - - - - - - - - - - - - - - */


- initFrame:(const NXRect *)frameRect
/*
 * Initializes the new ProbView object.  First, an initFrame: message is sent 
 * to super to initialize ProbView as a View.  Next, the ProbView sets its own 
 * state -- that it is opaque and that the origin of its coordinate system lies 
 * in the center of its area.  It then creates and initializes its associated 
 * objects, a Storage object, an NXCursor, and a Cell.  Finally, it loads into 
 * the Window Server some PostScript procedures that it will use in drawing 
 * itself. 
 */
{
	NXPoint	spot;
	
	[super initFrame:frameRect];
/*	[self translate:floor(frame.size.width/2) :floor(frame.size.height/2)]; */
	[self setOpaque:YES];
	[self setRadius:DEFAULTRADIUS];
	points = [[Storage alloc] initCount:0 elementSize:sizeof(NXPoint) description:"{ff}"];
	crossCursor = [NXCursor newFromImage:[NXImage newFromSection:"cross.tiff"]];
	spot.x = spot.y = 7.0;
	[crossCursor setHotSpot:&spot];
	/* 
	 * Normally, the next two message expressions would be combined, as:
	 * 		readOut = [[Cell alloc] initTextCell:""];
	 * but are here separated for clarity.
	 */
	readOut = [Cell alloc];
	[readOut initTextCell:"pointies"];
	[readOut setBezeled:YES];
	loadPSProcedures();


  PSsetgray(NX_BLACK);
  PSselectfont("Times-Roman",16.0);

  [self setFlipped: YES];

	return self;

}


- setDelegate:anObject
/*
 * Sets the ProbView's delegate instance variable to the supplied object.
 */
{
	delegate = anObject;
	return self;
}


- drawSelf:(const NXRect *)rects :(int)rectCount
/*
 * Draws the ProbView's background and axes.  If there are any points,
 * these are drawn too.
 *
 * (Note:  For simplicity, although ProbView only repaints the background 
 * of the update region, it redraws the entire axes.  For better performance, 
 * only those parts of the axes that fall within the update region should 
 * be redrawn.)   
 */
{
        char             c[WNML], ct[30];
	unsigned int	 i, j, k, l, n;
	int		 ii, jj, nn, bas;
        double           en, p, q, r, s, t, dp, pee, cue;
	double		 aa[KK], bb[KK], cc[KK], cq[KK];
	double		 bern(), dbern(), tbern(), fbern();
	NXPoint		 *aPoint;
	NXRect		 plotRect;

  struct tm  *timeptr;
  time_t  secsnow;

int     i0, i1, ss;
	
	if (rects == NULL) return self;

	/* 
	 * If we're printing, we need to load drawing procedures to the 
	 * printing context. This simple test is OK for a View that prints 
	 * a single page. If there were multiple pages, the procedures would 
	 * be loaded multiple times. Although this is inefficient, it causes 
	 * no error. Views that print multiple pages should load procedures 
	 * such as these in an endPrologue method (see the View spec sheet). 
	 */
	if (NXDrawingStatus != NX_DRAWING) 
		loadPSProcedures();

	/* paint visible area white then draw axes */
	PSsetgray(NX_WHITE);
	if (probBack) NXRectFill(&rects[0]);

//	PSsetgray(NX_DKGRAY);
// drawAxes(bounds.origin.x, bounds.origin.y, bounds.size.width, bounds.size.height);




	if (drawPlaid==YES) {		/* one cycle of evolution */
	  n=[[gridSize selectedCell] tag];
	  if (strcmp([[aSwitch selectedCell] title],"distance")==0) {
	    [ruleNumber setStringValue: "distance matrix"];
	    for (i=0; i<243; i++) for (j=0; j<243; j++) {
	      r=tdist(i,j,5);
	      PSsetcmykcolor(0.1*r,0.8*r,1.0-r,0.55*r);
	      PSrectfill((float)(j),(float)(i),1.0,1.0);
	      }
	    PSstroke();
	    }
	  if (strcmp([[aSwitch selectedCell] title],"circles")==0) {
	    needsClearing = YES;
	    [points empty];
	    sprintf(c,"center: %3d, %3d\n",plaidline[0],plaidline[1]);
	    [ruleNumber setStringValue: c];
	    for (i=0; i<243; i++) for (j=0; j<243; j++) {
	      p=tdist(i,plaidline[1],5);
	      q=tdist(j,plaidline[0],5);
	      s=(p+q);
	      PSsetcmykcolor(s,0.05*p,1.0-s,0.05*q);
	      PSrectfill((float)(j),(float)(i),1.0,1.0);
	      }
	    PSstroke();
	    PSsetgray(NX_BLACK); 
	    PSrectfill((float)(plaidline[0]),(float)(plaidline[1]),2.0,2.0);
	    PSstroke();
	    }
	  if (strcmp([[aSwitch selectedCell] title],"plaid")==0) {
	    wru(c);
	    [ruleNumber setStringValue: c];
	    savearry();
	    asctobin();
	    for (j=0; j<128; j++) {
	      ranlin();
//	      for (i=0; i<64; i++) {onegen(AL); pprob();}
	      }
	    renewarry();
	  }
	if (n>0) {
	  PSsetgray(NX_BLACK); 
	  PSsetlinewidth(0.0);
	  for (j=1, dp=243.0, i=0; i<n; i++) {j*=3; dp*=0.333;}
	  for (i=0, p=1.0; i<j+1; i++, p+=dp) {
	    PSmoveto(0.1,p);
	    PSlineto(243.0,p);
	    PSmoveto(p,0.1);
	    PSlineto(p,243.0);
	    }
	  PSstroke();
	  }
	}


        if (drawMeanF==YES) {
	i=[probEI selectedRow];
	j=[probEI selectedCol];
	sscanf([[probHowMany selectedCell] title],"%d",&k);
	switch (j) {
	  case 0: switch (i) {
	    case 5: lifreq(k,2);   break;
	    case 4: pdiff(100,6);  break;
//	    case 3: pddiff(100,6); break;
//	    case 2: ptdiff(100,6); break;
//	    case 1: pfdiff(100,6); break;
//	    case 0: pvdiff(100,6); break;
	    default: break;}

	  case 1: switch (i) {
//	    case 4: pidiff(100,6,1); break;
//	    case 3: pidiff(100,6,2); break;
//	    case 2: pidiff(100,6,3); break;
//	    case 1: pidiff(100,6,4); break;
//	    case 0: pidiff(100,6,5); break;
	    default: break;}
	  default: break;}

	if ([probGrid state]) {
	     PSsetlinewidth(0.0);
	     PSsetgray(NX_BLACK);
	     linegrid(10);
	     PSstroke();
	     }
	}

	if (drawBasin==YES) {
/* graph the probability differential for state l over 	*/
/* an extended triangle 	 			*/
   wru(c);
   [ruleNumber setStringValue: c];
	if ([[probEI cellAt:5:0] state]) bas=0; 
	if ([[probEI cellAt:4:0] state]) bas=1;
	if ([[probEI cellAt:3:0] state]) bas=2;
	if ([[probEI cellAt:2:0] state]) bas=3; 
	if ([[probEI cellAt:1:0] state]) bas=4;
	if ([[probEI cellAt:0:0] state]) bas=5;

	if ([[probEI cellAt:4:1] state]) bas=6;
	if ([[probEI cellAt:3:1] state]) bas=7;
	if ([[probEI cellAt:2:1] state]) bas=8;
	if ([[probEI cellAt:1:1] state]) bas=9;
	if ([[probEI cellAt:0:1] state]) bas=10;

nn=100;
en=1.0/((double)(nn));
// for (ii=-nn; ii<=3*nn; ii++) {
//   for (jj=-nn; ii+jj<=2*nn; jj++) {
for (ii=0; ii<=nn; ii++) {
for (jj=0; ii+jj<=nn; jj++) {
    aa[0]=((double)(ii))*en;
    aa[1]=((double)(jj))*en;
    aa[2]=1.0-aa[0]-aa[1];
    switch (bas) {
      case 0: cq[0]=aa[0]; cq[1]=aa[1]; cq[2]=aa[2];
              break;
      case 1: cq[0]=bern(0,aa); cq[1]=bern(1,aa); cq[2]=bern(2,aa);
              break;
      case 2: cq[0]=dbern(0,aa); cq[1]=dbern(1,aa); cq[2]=dbern(2,aa);
	      break;
      case 3: cq[0]=tbern(0,aa); cq[1]=tbern(1,aa); cq[2]=tbern(2,aa);
	      break;
      case 4: cq[0]=fbern(0,aa); cq[1]=fbern(1,aa); cq[2]=fbern(2,aa);
	      break;
      case 5: break;

      case 6: cq[0]=bern(0,aa); cq[1]=bern(1,aa); cq[2]=bern(2,aa);
              break;
      case 7: bb[0]=bern(0,aa); bb[1]=bern(1,aa); bb[2]=bern(2,aa);
	      cq[0]=bern(0,bb); cq[1]=bern(1,bb); cq[2]=bern(2,bb);
              break;
      case 8: bb[0]=bern(0,aa); bb[1]=bern(1,aa); bb[2]=bern(2,aa);
	      cc[0]=bern(0,bb); cc[1]=bern(1,bb); cc[2]=bern(2,bb);
	      cq[0]=bern(0,cc); cq[1]=bern(1,cc); cq[2]=bern(2,cc);
	      break;
      case 9: bb[0]=bern(0,aa); bb[1]=bern(1,aa); bb[2]=bern(2,aa);
	      cc[0]=bern(0,bb); cc[1]=bern(1,bb); cc[2]=bern(2,bb);
	      bb[0]=bern(0,cc); bb[1]=bern(1,cc); bb[2]=bern(2,cc);
	      cq[0]=bern(0,bb); cq[1]=bern(1,bb); cq[2]=bern(2,bb);
	      break;
      case 10: bb[0]=bern(0,aa); bb[1]=bern(1,aa); bb[2]=bern(2,aa);
	      cc[0]=bern(0,bb); cc[1]=bern(1,bb); cc[2]=bern(2,bb);
	      bb[0]=bern(0,cc); bb[1]=bern(1,cc); bb[2]=bern(2,cc);
	      cc[0]=bern(0,bb); cc[1]=bern(1,bb); cc[2]=bern(2,bb);
	      cq[0]=bern(0,cc); cq[1]=bern(1,cc); cq[2]=bern(2,cc);
	      break;
      default: break;
      }
   if ([probFlip state]) tcolordot(cq,aa); else tcolordot(aa,cq);
    }
  }
	     PSstroke();

	if ([probGrid state]) {
	     PSsetlinewidth(0.0);
	     PSsetgray(NX_BLACK);
	     linegrid(10);
	     PSstroke();
	     }

	}

	if (drawEntpy==YES) {
	     penry(100);
	     PSsetlinewidth(0.05);
	     PSsetgray(NX_BLACK);
	     linegrid(10);
	     PSstroke();
	     PSsetlinewidth(0.0);
           }

	if (drawEProb==YES) {
    wru(c);
    [ruleNumber setStringValue: c];
    switch([pswit selectedRow]) {
      case 0: xpdiff(100,6); break;
      case 1: psddiff(100,6); break;
      default: break;}
	     PSsetlinewidth(0.05);
	     PSsetgray(NX_BLACK);
	     xlinegrid(10);
	     PSstroke();
	     PSsetlinewidth(0.0);
           }

	if (drawXBlock==YES) {

   wru(c);
   [ruleNumber setStringValue: c];

   time(&secsnow);
   timeptr = localtime(&secsnow);
   sprintf(ct,"%d-%d-19%02d\n",
	((timeptr->tm_mon) + 1), 
	timeptr->tm_mday,
        timeptr->tm_year);
   [tiDat setStringValue: ct];

    l=0;
    if (strcmp([[probView selectedCell] title],"front")==0) l=0;
    if (strcmp([[probView selectedCell] title],"plan")==0) l=1;
    if (strcmp([[probView selectedCell] title],"side")==0) l=2;

    if ([[probLevel cellAt: 0: 0] state]) {
	  PSsetlinewidth(0.0);
	  PSsetcmykcolor(0.2,0.0,1.0,0.0);
	cudem(cubic,25,25,25,6);
    for (i=0; i<24; i++) for (j=0; j<24; j++) for (k=0; k<24; k++) {
      conth(cubic,0.0,i,j,k,l);
	}
		PSstroke();
      }
    if ([[probLevel cellAt: 1: 0] state]) {
	  PSsetlinewidth(0.0);
	  PSsetcmykcolor(0.0,1.0,0.2,0.0);
	cudem(cubic,25,25,25,7);
    for (i=0; i<24; i++) for (j=0; j<24; j++) for (k=0; k<24; k++) {
      conth(cubic,0.0,i,j,k,l);
	}
		PSstroke();
      }
    if ([[probLevel cellAt: 2: 0] state]) {
	  PSsetlinewidth(0.0);
	  PSsetcmykcolor(1.0,0.0,0.2,0.0);
	cudem(cubic,25,25,25,8);
    for (i=0; i<24; i++) for (j=0; j<24; j++) for (k=0; k<24; k++) {
      conth(cubic,0.0,i,j,k,l);
	}
		PSstroke();
      }
    }

	if (drawNBlk==YES) {
          wru(c);
          [ruleNumber setStringValue: c];
	  nblcur([[nblokLen selectedCell] title][0]);
	  for (i=0; i<KK; i++) 
		[[nblokAve cellAt: 0: i] setDoubleValue: blpr[i]];
	  }

	if (drawStat==YES) {
   wru(c);
   [ruleNumber setStringValue: c];
	    if (strcmp([[statNhood selectedCell] title],"totalistic")==0) {
		[self nxmoncar: 2];
		neisum(0,AL,48);
                }
	    if (strcmp([[statNhood selectedCell] title],"general")==0) {
		[self nxmoncar: 2];
		gnlsum(0,AL,48);
                }
	  }


	/* now take each point and draw it */
	PSsetgray(NX_BLACK);
	i = [points count];
	while (i--) {
		aPoint = (NXPoint *)[points elementAt:i];
		drawCircle(aPoint->x, aPoint->y, radius);
	} 
	return self;
}

- clear:sender
/*
 * Clears the ProbView by emptying its Storage object of all points
 * and then redisplaying the ProbView.  This action is taken only if
 * the ProbView's state requires it.
 */
{
	if (needsClearing) {
		[points empty];
		[self display];
		needsClearing = NO;
	}
	return self;
}

- sizeTo:(NXCoord)width :(NXCoord)height
/*
 * Ensures that whenever the ProbView is resized, the origin of its 
 * coordinate system is repositioned to the center of its area.
 */
{
//	  [self setFlipped: YES];
	[super sizeTo:width :height];
	if (drawview==4||drawview==7) {[self setDrawOrigin:0 :0]; return self;}
	if (drawview!=4&&drawview!=7) [self setDrawOrigin:-floor(width/2) : -floor(height/2)]; 
	return self;
}

- registerPoint:(NXPoint *)aPoint
/*
 * Adds a point to the list the ProbView keeps in its Storage object. 
 */
{
	[points addElement:aPoint];
	return self;
}

- mouseDown:(NXEvent *) theEvent
/*
 * Responds to a message the system sends whenever the user presses the mouse
 * button when the cursor is over the ProbView.  The ProbView changes the 
 * cursor to a cross-hairs image and then starts asking for mouse-dragged or 
 * mouse-up events.  As it receives mouse-dragged events, the ProbView updates 
 * the readOut text Cell with the cursor's coordinates.  If the user releases 
 * the mouse button while the cursor is over the ProbView, the ProbView 
 * registers the point and then sends a message to its delegate notifying it 
 * of the new point.
 */
{
	int	looping = YES, oldMask;
	NXPoint	aPoint;
	NXRect	plotRect, cellRect;
   	char	buffer[100];

         double  x, y, ex, wy, eks, wye;
	 int k;

	[crossCursor set];
   	[self getBounds:&plotRect];
 	NXSetRect(&cellRect,plotRect.origin.x,plotRect.origin.y,100.0,20.0);

	oldMask = [window addToEventMask:NX_MOUSEDRAGGEDMASK];
 				[window setDocEdited:YES];
		                aPoint = theEvent->location;
		                [self convertPoint:&aPoint fromView:nil];
                                ex=aPoint.x;
                                wy=aPoint.y;
				k=nearest(ex,wy);
	plaidline[0]=(int)ex;
	plaidline[1]=243-(int)wy;

//       if (theEvent->data.mouse.click==3) {} else 
//       if (theEvent->data.mouse.click==2) {} else 
	{
	[self lockFocus];
//         PSsetinstance(YES);
	do {
// 		PSnewinstance();
		aPoint = theEvent->location;
		[self convertPoint:&aPoint fromView:nil];
//                ex=aPoint.x;
//                wy=aPoint.y;
		sprintf(buffer, "(%d, %d)", (int)aPoint.x, (int)aPoint.y); 
  		[readOut setStringValue:buffer];
  		[readOut drawInside:&cellRect inView:self];
				PSsetgray(NX_BLACK);
//				PSmoveto(ex+12.0,wy);
//				PSarc(ex, wy, 12.0, 0.0, 360.0);
//				PSstroke();
//				cntr[k][0]=ex;
//				cntr[k][1]=wy;
		[window flushWindow];
		if (theEvent->type == NX_MOUSEUP) {
	    	/* on mouse-up, register point, inform delegate, and clean up state */
			[readOut setStringValue:""];
  			[readOut drawInside:&cellRect inView:self];
			[window flushWindow];
			if (NXPointInRect(&aPoint, &plotRect)) {
				PSsetgray(NX_BLACK);
				drawCircle(aPoint.x, aPoint.y, radius);
				[window flushWindow];
				[self registerPoint:&aPoint];
				needsClearing = YES;
				if (delegate && [delegate respondsTo:@selector(PlotView:pointDidChange:)])
					[delegate PlotView:self pointDidChange:&aPoint];
			}
			looping = NO;
		}	
   	} while (looping && (theEvent = 
		[NXApp getNextEvent:NX_MOUSEUPMASK|NX_MOUSEDRAGGEDMASK]));
	[self unlockFocus];


	}
	[window setEventMask:oldMask];
	[NXArrow set];
	return self;
}

- setRadius:(float)aFloat
/*
 * Sets the value for the radius of the ProbView's points.
 */
{
	radius = aFloat;
	return self;
}


- (float)radius
/*
 * Returns the value for the radius of the ProbView's points.
 */
{
	return radius;
}


- read:(NXTypedStream *)stream
/*
 * Unarchives the ProbView.  Initializes four of the ProbView's instance 
 * variables to the values stored in the stream. 
 */
{
	[super read:stream];
	delegate = NXReadObject(stream);
	NXReadTypes(stream, "@@@f", &points, &crossCursor, &readOut, &radius);
	return self;
}

- write:(NXTypedStream *)stream
/*
 * Archives the ProbView by writing its important instance variables to the 
 * stream. 
 */
{
	[super write:stream];
	NXWriteObjectReference(stream, delegate);
	NXWriteTypes(stream, "@@@f", &points, &crossCursor, &readOut, &radius);
	return self;
}

- awake
/*
 * Finishes initializing a ProbView that has just been unarchived (see the 
 * read: method).
 */
{
	loadPSProcedures();
	return [super awake];
}


- (const char *)inspectorName
{
    return "ProbViewInspector";
}

@end

/* end ProbView.m */
