
/* 
 * VN3DView.m -- Implementation file for the VN3DView class 
 *
 * You may freely copy, distribute, and reuse the code in this example.
 * NeXT disclaims any warranty of any kind, expressed or implied, as to 
 * its fitness for any particular use, nor even that it has a use.
 *
 */

#import "VN3DView.h"
#import "drawingFuncs.h"
#import "nxlcau33.h"
#import <math.h> 
#import <stdlib.h>
#import <string.h>

#define MB 12				/* maximum binomial coefficient */
#define NG 10				/* number of Golay Classes	*/

extern char ascrule[KK][KK][KK][KK][KK][KK][KK];
extern char auxrule[KK][KK][KK][KK][KK][KK][KK];
extern char golRule[];			/* evolutionary rule */
extern char sqrule[];			/* evolutionary rule */
extern char ttrule[];			/* evolutionary rule */

extern char    arr1[], arr2[];
extern char    arr4[];
extern  int    brr2[];
extern double  bp[];			/* Bernstein coefficients    */
extern int     gc[];			/* Golay code key array      */
extern int     tc[];			/* totalistic key array      */
extern int     stc[];			/* s-totalistic key array    */
extern int     gus[];			/* totalistic use stats      */
extern int     tus[];			/* totalistic use stats      */
extern int     stus[];			/* s-totalistic use stats    */
extern int     lf[];			/* number of live cells	     */
extern int     llf[];			/* save number of live cells */
       double  flf[KK];			/* save number of live cells */

extern NXColor colst[KK];    		/* Array for state colors    */

int    cellNood=NO;
int    checkBoard=NO;
int    drawARBack=YES;
int    drawEMeanPH=NO;
int    drawMeanPH=NO;
int    drawMosaic=NO;
int    drawFreqs=NO;
int    drawQuency=NO;
int    drawPlane=NO;
int    drawRturn=NO;
int    drawSalami=NO;
int    drawSmall=NO;
int    drawSQuency=NO;
int    drawUQuency=NO;
int    drawTrajec=NO;
int    slowFreqs=NO;
int    smallSmall=NO;
int    smallMosaic=NO;

int    lastPlane=0;			/* 0 = none, 1 = evol, 2 = mosaic */
int    lastTraj=0;			/* 0 = none, 1 = traj, 2 = return */
int    lastReturn=0;
int    moTiles=3;
int    pDots=NO;
int    pGrid=NO;
int    skipThree=NO;
int    skipSThree=NO;
int    stretchU=125;
int    trajLen=10;
int    repEMFg=0;
int    repPlEv=0;
int    rturnNum=1;

int     totpmut[TS] = {0,1,2,3,4,5,6,7};
int     stotpmut[KK*(TS-1)] = {0,1,2,3,4,5,6,7,8,9,10,11,12,13};
int     gtb[NG] = {0,1,2,2,3,3,4,4,5,6};
char    masstbl[8][8] = {{'2','0','0','0','0','1','0','0'},
			 {'0','0','0','0','1','0','0','0'},
			 {'0','0','0','1','0','0','0','0'},
			 {'0','0','1','0','0','0','0','0'},
			 {'0','1','0','0','0','0','0','0'},
			 {'1','0','0','0','0','0','0','0'},
			 {'0','0','0','0','0','0','0','0'},
			 {'0','0','0','0','0','0','0','0'}};
double  ttb[TS] = {1.0,7.0,21.0,35.0,35.0,21.0,7.0,1.0};
NXColor totcolst[TS];    		/* Array for totalistic colors   */
NXColor stotcolst[KK*(TS-1)];    	/* Array for s-totalistic colors */

extern int    drawview;
extern int    co, dn;

void    arbdot(int i, int j, int l);
void    arbline(double x, double y, int l);
void    argfram(int n, int l);
void    argdiag(int n, int l);
void    arscreen(char *arr1, int j, int k, int l, int m);
void    asctobin();
void	asctoqar();
void	asczero();
void    bargraf(int m, int n);
void    cuonegen(char *aa, int *bb, int l, int m, int n); 
int     istot();
int     ix2(int i, int j);
int     ix3(int i, int j, int k);
void    manyrect(int i, int j, int l, int m, int n, int q);
int     nearest(double x, double y);
void    onegen(int n);
void    ranblin(double p1, double p2);
void    ranpatch(double p[KK], int l, int m, int n);
void    renewarry();
void    savearry();
void    screen(char *aa, int m, int n);
void    smallgrid(int m, int n);
void    statecount(char *aa, int l, int m, int n);
void    suggrule(int i);
int     totonu(int n);
void    videodot(int row, int col, int color);
void    wrru(char *c, char ru[KK][KK][KK][KK][KK][KK][KK]);
void    wxsscrrul(char *c, char ru[KK][KK][KK][KK][KK][KK][KK]);
void    zerline();

@implementation VN3DView
 
/* -------------------------------- S Q A R -------------------------------- */

/* put up the principal window for the PairView. Since pair.nib is a 	  */
/* secondary .nib file, it is not loaded until there is a demand, which   */
/* would usually arise from the "Pair" option in the main menu.	   	  */

- loadSqar:sender {
  if (SqarWindow==nil) {
    if (![NXApp loadNibSection: "vn3d.nib" owner: self withNames: NO]) 
	{return nil;}
    }
  dn=3;
  [SqarWindow makeKeyAndOrderFront: sender];
  [sqarWindow orderFront: nil];
  [GolayPanel orderFront: nil];
  [self golaySurround: sender];
  [self copyRule: sender];
return self;
}

- chmindex:sender {idempot(); return self;}

void drawsquare() {int i, j;
  for (i=0; i<128; i++) for (j=0; j<128; j++) videodot(j,i,arr1[i*128+j]+1);
}

void drawcube() {int i, j, k;
  for (i=0; i<32; i++) for (j=0; j<32; j++) for (k=0; k<16; k++) 
    videodot(j+32*(k/4),i+32*(k%4),arr1[ix3(i,j,k)]+1);
}

void manysquare(i,j) int i, j; {int k, l;
  for (k=0; k<128; k++) for (l=0; l<128; l++)  
    videodot(k+128*i,l+128*j,arr1[ix2(k,l)]+1);
}

void drawgrid() {int i;
	  PSsetgray(NX_BLACK); 
	  PSsetlinewidth(0.1);
  for (i=0; i<5; i++) {
  PSmoveto((double)(128.0*i),0.0); PSlineto((double)(128.0*i),512.0);}
  for (i=0; i<5; i++) {
  PSmoveto(0.0,(double)(128.0*i)); PSlineto(512.0,(double)(128.0*i));}
  PSstroke();
}

- printSQAR:sender {
   if (keyWindow==SqarWindow) 
	{[SqarWindow smartPrintPSCode: self];}
   else if (keyWindow==GolayPanel || keyWindow==stretchWindow) 
	{[keyWindow smartPrintPSCode: self];}
   else if (keyWindow==planeWindow) {
    switch (lastPlane) {
      case 0: break;
      case 1: drawPlane=YES;   break;
      case 2: drawMosaic=YES;  break;
      case 3: drawSmall=YES;   break;
      case 4: smallMosaic=YES; break;
      default: break;}
    [planeWindow smartPrintPSCode: self];
    switch (lastPlane) {
      case 0: break;
      case 1: drawPlane=NO;   break;
      case 2: drawMosaic=NO;  break;
      case 3: drawSmall=NO;   break;
      case 4: smallMosaic=NO; break;
      default: break;}
    }
return self;
}

- windowDidBecomeKey:sender {
	keyWindow=sender;
	[menuCopy setTarget: self];
	[menuCopy setAction: @selector(copySQAR:)];
	[menuCopy setEnabled: YES];
	[menuPrint setTarget: self];
	[menuPrint setAction: @selector(printSQAR:)];
	[menuPrint setEnabled: YES];
return self;
}

- windowDidResignKey:sender {
	[menuCopy setEnabled: NO];
	[menuPrint setEnabled: NO];
return self;
}

- copySqarView:sender {
NXRect zbounds;
id     pBoard;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [SqarView getBounds: &zbounds];
  [SqarView writePSCodeInside: &zbounds to: pBoard]; 
return self;
}

- copyGolayPanel:sender {
NXRect zbounds;
id     pBoard;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [GolayView getBounds: &zbounds];
  [GolayView writePSCodeInside: &zbounds to: pBoard]; 
return self;
}

- copyQuency:sender {
NXRect zbounds;
id     pBoard;
  drawQuency = YES;
  drawview = 14;
  drawARBack = NO;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  if ([justTheView state]) {
    [freqView getBounds: &zbounds];
    [freqView writePSCodeInside: &zbounds to: pBoard]; 
    } else {
    [[freqWindow contentView] getBounds: &zbounds];
    [[freqWindow contentView] writePSCodeInside: &zbounds to: pBoard]; 
    }
  drawARBack = YES;
  drawQuency = NO;
return self;
}

- copySQuency:sender {
NXRect zbounds;
id     pBoard;
  drawSQuency = YES;
  drawview = 14;
  drawARBack = NO;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  if ([justTheSView state]) {
    [sfreqView getBounds: &zbounds];
    [sfreqView writePSCodeInside: &zbounds to: pBoard]; 
    } else {
    [[sfreqWindow contentView] getBounds: &zbounds];
    [[sfreqWindow contentView] writePSCodeInside: &zbounds to: pBoard]; 
    }
  drawARBack = YES;
  drawSQuency = NO;
return self;
}

- copyCube:sender {
NXRect zbounds;
id     pBoard;
  drawSalami = YES;
  drawview = 14;
  drawARBack = NO;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
    [cubeView getBounds: &zbounds];
    [cubeView writePSCodeInside: &zbounds to: pBoard]; 
  drawARBack = YES;
  drawSalami = NO;
return self;
}

- copyWhoosit: (int *) flag {
NXRect zbounds;
id     pBoard;
  *flag = YES;
  drawview=13;
  drawARBack=NO;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [trajView getBounds: &zbounds];
  [trajView writePSCodeInside: &zbounds to: pBoard]; 
  drawARBack=YES;
  *flag = NO;
return self;
}

- copyWhatsit: (int *) flag {
NXRect zbounds;
id     pBoard;
  *flag = YES;
  drawARBack=NO;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [planeView getBounds: &zbounds];
  [planeView writePSCodeInside: &zbounds to: pBoard]; 
  drawARBack=YES;
  *flag = NO;
return self;
}

- copySQAR:sender {
   if (keyWindow==SqarWindow) {[self copySqarView: self];}
   else if (keyWindow==GolayPanel) {[self copyGolayPanel: self];}
   else if (keyWindow==trajWindow) {
     savearry();
     switch (lastTraj) {
          case 0: break;
          case 1: [self copyWhoosit: &drawTrajec]; break;
          case 2: [self copyWhoosit: &drawRturn];  break;
          case 3: [self copyWhoosit: &drawFreqs];  break;
          case 4: [self copyWhoosit: &slowFreqs];  break;
          default: break;}
     renewarry();}
   else if (keyWindow==planeWindow) switch (lastPlane) {
     case 0: break;
     case 1: savearry(); [self copyWhatsit: &drawPlane]; renewarry();   break;
     case 2: savearry(); [self copyWhatsit: &drawMosaic]; renewarry();  break;
     case 3: drawview=7; repPlEv=1; [self copyWhatsit: &drawSmall]; break;
     case 4: savearry(); drawview=11; [self copyWhatsit: &smallMosaic]; 
		renewarry(); break;
     default: break;}
   else if (keyWindow==freqWindow)    [self copyQuency: self];
   else if (keyWindow==sfreqWindow)   [self copySQuency: self];
   else if (keyWindow==stretchWindow) [self copyStretch: self];
   else if (keyWindow==cubeWindow)    [self copyCube: self];
return self;
}

- drawPlEvol:sender {int i;
  repPlEv=[[sender selectedCell] tag];
  drawview=7;
  drawPlane=YES;
  asctobin();
  if (repPlEv==0) [self display]; else for (i=0; i<repPlEv; i++) {
     [genEv setIntValue: i];
     onegen(AL);
     [self display];
     }
  [genEv setIntValue: repPlEv];
  drawPlane=NO;
  lastPlane=1;
return self;
}

- printPlEvol:sender {
    repPlEv=0;
    drawview=7;
    drawPlane=YES;
    [keyWindow smartPrintPSCode: self];
    drawPlane=NO;
return self;
}

- drawMosaic:sender {
    drawview=11;
    drawMosaic=YES;
    [self display];
    drawMosaic=NO;
    lastPlane=2;
return self;
}

- printMosaic:sender {
    drawview=11;
    drawMosaic=YES;
    [keyWindow smartPrintPSCode: self];
    drawMosaic=NO;
return self;
}

- showProbability:sender {int i; double pp, p[KK];
  for (i=0; i<KK; i++) p[i]=[[sender cellAt: i: 0] doubleValue];
  for (pp=0, i=0; i<KK; i++) pp+=p[i];
  for (i=0; i<KK; i++) {
    [[pSliders cellAt: i: 0] setDoubleValue: p[i]/pp];
    [[pWindows cellAt: i: 0] setDoubleValue: p[i]/pp];
    }
return self;
}

- randomLine:sender {double p[KK]; int i;
  for (i=0; i<KK; i++) p[i]=[[pSliders cellAt:i:0] doubleValue];
  ranblin(p[0],p[1]);
return self;
}

- sCellLine:sender {
  smallSmall=YES;
  [planeView display];
  smallSmall=NO;
return self;
}

- uniformLine:sender {ranblin(0.5,0.5); return self;}

void ranpatch(p,l,m,n) double p[KK]; int l, m, n; {int a, b, c, i, j, k, l;
  for (i=0; i<l; i++) for (j=0; j<m; j++) for (k=0; k<n; k++) {
    l=n*(m*i+j)+k;
    a=(int)(1000.0*p[0]);
    b=a+(int)(1000.0*p[1]);
    c=random()%1000;
    if (c<a) arr1[l]=0; else if (c<b) arr1[l]=1; else arr1[l]=2;
    }
  }

- checkerBoard:sender {
  checkBoard=YES;
  [planeView display];
  checkBoard=NO;
return self;
}

- uprule:sender {suggrule(0); [self anyrule: self]; return self;}
- lerule:sender {suggrule(1); [self anyrule: self]; return self;}
- dorule:sender {suggrule(3); [self anyrule: self]; return self;}
- rirule:sender {suggrule(2); [self anyrule: self]; return self;}
- idrule:sender {suggrule(4); [self anyrule: self]; return self;}
- mjrule:sender {suggrule(5); [self anyrule: self]; return self;}

- anyrule:sender {
   [self copyRule: self];
   [rulePanel setAllRules: self];
return self;}

void suggrule(i) int i; {int j, k[KK], i0, i1, i2, i3, i4, i5, i6;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
  for (i4=0; i4<KK; i4++)
  for (i5=0; i5<KK; i5++)
  for (i6=0; i6<KK; i6++) {
    switch (i) {
      case 0: j=i2; break;
      case 1: j=i3; break;
      case 2: j=i1; break;
      case 3: j=i0; break;
      case 4: j=i4; break;
      case 5: j=i5; break;
      case 6: j=i6; break;
      case 7: j=0; if (i0+i1+i2+i3+i4+i5+i6>3) j=1; break;
      default: j=0; break;}
    ascrule[i0][i1][i2][i3][i4][i5][i6] = '0'+j;
    }
}

- mrule:sender {massrule(); [rulePanel setAllRules: self]; return self;}

void massrule() {int i, j, k[KK], i0, i1, i2, i3, i4, i5, i6;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
  for (i4=0; i4<KK; i4++)
  for (i5=0; i5<KK; i5++)
  for (i6=0; i6<KK; i6++) {
    for (i=0; i<KK; i++) k[i]=0;
    k[i0]++; k[i1]++; k[i2]++; k[i3]++; k[i4]++; k[i5]++; k[i6]++;
    ascrule[i0][i1][i2][i3][i4][i5][i6]=masstbl[k[2]][k[1]];
// printf("%2d, %2d, %c\n",k[0],k[1],masstbl[k[0]][k[1]]);
    }
}

/* --------------------------- E V O L U T I O N --------------------------- */

- drawSmEvol:sender {int k;
    drawview=7;
    drawSmall=YES;
    asctoqar();
    if ([reStore state]) savearry();
    sscanf([[sender selectedCell] title],"%d",&repPlEv);
    [self display];
    if (repPlEv>0) for (k=0; k<repPlEv; k++) [self display];
    if ([reStore state]) renewarry();
    drawSmall=NO;
    lastPlane=3;
return self;
}

- printSmEvol:sender {
    drawview=7;
    drawSmall=YES;
    repPlEv=0;
    [keyWindow smartPrintPSCode: self];
    drawSmall=NO;
return self;
}

- drawSmMosaic:sender {
    cellNood=[cellNhood state];
    moTiles=[moTies selectedRow]+2;
    drawview=11;
    smallMosaic=YES;
    asctoqar();
    savearry();
    [self display];
    renewarry();
    smallMosaic=NO;
    lastPlane=4;
return self;
}

- printSmMosaic:sender {
    drawview=11;
    smallMosaic=YES;
    savearry();
    [keyWindow smartPrintPSCode: self];
    renewarry();
    smallMosaic=NO;
return self;
}

- drawCube:sender {
    [cubeWindow makeKeyAndOrderFront: sender];
    drawview=14;
    drawSalami=YES;
    [cubeView display];
    drawSalami=NO;
    lastPlane=5;
return self;
}

- evolveCube:sender {
    [cubeWindow makeKeyAndOrderFront: sender];
    drawview=14;
    drawSalami=YES;
    asctoqar();
    cuonegen(arr1,brr2,25,25,25);
    [cubeView display];
    drawSalami=NO;
    lastPlane=5;
return self;
}

- printCube:sender {
    drawview=11;
    drawSalami=YES;
    [keyWindow smartPrintPSCode: self];
    drawSalami=NO;
return self;
}

- drawTraj:sender {
    sscanf([[sender selectedCell] title],"%d",&trajLen);
    pDots=[probDots state];
    pGrid=[probGrid state];
    drawview=9;
    drawTrajec=YES;
    asctoqar();
    savearry();
    [trajView display];
    renewarry();
    drawTrajec=NO;
    lastTraj=1;
return self;
}

- printTraj:sender {
    drawview=9;
    drawTrajec=YES;
    savearry();
    [keyWindow smartPrintPSCode: self];
    renewarry();
    drawTrajec=NO;
return self;
}

- drawRtrn:sender {
    sscanf([[sender selectedCell] title],"%d",&rturnNum);
    drawview=13;
    drawRturn=YES;
    asctoqar();
    savearry();
    [trajView display];
    renewarry();
    drawRturn=NO;
    lastTraj=2;
return self;
}

- printRtrn:sender {
    drawview=13;
    drawRturn=YES;
    savearry();
    [keyWindow smartPrintPSCode: self];
    renewarry();
    drawRturn=NO;
return self;
}

- drawFreq:sender {
    drawview=13;
    drawFreqs=YES;
    asctoqar();
    savearry();
    [trajView display];
    renewarry();
    drawFreqs=NO;
    lastTraj=3;
return self;
}

- printFreq:sender {
    drawview=13;
    drawFreqs=YES;
    savearry();
    [keyWindow smartPrintPSCode: self];
    renewarry();
    drawFreqs=NO;
return self;
}

- slowFreq:sender {
    drawview=13;
    slowFreqs=YES;
    asctoqar();
    savearry();
    [trajView display];
    renewarry();
    slowFreqs=NO;
    lastTraj=4;
return self;
}

- printSlowFreq:sender {
    drawview=13;
    slowFreqs=YES;
    savearry();
    [keyWindow smartPrintPSCode: self];
    renewarry();
    slowFreqs=NO;
return self;
}

- drawFrequency:sender {
    skipThree=[eachAndAll state];
    totpmut[0]=[freqPerm0 selectedCol];
    totpmut[1]=[freqPerm1 selectedCol];
    totpmut[2]=[freqPerm2 selectedCol];
    totpmut[3]=[freqPerm3 selectedCol];
    totpmut[4]=[freqPerm4 selectedCol];
    totpmut[5]=[freqPerm5 selectedCol];
    totpmut[6]=[freqPerm6 selectedCol];
    totpmut[7]=[freqPerm7 selectedCol];
    totcolst[0]=[freqColor0 color];
    totcolst[1]=[freqColor1 color];
    totcolst[2]=[freqColor2 color];
    totcolst[3]=[freqColor3 color];
    totcolst[4]=[freqColor4 color];
    totcolst[5]=[freqColor5 color];
    totcolst[6]=[freqColor6 color];
    totcolst[7]=[freqColor7 color];
    drawview=14;
    drawQuency=YES;
    asctoqar();
    savearry();
    [freqView display];
    renewarry();
    drawQuency=NO;
return self;
}

- printFrequency:sender {
    drawview=14;
    drawQuency=YES;
    savearry();
    [keyWindow smartPrintPSCode: self];
    renewarry();
    drawQuency=NO;
return self;
}

- drawSemiFreq:sender {int i;
    skipSThree=[eachAndSAll state];
    for (i=0; i<TS-1; i++) {
	stotpmut[i]=[[sfreqPerm0 cellAt: i: 0] intValue];
	stotpmut[i+TS-1]=[[sfreqPerm1 cellAt: i: 0] intValue];
	}
    stotcolst[0]=[sfreqColor00 color];
    stotcolst[1]=[sfreqColor01 color];
    stotcolst[2]=[sfreqColor02 color];
    stotcolst[3]=[sfreqColor03 color];
    stotcolst[4]=[sfreqColor04 color];
    stotcolst[5]=[sfreqColor05 color];
    stotcolst[6]=[sfreqColor06 color];
    stotcolst[7]=[sfreqColor10 color];
    stotcolst[8]=[sfreqColor11 color];
    stotcolst[9]=[sfreqColor12 color];
    stotcolst[10]=[sfreqColor13 color];
    stotcolst[11]=[sfreqColor14 color];
    stotcolst[12]=[sfreqColor15 color];
    stotcolst[13]=[sfreqColor16 color];
    drawview=14;
    drawSQuency=YES;
    asctoqar();
    savearry();
    [sfreqView display];
    renewarry();
    drawSQuency=NO;
return self;
}

- printSemiFreq:sender {
    drawview=14;
    drawSQuency=YES;
    savearry();
    [keyWindow smartPrintPSCode: self];
    renewarry();
    drawSQuency=NO;
return self;
}

/* draw rectangle (i,j) in a mosaic of (u,v) */
/* whose individual rectangles measure (l,m) */
void manyrect(i,j,l,m,n,q) int i, j, l, m, n, q; {int ii, jj, pj;
  asctoqar();
  for (ii=0; ii<m; ii++) for (jj=0; jj<n; jj++) 
    if (cellNood) {pj=totpmut[tc[brr2[ii+n*jj]]];
		 NXSetColor(totcolst[pj]);
	         videodot(i*m+ii,j*n+jj,5);
		 }
            else videodot(i*m+ii,j*n+jj,arr1[ii+n*jj]);
  cuonegen(arr1,brr2,l,m,n);
}

/* m = high/moTiles, n = wide/moTiles */
void smallgrid(m,n) int m, n; {int i; double em, en, mo;
  em=(double)m;
  en=(double)n;
  mo=(double)moTiles;
	  PSsetgray(NX_BLACK); 
	  PSsetlinewidth(0.1);
  for (i=0; i<moTiles+1; i++) {
  PSmoveto((double)(n*i),0.0); PSlineto((double)(n*i),mo*em);}
  for (i=0; i<moTiles+1; i++) {
  PSmoveto(0.0,(double)(m*i)); PSlineto(mo*en,(double)(m*i));}
  PSstroke();
}

/* -----------------------  D E F I N E   R U L E S  ----------------------- */

/* - - - - - - - -   B E R N S T E I N   H I S T O G R A M S   - - - - - - */

- drawBern:sender {
    drawview=9;
    drawMeanPH=YES;
    [self display];
    drawMeanPH=NO;
    [window setDocEdited:YES];
    lastReturn=1;
return self;
}

- drawEmpBern:sender {
    sscanf([[sender selectedCell] title],"%d",&repEMFg);
    drawview=9;
    drawEMeanPH=YES;
    savearry();
    [self display];
    renewarry();
    drawEMeanPH=NO;
    [window setDocEdited:YES];
    lastReturn=3;
return self;
}

- golaySurround:sender {id cell; char *c; int i, j, ii;
    for (i=0; i<NG; i++) for (j=0; j<MB; j++) {
      cell=[theZMatrix cellAt: j: i];
      if ([cell isEnabled]) {
	c=[cell title];
	ii=c[0]-'0'+KK*(c[1]-'0'+KK*(c[2]-'0'+KK*(c[3]-'0'+
		KK*(c[4]-'0'+KK*(c[5]-'0')))));
        gc[ii]=i; gc[DS/KK+ii]=NG+i;
	}
      }
return self;
}

- enterRule:sender {id cell; char *c, ch[WNML]; int i, j;
char    tiempo[30], fecha[30];
struct  tm  *timeptr;
time_t  secsnow;

   time(&secsnow);
   timeptr = localtime(&secsnow);
   sprintf(tiempo,"%02d:%02d:%02d\n",
	timeptr->tm_hour, 
	timeptr->tm_min,
        timeptr->tm_sec);
   [tiTime setStringValue: tiempo];
   sprintf(fecha,"%d-%d-19%02d\n",
	((timeptr->tm_mon) + 1), 
	timeptr->tm_mday,
        timeptr->tm_year);
   [tiDat setStringValue: fecha];

    for (i=0; i<NG; i++) for (j=0; j<MB; j++) {
      cell=[theZMatrix cellAt: j: i];
      if ([cell isEnabled]) {
	c=[cell title];
        ascrule[c[0]-'0']
		[c[1]-'0']
		[c[2]-'0']
		[c[3]-'0']
		[c[4]-'0']
		[c[5]-'0']
		[0]         =[cell intValue]+'0';
	}
      cell=[theOMatrix cellAt: j: i];
      if ([cell isEnabled]) {
	c=[cell title];
        ascrule[c[0]-'0']
		[c[1]-'0']
		[c[2]-'0']
		[c[3]-'0']
		[c[4]-'0']
		[c[5]-'0']
		[1]         =[cell intValue]+'0';
	}
      }
      wrru(ch,ascrule);
      [ruleNumber setStringValue: ch];
      wxsscrrul(ch,ascrule);
      [xsRuleNumber setStringValue: ch];
      wysscrrul(ch,ascrule);
      [ysRuleNumber setStringValue: ch];
      if (istot()) {
		[tRuleNumber setIntValue: totonu(0)];
		for (i=0; i<TS; i++) 
		  [[theToMatrix cellAt: 0: i] setIntValue: ttrule[i]-'0'];
		}
      else [tRuleNumber setStringValue: "enter totalistic #"];
      [SqarWindow setDocEdited: NO];
      [rulePanel setAllRules: self];
      [bernView selfBern: nil];
return self;
}

- resetRule:sender {int i, j; 
    for (i=0; i<NG; i++) for (j=0; j<MB; j++) {
	[[theOMatrix cellAt: j: i] setIntValue: NO];
	[[theZMatrix cellAt: j: i] setIntValue: NO];
	}
return self;
}

- copyRule:sender {id cell; char *c, ch[WNML]; int i, j;
    for (i=0; i<NG; i++) for (j=0; j<MB; j++) {
      cell=[theZMatrix cellAt: j: i];
      if ([cell isEnabled]) {
	c=[cell title];
        [cell setIntValue: ascrule[c[0]-'0']
				[c[1]-'0']
				[c[2]-'0']
				[c[3]-'0']
				[c[4]-'0']
				[c[5]-'0']
				[0]-'0'        ];
	}
      cell=[theOMatrix cellAt: j: i];
      if ([cell isEnabled]) {
	c=[cell title];
        [cell setIntValue: ascrule[c[0]-'0']
				[c[1]-'0']
				[c[2]-'0']
				[c[3]-'0']
				[c[4]-'0']
				[c[5]-'0']
				[1]-'0'        ];
	}
      }
      wrru(ch,ascrule);
      [ruleNumber setStringValue: ch];
      wxsscrrul(ch,ascrule);
      [xsRuleNumber setStringValue: ch];
      wysscrrul(ch,ascrule);
      [ysRuleNumber setStringValue: ch];
      if (istot()) {
		[tRuleNumber setIntValue: totonu(0)];
		for (i=0; i<TS; i++) 
		  [[theToMatrix cellAt: 0: i] setIntValue: ttrule[i]-'0'];
		}
      else [tRuleNumber setStringValue: "enter totalistic #"];
      [SqarWindow setDocEdited: NO];
      [bernView selfBern: nil];
return self;
}

- readRule:sender 
    {[rulePanel readWRule: sender]; [self copyRule: self]; return self;}

- readTRule:sender 
    {[rulePanel readTRule: sender]; [self copyRule: self]; return self;}

- enterTRule:sender {char ch[WNML];
char    tiempo[30], fecha[30];
struct  tm  *timeptr;
time_t  secsnow;

   time(&secsnow);
   timeptr = localtime(&secsnow);
   sprintf(tiempo,"%02d:%02d:%02d\n",
	timeptr->tm_hour, 
	timeptr->tm_min,
        timeptr->tm_sec);
   [tiTime setStringValue: tiempo];
   sprintf(fecha,"%d-%d-19%02d\n",
	((timeptr->tm_mon) + 1), 
	timeptr->tm_mday,
        timeptr->tm_year);
   [tiDat setStringValue: fecha];

     ttrule[[sender selectedCol]] = [[sender selectedCell] state]+'0';
     totoasc();
     wrru(ch,ascrule);
     [ruleNumber setStringValue: ch];
     [rulePanel setAllRules: self];
     [SqarWindow setDocEdited: NO];
     if (istot()) [tRuleNumber setIntValue: totonu(0)];
       else [tRuleNumber setStringValue: "enter totalistic #"];
return self;
}

- copyTRule:sender {char ch[WNML]; int i;
     wrru(ch,ascrule);
     [ruleNumber setStringValue: ch];
     if (istot()) {
		[tRuleNumber setIntValue: totonu(0)];
		for (i=0; i<TS; i++) 
		  [[theToMatrix cellAt: 0: i] setIntValue: ttrule[i]-'0'];
		}
       else [tRuleNumber setStringValue: "enter totalistic #"];
     [SqarWindow setDocEdited: NO];
     [bernView selfBern: nil];
return self;
}

/* -------------------------------- * * * * -------------------------------- */

- initFrame:(const NXRect *)frameRect
/*
 * Initializes the new VN3DView object.  First, an initFrame: message is sent 
 * to super to initialize VN3DView as a View.  Next, the VN3DView sets its own 
 * state -- that it is opaque and that the origin of its coordinate system lies 
 * in the center of its area.  It then creates and initializes its associated 
 * objects, a Storage object, an NXCursor, and a Cell.  Finally, it loads into 
 * the Window Server some PostScript procedures that it will use in drawing 
 * itself. 
 */
{
    NXPoint	spot;
	
    [super initFrame:frameRect];
/*  [self translate:floor(frame.size.width/2) :floor(frame.size.height/2)]; */
    [self setOpaque:YES];
    crossCursor=[NXCursor newFromImage:[NXImage newFromSection:"cross.tiff"]];
    spot.x = spot.y = 7.0;
    [crossCursor setHotSpot:&spot];

  PSsetgray(NX_BLACK);
  PSselectfont("Times-Roman",16.0);

  [self setFlipped: YES];
  drawview=7;			      /* prefer origin in upper left corner */
	return self;
}

- setDelegate:anObject
/*
 * Sets the VN3DView's delegate instance variable to the supplied object.
 */
{
	delegate = anObject;
	return self;
}

- drawSelf:(const NXRect *)rects :(int)rectCount
/*
 * Draws the VN3DView's background and axes.  If there are any points,
 * these are drawn too.
 *
 * (Note:  For simplicity, although VN3DView only repaints the background 
 * of the update region, it redraws the entire axes.  For better performance, 
 * only those parts of the axes that fall within the update region should 
 * be redrawn.)   
 */
{
	int		 i, j, k, l, n, ii, jj;
        double           p, pp[KK], q, qq[KK], r, s, t, mo, en;
	NXRect		 plotRect;

	if (rects == NULL) return self;

	/* paint visible area white then draw axes */
	PSsetgray(NX_WHITE);
	if (drawARBack) NXRectFill(&rects[0]);
	PSsetgray(NX_DKGRAY);

	if (drawTrajec==YES) {
	     PSsetlinewidth(0.1);
	     PSsetgray(NX_BLACK);
	     if (pGrid) linegrid(10); else linegrid(1);
	     PSstroke();
	  statecount(arr1,16,16,16);
	  for (j=0; j<KK; j++) flf[j]=(double)lf[j];
	  if (pDots) videotrdot(flf,3); else videotrline(flf,0);
	  for (i=0; i<trajLen; i++) {
	    statecount(arr1,16,16,16);
	    for (j=0; j<KK; j++) flf[j]=(double)lf[j];
	    if (pDots) videotrdot(flf,3); else videotrline(flf,3);
	    cuonegen(arr1,brr2,16,16,16);
	    }
	  PSstroke();
	  }

	if (drawRturn==YES) {
	  argfram(75,3);
	  argdiag(75,3);
	  bargraf(TS-1,75);
	  for (i=0; i<75; i++) {
	    p=((double)i)/75.0; q=1.0-p;
	    ranblin(q,p);
	    for(j=0; j<rturnNum; j++) cuonegen(arr1,brr2,16,16,16);
	    arbdot(i,(75*lf[1])/4096,3);
	    }
	  }

	if (drawFreqs==YES) {
	  argfram(75,3);
	  for (i=0; i<75; i++) {
	    cuonegen(arr1,brr2,16,16,16);
	    for (j=0, k=0; j<TS; j++)
		if (tus[j]>57) for (l=0; l<tus[j]; l+=58) arbdot(i,k++,j%3);
	    arbdot(i,(75*lf[1])/4096,3);
	    }
// printf("teecees:"); for (i=0; i<DS; i++) printf("%5d ",tc[i]); printf("\n");
printf("tussies: "); for (i=0; i<TS; i++) printf("%5d ",tus[i]); printf("\n");
	  }

	if (slowFreqs==YES) {
	  argfram(75,3);
	  for (i=0; i<75; i+=3) {
	    cuonegen(arr1,brr2,16,16,16);
	    for (j=0, k=0; j<TS; j++)
		if (tus[j]>57) for (l=0; l<tus[j]; l+=58) {
		  arbdot(i,k,j%3);
		  arbdot(i+1,k,j%3);
		  arbdot(i+2,k,j%3);
		  k++;
		  }
	    arbdot(i,(75*lf[1])/4096,3);
	    arbdot(i+1,(75*lf[1])/4096,3);
	    arbdot(i+2,(75*lf[1])/4096,3);
	    }
	  }

	if (drawQuency==YES) {
	  argfram(125,3);
	  for (i=0; i<125; i++) {
	    ii=i; if (skipThree) ii=i/3+43*(i%3);
	    cuonegen(arr1,brr2,16,16,16);
	    for (j=0, k=0; j<TS; j++) {
		jj=totpmut[j];
		NXSetColor(totcolst[jj]);
		if (tus[jj]>32) 
		    for (l=0; l<tus[jj]; l+=33) arbdot(ii,k++,5);
		}
	    arbdot(ii,(125*lf[1])/4096,3);
	    }
	  }

	if (drawSQuency==YES) {
	  argfram(125,3);
	  for (i=0; i<125; i++) {
	    ii=i; if (skipSThree) ii=i/3+43*(i%3);
	    cuonegen(arr1,brr2,16,16,16);
	    for (j=0, k=0; j<KK*(TS-1); j++) {
		jj=stotpmut[j];
		NXSetColor(stotcolst[jj]);
		if (stus[jj]>32) 
		    for (l=0; l<stus[jj]; l+=33) arbdot(ii,k++,5);
		}
	    arbdot(ii,(125*lf[1])/4096,3);
	    }
printf("stotpmut: "); 
for (i=0; i<KK*(TS-1); i++) printf("%5d ",stotpmut[i]); 
printf("\n");
printf("stussies: "); 
for (i=0; i<KK*(TS-1); i++) printf("%5d ",stus[i]); 
printf("\n");
	  }

        if (checkBoard==YES) {
   	  [self getBounds: &plotRect];
	  mo=(double)moTiles;
	  j=(int)(plotRect.size.width/mo);
	  i=(int)(plotRect.size.height/mo);
	  zerline();
          for (k=0; k<i/2; k++) {
	    for (l=0; l<j/2; l++) arr1[k*j+l]=0;
            for (l=j/2; l<j; l++) arr1[k*j+l]=1;
            }
          for (k=i/2; k<i; k++) {
	    for (l=0; l<j/2; l++) arr1[k*j+l]=1;
            for (l=j/2; l<j; l++) arr1[k*j+l]=0;
            }
	  }

        if (smallSmall==YES) {
	  mo=(double)moTiles;
   	  [self getBounds: &plotRect];
	  j=(int)(plotRect.size.width/mo);
	  i=(int)(plotRect.size.height/mo);
	  zerline();
          for (k=i/3; k<2*(i/3); k++) for (l=j/3; l<2*(j/3); l++) 
		arr1[k*j+l]=random()%KK;
	  }

        if (drawSmall==YES) {
	  [window setTitle: "evolution"];
   	  [self getBounds: &plotRect];
	  j=(int)(plotRect.size.width/4.0);       /* 4.0 for drawview=7 */
	  i=(int)(plotRect.size.height/4.0);
	  if (j*i*i < AL) {
	    arscreen(arr1,j,i,i,2);
	    if(repPlEv>0) cuonegen(arr1,brr2,j,i,i);
	    } else printf("volume = %6d\n",j*i*i);
	  }

        if (smallMosaic==YES) {
	  mo=(double)moTiles;
	  [window setTitle: "mosaic"];
   	  [self getBounds:&plotRect];
	  j=(int)(plotRect.size.width/mo);
	  i=(int)(plotRect.size.height/mo);
	  if (i*j*j < AL) {
	    for (k=0; k<moTiles; k++) for (l=0; l<moTiles; l++) {
	      manyrect(k,l,i,j,j,3);
	      [window flushWindow];
	      }
	    smallgrid(i,j);
	    } else printf("volume = %6d\n",i*j*j);
	  }

        if (drawSalami==YES) {
	  mo=(double)moTiles;
	  [window setTitle: "slices of a 5x5x5 cube"];
	  for (i=0; i<5; i++) for (j=0; j<5; j++) 
	    for (ii=0; ii<25; ii++) for (jj=0; jj<25; jj++) 
	      videodot(i*25+ii,j*25+jj,arr1[ii+25*(jj+25*(i+5*j))]);
	  PSsetgray(NX_BLACK); 
	  PSsetlinewidth(0.1);
	  for (i=0; i<6; i++) {
	  PSmoveto((double)(50*i),0.0); PSlineto((double)(50*i),250.0);}
	  for (i=0; i<6; i++) {
	  PSmoveto(0.0,(double)(50*i)); PSlineto(250.0,(double)(50*i));}
	  PSstroke();
	  }

        if (drawMeanPH==YES) {
	  pdiff(100,6);
	  if (YES) {
	     PSsetlinewidth(0.0);
	     PSsetgray(NX_BLACK);
	     linegrid(10);
	     PSstroke();
	     }
	}

        if (drawEMeanPH==YES) {
	     PSsetlinewidth(0.0);
	     PSsetgray(NX_BLACK);
	     linegrid(10);
	     PSstroke();
	n=100;
	en=1.0/((double)(n));
	for (i=0; i<=n; i++) for (j=0; i+j<=n; j++) {
	  pp[0]=((double)(i))*en;
	  pp[1]=((double)(j))*en;
	  pp[2]=1.0-pp[0]-pp[1];
	  ranpatch(pp,16,16,16);
	  for (l=0; l<repEMFg; l++) cuonegen(arr1,brr2,16,16,16);
	  for (k=0; k<KK; k++) flf[k]=(double)lf[k];
	  for (k=0, s=0.0; k<KK; k++) s+=flf[k];
	  for (k=0; k<KK; k++) qq[k]=pp[k]-flf[k]/s;
	  for (k=0, t=0.0; k<KK; k++) t+=qq[k]*qq[k];
	  videotrdot(pp,sqco(t,i));
	  }
	}

        if (drawPlane==YES) {
   switch (dn) {
     case 2: 
	[window setTitle: "von Neumann (2,1) Plane Evolution"];
	drawsquare();
	break;
     case 3: 
	[window setTitle: "3-D Solid Tetrahedron Evolution"];
	drawcube();
	drawgrid();
	break;
     default: [window setTitle: "VN3D22 Plane Evolution"];break;}
	}

        if (drawMosaic==YES) {
   switch (dn) {
     case 2: 
	[window setTitle: "von Neumann (2,1) manysquare Plane Evolution"];
	for (i=0; i<4; i++) for (j=0; j<4; j++) {
	  manysquare(i,j);
	  onegen(AL);
	  [window flushWindow];
	  }
	drawgrid();
	break;
     default: [window setTitle: "VN3D22 Plane Evolution"];break;}
	}

	return self;
}

- sizeTo:(NXCoord)width :(NXCoord)height
/*
 * Ensures that whenever the VN3DView is resized, the origin of its 
 * coordinate system is repositioned to the center of its area.
 */
{
//	  [self setFlipped: YES];
	[super sizeTo: width: height];
	if (drawview==4||drawview==6||drawview==7||drawview>=10) 
		{[self setDrawOrigin:0 :0]; return self;}
	else
		[self setDrawOrigin: -floor(width/2) : -floor(height/2)]; 
	return self;
}

- mouseDown:(NXEvent *) theEvent
/*
 * Responds to a message the system sends whenever the user presses the mouse
 * button when the cursor is over the VN3DView.  The VN3DView changes the 
 * cursor to a cross-hairs image and then starts asking for mouse-dragged or 
 * mouse-up events.  As it receives mouse-dragged events, the VN3DView updates 
 * the readOut text Cell with the cursor's coordinates.  If the user releases 
 * the mouse button while the cursor is over the VN3DView, the VN3DView 
 * registers the point and then sends a message to its delegate notifying it 
 * of the new point.
 */
{
	int	looping = YES, oldMask;
	NXPoint	aPoint;
	NXRect	plotRect;

      /*   double  x, y,  eks, wye; */
	 double ex, wy;
	 int k;

	[crossCursor set];
	oldMask = [window addToEventMask:NX_MOUSEDRAGGEDMASK];
 				[window setDocEdited:YES];
		                aPoint = theEvent->location;
				aPoint.y = aPoint.y;
		                [self convertPoint:&aPoint fromView:nil];
                                ex=aPoint.x;
                                wy=aPoint.y;
				k=nearest(ex,wy);

//       if (theEvent->data.mouse.click==3) {} else 
//       if (theEvent->data.mouse.click==2) {} else 
	{
	[self lockFocus];
//         PSsetinstance(YES);
	do {
// 		PSnewinstance();
		aPoint = theEvent->location;
		[self convertPoint:&aPoint fromView:nil];
		aPoint.y= (plotRect.size.height-1)-aPoint.y;
                ex=aPoint.x;
                wy=aPoint.y;
				PSsetgray(NX_BLACK);
		[window flushWindow];
		if (theEvent->type == NX_MOUSEUP) {
	/* on mouse-up, register point, inform delegate, and clean up state */
			[window flushWindow];
			looping = NO;
		}	
   	} while (looping && (theEvent = 
		[NXApp getNextEvent:NX_MOUSEUPMASK|NX_MOUSEDRAGGEDMASK]));
	[self unlockFocus];

	}

	[window setEventMask:oldMask];
	[NXArrow set];
	return self;
}

@end

/* end VN3DView.m */
