
/* HEXL31.C					*/
/* Two-Dimensional Hexagonal Life		*/
/* with three states per cell			*/
/*						*/
/* Reference:					*/
/*						*/
/* Harold V. McIntosh, 16 May 1987		*/
/* 7 July 1987 - separate rules for each state	*/
/* 8 July 1988 - binary, 3 neighborhood classes	*/
/* 9 July 1988 - hexagonal lattice 		*/

# define COLGRAF     4  /* graph resolution			*/
# define T80X25      3  /* text  resolution			*/
# define WHCYMAG     1  /* color quad for normal screen		*/
# define YELREGR     2  /* color quad for alternative screen	*/
# define AL	    80  /* edge of square of evolution		*/
# define KK	     3  /* number of states per cell		*/
# define BD	     7	/* number of cells in a neighborhood	*/
# define BE	  BD+1	/* number of Bernstein coefficients	*/
# define GK       2187	/* KK^BD= total number of neighborhoods	*/
# define GH      GK/KK	/* GK/KK= 1/KKth no. of neighborhoods	*/
# define NC	    92  /* number of symmetry classes w/o ctr	*/
# define TS	 NC*KK  /* number of symmetry classes w/ cntr	*/
# define NRR	     3  /* row coordinate for neighborhood disp	*/
# define NRC	    25	/* column coordinate for neighborhood d	*/		
# define TRR	     2  /* row coordinate for totalistic rule	*/
# define TRC	     0	/* column coordinate for t'listic  rule	*/		
# define AUR	    64	/* row (from bottom) for planar array	*/
# define AUC	    60	/* column for planar array		*/
# define BER	    56	/* row (from bottom) for Bernstein p.	*/
# define BEC	   219  /* column for Bernstein Polynomial	*/
# define PLR	    60  /* row for planar array			*/
# define PLC	     0  /* column for planar array		*/

# define lcell(i,j) arr1[li+i][lj-(j)]=1

char    trule[TS];	/* evolutionary rule */
char    arule[TS];	/* auxiliary rule flags */
int     li, lj;		/* cursor in cell array */
int     nr[NC];		/* nbrhd representative of symm class */
int     rs[TS];		/* rule use statistics */
int     gc[GK];		/* Golay code key */
int     lf[KK];		/* state count during evolution */
int     sc[NC][KK];	/* state count by symmetry class */
int     wc[NC];		/* size (weight) of symm class */
double  bp[NC];		/* Bernstein coefficients */

main() {
char c;
int  i, j, ln;
int  arr1[AL][AL];

    videomode(COLGRAF);
    videopalette(YELREGR);

    li=AL/2; lj=AL/2;

    for (i=0; i<KK; i++) lf[i]=0;
    for (i=0; i<TS; i++) rs[i]=0;
    for (i=0; i<TS; i++) arule[i]=' ';

  videocursor(0,18,0);
  golta();
  for (j=1; j<KK; j++) for (i=0; i<GH; i++) gc[i+j*GH]=gc[i]+j*NC;
  for (i=0; i<NC; i++) bp[i]=((double)wc[i]);

    conlif();

    hscrrul();
    wscrrul();
    rmenu();
    frame(3);
    rancen(arr1);
    screen(arr1);
    frame(3);

    while (0<1) {					/* set up one run */
    rmenu();
    hscrrul();
    wscrrul();
    videocursor(0,0,0);
    videoputc('?',1);
    c=kbdin();
    videocursor(0,0,0);
    videoputc(c,1);
    if (c=='q') break;
    switch (c) {
	case 'g':					/* one cycle of evolution */
	    frame (1);
	    onegen(arr1);
	    screen(arr1);
            trdot((double)lf[0],(double)lf[1],(double)lf[2],3);
	    break;
        case ':':				/* multiple cycles of evolution */
	    i=0;
	    rstat();
	    frame (1);
/*            trgrid(10,3); */
            pdiff(87);
            do {
	      i++;
	      videocursor(0,8,0);
	      printf("%5d",i);
	      onegen(arr1);
	      screen(arr1);
              trdot((double)lf[0],(double)lf[1],(double)lf[2],3);
	      rstat();
	      } while (!kbdst());
            break;
    case 'M':					/* record rule usage */
      frame (1);
      videoscroll(18,0,24,36,0,0);
      for (ln=0; ln<40; ln++) {
        for (i=0; i<TS; i++) rs[i]=0;
        videocursor(0,8,0);
        printf("%5d",ln+1);
        onegen(arr1);
        screen(arr1);
        rinci(40-ln);
        if (kbdst()) {kbdin(); break;};
	};
      break;
	case 's':
	    rstat();
	    break;
	case 't':					/* probability plot */
/*            trgrid(10,3); */
            pdiff(91);
	    break;
        case 'r':					/* edit rule */	
	    edrule();
	    videocursor(0,4,0);
	    break;
	case 'l':					/* edit cell array */
	    edline(arr1);
	    break;
/*        case '#':					/* read stored rule */
/*	    xmenu(NX);
/*	    xtoasc(lim(1,numin(0),NX)-1);
/*	    rmenu();
/*            break;
/*	case 'D':					/* run through samples */
/*	    for (i=0; i<NX; i++) {
/*	      xmenu(i);
/*	      xtoasc(i);
/*	      ranlin();
/*	      screen(arr1);
/*	      };
/*	    videomode(T80X25);
/*	    rmenu();
/*	    break;
/*	case 'd':					/* de Bruijn diagram */
/*	    edijn();
/*	    rmenu();
/*	    lmenu();
/*	    break; */
       case 'u':				/* sparse init arry */
	    zarry(arr1);
	    arr1[AL/4][AL/4]=1;
	    arr1[AL/2+1][AL/2]=1;
	    arr1[AL/2-1][AL/2]=1;
	    arr1[AL/2][AL/2+1]=1;
	    arr1[AL/2][AL/2-1]=1;
	    screen(arr1);
	    break;
	case 'v':					/* Golay's Life */
	    conlif();
	    break;
	case 'w':					/* random rule */
	    for (i=0, j=0; j<TS; j++) {
	      if (i==0) i=rand();
	      if (arule[j]==' ') {
		trule[j]='0'+i%(KK+2);
		if (trule[j]=='0'+KK) trule[j]='0';
		if (trule[j]=='0'+KK+1) trule[j]='0';
		i/=(KK+2);
		};
	      };
	    rufreq(1);
	    break;
	case 'X':					/* random rule */
	    ranrul();
	    rufreq(1);
	    break;
	case 'x':					/* random rule */
	    for (i=0, j=0; j<TS; j++) {
	      if (i==0) i=rand();
	      if (arule[j]==' ') {trule[j]='0'+i%KK; i/=KK;};
	      };
	    rufreq(1);
	    break;
	case 'y':					/* random line */
	    rancen(arr1);
	    screen(arr1);
	    break;
	case 'Y':					/* random line */
	    ranarr(arr1);
	    screen(arr1);
	    break;
	case 'z':
	    videomode(COLGRAF);
	    videopalette(YELREGR);
	    break;
/*	case 'U':					/* push rule */
/*	    if (rulptr<10) rulptr++;
/*	    for (i=0; i<KK; i++) {
/*	    rulstk[rulptr][i]=ascrule[i];
/*	    };
/*	    xmenu(rulptr);
/*	    break;
/*	case 'V':					/* pop rule */
/*	    for (i=0; i<KK; i++) {
/*	    ascrule[i]=rulstk[rulptr][i];
/*	    };
/*	    if (rulptr>0) rulptr--;
/*	    rmenu();
/*	    break;
/*	case 'G':					/* fetch rule */
/*	    for (i=0; i<KK; i++) {
/*	    ascrule[i]=rulstk[rulptr][i];
/*	    };
/*	    rmenu();
/*	    break;
/*	case '=':
/*	    for (i=1; i<8;  i++) {
/*	    for (j=0; j<SL; j++) arr1[SL*i+j]=arr1[j];};
/*	    lmenu();
/*	    break;
/*	case '~':
/*	    for (i=1; i<16;  i++) {
/*	    for (j=0; j<20; j++) arr1[20*i+j]=arr1[j];};
/*	    lmenu();
/*	    break; */
        default: break;
        }; /* end switch */
    };	   /* end while */
  videomode(T80X25);
}	   /* end main */

/* calculate one generation of evolution */
onegen(arr1) int  arr1[AL][AL]; {int i0, i1, gn, arr2[AL][AL];

/* count neighbors of main square */
frame(1); /* phase to count neighbors */

for (i0=1; i0<AL-1; i0++) {
for (i1=1; i1<AL-1; i1++) {
  gn=arr1[i0-1][i1];
  gn=KK*gn+arr1[i0-1][i1-1];
  gn=KK*gn+arr1[i0][i1-1];
  gn=KK*gn+arr1[i0+1][i1];
  gn=KK*gn+arr1[i0+1][i1+1];
  gn=KK*gn+arr1[i0][i1+1];
  gn+=arr1[i0][i1]*GH;
  gn=gc[gn];
  arr2[i0][i1]=gn;
  rs[gn]++;
  };};

/* count neighbors for top edge (i1=0) */

for (i0=1; i0<AL-1; i0++) {
  gn=arr1[i0-1][0];
  gn=KK*gn+arr1[i0-1][AL-1];
  gn=KK*gn+arr1[i0][AL-1];
  gn=KK*gn+arr1[i0+1][0];
  gn=KK*gn+arr1[i0+1][1];
  gn=KK*gn+arr1[i0][1];
  gn+=arr1[i0][0]*GH;
  gn=gc[gn];
  arr2[i0][0]=gn;
  rs[gn]++;
  };

/* count neighbors for bottom edge (i1=AL-1) */

for (i0=1; i0<AL-1; i0++) {
  gn=arr1[i0-1][AL-1];
  gn=KK*gn+arr1[i0-1][AL-2];
  gn=KK*gn+arr1[i0][AL-2];
  gn=KK*gn+arr1[i0+1][AL-1];
  gn=KK*gn+arr1[i0+1][0];
  gn=KK*gn+arr1[i0][0];
  gn+=arr1[i0][AL-1]*GH;
  gn=gc[gn];
  arr2[i0][AL-1]=gn;
  rs[gn]++;
  };

/* count neighbors for left edge (i0=0) */

for (i1=1; i1<AL-1; i1++) {
  gn=arr1[AL-1][i1];
  gn=KK*gn+arr1[AL-1][i1-1];
  gn=KK*gn+arr1[0][i1-1];
  gn=KK*gn+arr1[1][i1];
  gn=KK*gn+arr1[1][i1+1];
  gn=KK*gn+arr1[0][i1+1];
  gn+=arr1[0][i1]*GH;
  gn=gc[gn];
  arr2[0][i1]=gn;
  rs[gn]++;
  };

/* count neighbors of right edge (i0=AL-1) */

for (i1=1; i1<AL-1; i1++) {
  gn=arr1[AL-2][i1];
  gn=KK*gn+arr1[AL-2][i1-1];
  gn=KK*gn+arr1[AL-1][i1-1];
  gn=KK*gn+arr1[0][i1];
  gn=KK*gn+arr1[0][i1+1];
  gn=KK*gn+arr1[AL-1][i1+1];
  gn+=arr1[AL-1][i1]*GH;
  gn=gc[gn];
  arr2[AL-1][i1]=gn;
  rs[gn]++;
  };

/* count neighbors for upper left hand corner */

  gn=arr1[AL-1][0];
  gn=KK*gn+arr1[AL-1][AL-1];
  gn=KK*gn+arr1[0][AL-1];
  gn=KK*gn+arr1[0][1];
  gn=KK*gn+arr1[1][1];
  gn=KK*gn+arr1[0][1];
  gn+=arr1[0][0]*GH;
  gn=gc[gn];
  arr2[0][0]=gn;
  rs[gn]++;

/* count neighbors for upper right hand corner */

  gn=arr1[AL-2][0];
  gn=KK*gn+arr1[AL-2][AL-1];
  gn=KK*gn+arr1[AL-1][AL-1];
  gn=KK*gn+arr1[0][0];
  gn=KK*gn+arr1[0][1];
  gn=KK*gn+arr1[AL-1][1];
  gn+=arr1[AL-1][0]*GH;
  gn=gc[gn];
  arr2[AL-1][0]=gn;
  rs[gn]++;

/* count neighbors for bottom left hand corner */

  gn=arr1[AL-1][AL-1];
  gn=KK*gn+arr1[AL-1][AL-2];
  gn=KK*gn+arr1[0][AL-2];
  gn=KK*gn+arr1[1][AL-1];
  gn=KK*gn+arr1[1][0];
  gn=KK*gn+arr1[0][0];
  gn+=arr1[0][AL-1]*GH;
  gn=gc[gn];
  arr2[0][AL-1]=gn;
  rs[gn]++;

/* count neighbors for bottom right hand corner */

  gn=arr1[AL-2][AL-1];
  gn=KK*gn+arr1[AL-2][AL-2];
  gn=KK*gn+arr1[AL-1][AL-2];
  gn=KK*gn+arr1[0][AL-1];
  gn=KK*gn+arr1[0][0];
  gn=KK*gn+arr1[AL-1][0];
  gn+=arr1[AL-1][AL-1]*GH;
  gn=gc[gn];
  arr2[AL-1][AL-1]=gn;
  rs[gn]++;

frame(2); /* phase to update cells */
for (i0=0; i0<KK; i0++) lf[i0]=0;
for (i0=0; i0<AL; i0++) {
for (i1=0; i1<AL; i1++) {
  arr1[i0][i1]=trule[arr2[i0][i1]]-'0';
  lf[arr1[i0][i1]]++;
};};
frame(3); /* phase to show results */
}	  /* end onegen */
    
/* display a screen of evolution */
screen(arr1) int  arr1[AL][AL]; {int i0, i1;
  for (i1=0; i1<AL; i1++) {
  for (i0=0; i0<AL; i0++) {
    adot(i0,i1,arr1[i0][i1]);
    };};
}

/* display a frame around the cell array */
frame(l) int l; {int j;
for (j=0; j<AL; j++) {
  adot(0,j,l);
  adot(AL-1,j,l);
  adot(j,0,l);
  adot(j,AL-1,l);
  };
}

/* make a frame for a graph 		 */
/* (x,y) = lower left corner; e.g. (0,0) */
/* n     = dimension of frame            */
gfram(x,y,n) int x, y, n; {int i;
for (i=0; i<=n; i++) {
  videodot(199-y-i,x,0);
  videodot(199-y-i,x+n,0);
  videodot(199-n-y,x+i,0);
  videodot(199-y,x+i,0);
  };
for (i=0; i<=n; i+=10) {
  videodot(199-y-i,x,3);
  videodot(199-y-i,x+n,3);
  videodot(199-n-y,x+i,3);
  videodot(199-y,x+i,3);
  };
}

/* put a diagonal in a graph */
gdiag(x,y,n) int x, y, n; {int i;
for (i=0; 2*i<n; i++) videodot(199-y-2*i,x+2*i,3);
}

/* graph incidence of rule usage */
/* beware limited count capacity  */
rinci(j) int j; {int i; for (i=0; i<TS; i++) sdot(i,j,rs[i]==0?1:3);}

/* display the evolutionary rule as a colored line */
rulgr(i) int i; {int j; for (j=0; j<TS; j++) rdot(j,i,trule[j]-'0');}

/* display the auxiliary rule as a colored line */
auxgr(i) int i; {int j; for (j=0; j<TS; j++) rdot(j,i,arule[j]==' '?1:2);}

/* graph statistics of rule usage */
/* beware limited count capacity  */
/* large counts look negative     */
rstat() {int i, j, m, n; double r, s, t;
for (i=0, s=0.0; i<TS; i++) s+=(double)rs[i];
if  (s<1.0) {for (i=0; i<TS; i++) rs[i]=0; return;};
for (i=0; i<TS; i++) {
  r=((double)rs[i]);
  t=(666.7*r)/(r+s);
  n=(int)t;
  sdot(i,33,0);
  m=33; if (n>33) {n=33; m=34;};
  if (n>0) for (j=0; j<n; j++) sdot(i,j,1);
  for (j=n; j<m; j++) sdot(i,j,2);
  rs[i]=0;
  };
}

/* determine the number of ancestors of each state in trule */
rufreq(l) int l; {int i;
for (i=0; i<KK; i++) lf[i]=0;
for (i=0; i<TS; i++) lf[trule[i]-'0']++;
trdot((double)lf[0],(double)lf[1],(double)lf[2],l);
}

/* generate the Golay code table */
golta() {int i, j, k;
int i0, i1, i2, i3, i4, i5;
k=0;
for (i5=0; i5<KK; i5++) {
for (i4=0; i4<KK; i4++) {
for (i3=0; i3<KK; i3++) {
for (i2=0; i2<KK; i2++) {
for (i1=0; i1<KK; i1++) {
for (i0=0; i0<KK; i0++) {
i=binq(i0,i1,i2,i3,i4,i5);
j=binq(i1,i2,i3,i4,i5,i0);
if (j<i) i=j;
j=binq(i2,i3,i4,i5,i0,i1);
if (j<i) i=j;
j=binq(i3,i4,i5,i0,i1,i2);
if (j<i) i=j;
j=binq(i4,i5,i0,i1,i2,i3);
if (j<i) i=j;
j=binq(i5,i0,i1,i2,i3,i4);
if (j<i) i=j;
j=binq(i5,i4,i3,i2,i1,i0);
if (j<i) i=j;
j=binq(i4,i3,i2,i1,i0,i5);
if (j<i) i=j;
j=binq(i3,i2,i1,i0,i5,i4);
if (j<i) i=j;
j=binq(i2,i1,i0,i5,i4,i3);
if (j<i) i=j;
j=binq(i1,i0,i5,i4,i3,i2);
if (j<i) i=j;
j=binq(i0,i5,i4,i3,i2,i1);
if (j<i) i=j;
j=binq(i0,i1,i2,i3,i4,i5);
if (i==j) {
  gc[i]=k;
  sc[k][i0]++; sc[k][i1]++; sc[k][i2]++;
  sc[k][i3]++; sc[k][i4]++; sc[k][i5]++;
  wc[k]=1;
  nr[k]=j;
  k++;
} else {
  gc[j]=gc[i]; wc[gc[i]]++;
  };
/*
printf("  %2X, gc[%3d]=%3d, k=%3d, (%d)\n",j,j,i,k,i0+i1+i2+i3+i4+i5);
*/
};};};};};};
/* printf("no. cl. = %3d",k); */
}

/* calculate binary equivalent */
int binq(i0,i1,i2,i3,i4,i5)
int i0, i1, i2, i3, i4, i5; {
return i0+KK*(i1+KK*(i2+KK*(i3+KK*(i4+KK*i5))));}

/* calculate Golay class of a cell */
int golcl(i0,i1,arr1) int  i0, i1, arr1[AL][AL]; {int gn;
gn=arr1[i0-1][i1];
gn=KK*gn+arr1[i0-1][i1-1];
gn=KK*gn+arr1[i0][i1-1];
gn=KK*gn+arr1[i0+1][i1];
gn=KK*gn+arr1[i0+1][i1+1];
gn=KK*gn+arr1[i0][i1+1];
gn+=arr1[i0][i1]*GH;
return gc[gn];
}

/* set up a triangular gridwork   */
/* n = number of ruling intervals */
/* l = color of lines		  */
trgrid(n,l) int n, l; {
int i, j;
double a, b, c, en;
if(n==0) return;
en=((double)(n));
for (i=0; i<=n;   i++) {
for (j=0; i+j<=n; j++) {
  a=((double)(i));
  b=((double)(j));
  c=en-a-b;
  trdot(a,b,c,l);
  };};
}

/* evaluate the kth component Bernstein polynomial at point p */
double bern(k,x,y,z) int k; double x, y, z; {
int    i, j;
double s, q[BE][BE], bpow();
for (i=0; i<BE; i++) for (j=0; j<BE; j++) 
  if (i+j<BE) q[i][j]=bpow(i,x)*bpow(j,y)*bpow(BD-i-j,z);
for (i=0, s=0.0; i<NC; i++) {
  if (trule[i]-'0'==k) s+=bp[i]*q[sc[i][0]+1][sc[i][1]];
  if (trule[i+NC]-'0'==k) s+=bp[i]*q[sc[i][0]][sc[i][1]+1];
  if (trule[i+2*NC]-'0'==k) s+=bp[i]*q[sc[i][0]][sc[i][1]];
  };
return s;
}

/* calculate a power */
/* using this function is bad policy */
/* because of the redundancy involved */
double bpow(k,x) int k; double x; {int i; double s;
if (k==0) return 1.0;
if (k==1) return x;
for (s=x, i=1; i<k; i++) s*=x;
return s;
}

/* graph the probability differential for state l over a triangle */
int pdiff(n) int n; {int i, j; double a, b, c, en, t, sqsu();
if (n<8) return;
pdiff(n/3);
en=((double)(n));
for (i=0; i<=n; i++) {
for (j=0; i+j<=n; j++) {
  a=((double)(i));
  b=((double)(j));
  c=en-a-b;
  t=sqsu(a,b,c);
  trdot(a,b,c,sqco(t,i));
  if (kbdst()) {kbdin(); return;};
  };};
}

/* calculate the squaresum of the three polynomials at a point */
double sqsu(x,y,z) double x, y, z; {double s, u, v, w, bern();
s=x+y+z; x=x/s, y=y/s; z=z/s;
u=bern(0,x,y,z)-x;
v=bern(1,x,y,z)-y;
w=bern(2,x,y,z)-z;
return (u*u+v*v+w*w);
}

/* generate squaresum compatible contour values */
int sqco(t,i) int i; double t; {int l;
if (t<0.0075) l=0; else {
if (t<0.0750) {if (i%2) l=0; else l=3;} else {
if (t<0.1250) l=3; else {
if (t<0.2500) {if (i%2) l=3; else l=2;} else {
if (t<0.5000) l=2; else {
if (t<0.7500) {if (i%2) l=2; else l=1;} else {
if (t<1.0000) l=1; else {
	      if (i%2) l=1; else l=0;
   }}}}}}}
return l;
}

/* adot(i,j,l) - place dot on automaton screen at (i,j) in color l */
adot(i,j,l) int i, j, l; {videodot(199-PLR-(AL-1-j),PLC+(AL-1-j)+2*i,l);}

/* bdot(i,j,l) - place dot on Bernstein screen at (i,j) in color l */
bdot(i,j,l) int i, j, l; {videodot(199-j,i,l);}

/* rdot(i,j,l) - place dot on the rule line at (i,j) in color l */
rdot(i,j,l) int i, j, l; {videodot(199-150-j,20+i,l);}

/* sdot(i,j,l) - place dot on statistics screen at (i,j) in color l */
sdot(i,j,l) int i, j, l; {videodot(199-10-j,10+i,l);}

/* trdot(x,y,z,l) - plot a single point on a triangular grid */
trdot(x,y,z,l) double x, y, z; int l; {double s;
s=199.0/(x+y+z); y=y*s; z=z*s;
videodot(199-BER-((int)(0.433*z)),BEC+((int)(0.250*(y+y+z))),l);
}

/* clear the cell array */
zarry(arr1) int  arr1[AL][AL]; {int i0, i1;
for (i0=0; i0<AL; i0++) {
for (i1=0; i1<AL; i1++) {
  arr1[i0][i1]=0;
  };};
}

/* display rule menu */
rmenu() {
    videocursor(0,0,33);
    scrstr("HEXL31");
    rulgr(2);
    auxgr(0);
/*    scrrul();  */
    videocursor(0,TRR,TRC);
    }

/* display rule number by sum in color */
scrrul() {int i, j; 
for (j=0; j<KK; j++) {
for (i=0; i<NC; i++) {
  videocursor(0,TRR+j,TRC+8+i);
  resch(j*NC+i);
  };};
}

/* display rule number as a hexadecimal */
hscrrul() {int i; long int k;
  for (i=0, k=0; i<8; i++) k=KK*k+trule[i]-'0';
  videocursor(0,0,9);
  printf("        ");
  videocursor(0,0,9);
  printf("%7.7lX",k);
  videocursor(0,0,0);
  }

/* display Wolfram rule number of cross section */
wscrrul() {int i;
  i=trule[25]-'0';
  i=KK*i+trule[21]-'0';
  i=KK*i+trule[10]-'0';
  i=KK*i+trule[2] -'0';
  i=KK*i+trule[21]-'0';
  i=KK*i+trule[17]-'0';
  i=KK*i+trule[2] -'0';
  i=KK*i+trule[0] -'0';
  videocursor(0,0,18);
  printf("     ");
  videocursor(0,0,18);
  printf("[%2d]",i);
  videocursor(0,0,0);
  }

/* edit the rule */
edrule() {char c; int i, j;
  j=0; i=0;
  while (0<1) {
    shon(j,nr[i]);
    videocursor(0,TRR+1,TRC+12);
    videoputc(trule[j*NC+i],3);
    videocursor(0,TRR+1,TRC+12);
    rulgr(2);
    auxgr(0);
    rdot(j*NC+i,1,3);
    c=kbdin();
      if (c=='\015') break;				/* carriage return */
      switch (c) {
        case '0': case '1': case '2':			/* state = 0,1,2 */
          trule[j*NC+i]=c;
	  rdot(j*NC+i,1,0);
          i++; if (i==NC) {i=0; j++; if (j==KK) j=0;};
	  break;
        case ' ': case '\315':				/* space = advance */
	  rdot(j*NC+i,1,0);
          i++; if (i==NC) {i=0; j++; if (j==KK) j=0;};
	  break;
        case '\010': case '\313':			/* backspace */
	  rdot(j*NC+i,1,0);
          if (i>0) i--; else if(j>0) {i=NC-1; j--;};
	  break;
	case '<':					/* cursor to front */
	  rdot(j*NC+i,1,0);
	  i=0; j=0;
	  break;
	case '*':					/* cursor to center */
	  rdot(j*NC+i,1,0);
	  i=0; j=1;
	  break;
	case '>':					/* cursor to end */
	  rdot(j*NC+i,1,0);
	  i=NC-1; j=KK-1;
	  break;
	case '\322':					/* insert */
	  arule[j*NC+i]='+';
	  rdot(j*NC+i,1,0);
          i++; if (i==NC) {i=0; j++; if (j==KK) j=0;};
	  break;
	case '\323':					/* delete */
	  arule[j*NC+i]=' ';
	  rdot(j*NC+i,1,0);
          i++; if (i==NC) {i=0; j++; if (j==KK) j=0;};
	  break;
	case '\013': case '\310':			/* up arrow */
	  rdot(j*NC+i,1,0);
          if (j==0) j=KK-1; else j--;
	  break;
	case '\012': case '\320':			/* down arrow */
	  rdot(j*NC+i,1,0);
          j++; if (j==KK) j=0;
	  break;
	default:
	  break;
          };
    };
eran();
rdot(j*NC+i,1,0);
videocursor(0,TRR+1,TRC+12);
videoputc(' ',3);
videocursor(0,TRR+1,TRC+12);
}

/* restore character in edrule */
resch(i) int i; {videoputc(trule[i],(arule[i]==' ')?1:2);}

/* show neighborhood in edrule */
/* i = center cell */
/* j = Golay code  */
shon(i,j) int i, j; {int k;
  videocursor(0,NRR,NRC);
  videoputc('0'+i,1);
  videocursor(0,NRR,NRC+2);
  k=j; videoputc('0'+k%KK,1);
  videocursor(0,NRR-1,NRC+1);
  k=k/KK; videoputc('0'+k%KK,1);
  videocursor(0,NRR-1,NRC-1);
  k=k/KK; videoputc('0'+k%KK,1);
  videocursor(0,NRR,NRC-2);
  k=k/KK; videoputc('0'+k%KK,1);
  videocursor(0,NRR+1,NRC-1);
  k=k/KK; videoputc('0'+k%KK,1);
  videocursor(0,NRR+1,NRC+1);
  videoputc('0'+k/KK,1);
}

/* erase the neighborhood in edrule */
eran() {
  videocursor(0,NRR,NRC); videoputc(' ',1);
  videocursor(0,NRR,NRC+2); videoputc(' ',1);
  videocursor(0,NRR-1,NRC+1); videoputc(' ',1);
  videocursor(0,NRR-1,NRC-1); videoputc(' ',1);
  videocursor(0,NRR,NRC-2); videoputc(' ',1);
  videocursor(0,NRR+1,NRC-1); videoputc(' ',1);
  videocursor(0,NRR+1,NRC+1); videoputc(' ',1);
}

/* edit the plane of cells */
edline(arr1) int arr1[AL][AL]; {
char c;
int  i, j, k, arr2[AL][AL];

for (i=0; i<AL; i++) for (j=0; j<AL; j++) arr2[i][j]=0;

videoscroll(18,0,24,36,0,0);
videocursor(0,19,0);
scrstr("insert states with 0,1,.");
videocursor(0,20,0);
scrstr("move cursor using arrows");
videocursor(0,21,0);
scrstr("NSEW R CGO fgmMxz *<>^_");
videocursor(0,22,0);
scrstr("carriage return exits");

while (0<1) {
  adot(li,lj,3);
  videocursor(0,0,1);
  videoputc('?',1);
  c=kbdin();
  videocursor(0,0,1);
  videoputc(c,1);
  if (c == '\015') break;
  switch (c) {
    case '0': case '1':	case '2':	/* enter state */
      i=c-'0';
      arr1[li][lj]=i;
      adot(li,lj,i);
      break;
    case '\012': case '\320':		/* down    */
      adot(li,lj,arr1[li][lj]);
      lj++; if (lj==AL) lj=0;
      break;
    case '\013': case '\310':		/* up      */
      adot(li,lj,arr1[li][lj]);
      if (lj==0) lj=AL; lj--;
      break;
    case '\014': case '\315': case ' ':	/* forward */
      adot(li,lj,arr1[li][lj]);
      li++; if (li==AL) li=0;
      break;
    case '\010': case '\313':		/* back    */
      adot(li,lj,arr1[li][lj]);
      if (li==0) li=AL; li--;
      break;
    case '<':				/* left margin  */
      adot(li,lj,arr1[li][lj]);
      li=0;
      break;
    case '>':				/* right margin */
      adot(li,lj,arr1[li][lj]);
      li=AL-1;
      break;
    case '^':				/* top margin   */
      adot(li,lj,arr1[li][lj]);
      lj=0;
      break;
   case '_':				/* bottom margin  */
      adot(li,lj,arr1[li][lj]);
      lj=AL-1;
      break;
   case '*':				/* center cursor */
      adot(li,lj,arr1[li][lj]);
      li=AL/2; 
      lj=AL/2;
      break;
    case 'a':				/* insert glider */
      lcell(0,2);
      lcell(1,1); lcell(1,3);
      lcell(3,2);
      lcell(4,0); lcell(4,1);
      screen(arr1);
      break; 
     case 'f':				/* generate a frame */
      frame (2);
      break;
    case 'g':				/* one cycle of evolution */
      frame (1);
      onegen(arr1);
      screen(arr1);
      break;
 case 'm':				/* mark neighborhood */
      arule[golcl(li,lj,arr1)]='+';
      videocursor(0,TRR,TRC+8);
      hscrrul();
      break;
    case 'M':				/* mark neighborhood over a region */
      if (li<10||li>AL-11) break;
      if (lj<10||lj>AL-11) break;
      for (i=li-10; i<li+10; i++) {
      for (j=lj-10; j<lj+10; j++) {
        arule[golcl(i,j,arr1)]='+';
        };};
      videocursor(0,TRR,TRC+8);
      hscrrul();
      break;
    case 'x':				/* uncommitted transitions */
      for (k=0; k<TS; k++) arule[k]=' ';
      videocursor(0,TRR,TRC+8);
      hscrrul();
      break;
    case 'z':				/* clear array  */
      zarry(arr1);
      screen(arr1);
      break;
    case 'N':				/* rotate array up */
      for (i=0; i<AL; i++) {
        k=arr1[i][0]; 
        for (j=1; j<AL; j++) arr1[i][j-1]=arr1[i][j];
	arr1[i][AL-1]=k;
	};
      if (lj==0) lj=AL; lj--;
      screen(arr1);
      break;
    case 'S':				/* rotate array down */
      for (i=0; i<AL; i++) {
	k=arr1[i][AL-1];
        for (j=1; j<AL; j++) arr1[i][AL-j]=arr1[i][AL-j-1];
	arr1[i][0]=k;
	};
      lj++; if (lj==AL) lj=0;
      screen(arr1);
      break;
    case 'E':				/* rotate array right */
      for (j=0; j<AL; j++) {
	k=arr1[AL-1][j];
        for (i=1; i<AL; i++) arr1[AL-i][j]=arr1[AL-i-1][j];
	arr1[0][j]=k;
	};
      li++; if (li==AL) li=0;
      screen(arr1);
      break;
    case 'W':				/* rotate array left */
      for (j=0; j<AL; j++) {
	k=arr1[0][j];
        for (i=1; i<AL; i++) arr1[i-1][j]=arr1[i][j];
	arr1[AL-1][j]=k;
	};
      if (li==0) li=AL; li--;
      screen(arr1);
      break;
    case 'R':				/* reflect array in diagonal */
      for (j=1; j<AL; j++) {
      for (i=0; i<j;  i++) {
	k=arr1[i][j];
	arr1[i][j]=arr1[j][i];
        arr1[j][i]=k;
        };};
      k=li; li=lj; lj=k;
      screen(arr1);
      break;
    case 'C':				/* save cell array */
      for (j=0; j<AL; j++) {
      for (i=0; i<AL; i++) {
	arr2[i][j]=arr1[i][j];
	};};
      break;
    case 'G':				/* recover saved cell array */
      for (j=0; j<AL; j++) {
      for (i=0; i<AL; i++) {
	arr1[i][j]=arr2[i][j];
	};};
      screen(arr1);
      break;
    case 'O':				/* or saved cell array */
      for (j=0; j<AL; j++) {
      for (i=0; i<AL; i++) {
	arr1[i][j]|=arr2[i][j];
	};};
      screen(arr1);
      break;
    default:
      break;
      };				/* end switch */
  };					/* end while */
adot(li,lj,arr1[li][lj]);
videoscroll(18,0,24,36,0,0);
videocursor(0,0,1);
videoputc(' ',1);
}				        /* end edline */

/* generate the transitions for Golay's Life */
conlif() {int i;
    for (i=0; i<TS; i++) trule[i]='0';
    trule[2]='1';
    trule[3]='1';
    trule[15]='1';
    trule[16]='1';
}

/* generate random cells in arr1 */
ranarr(arr1) int  arr1[AL][AL]; {int i0, i1, i, c;
i=0;
for (i0=0; i0<AL; i0++) {
for (i1=0; i1<AL; i1++) {
  if (i%8 == 0) c=rand(); i++;
  arr1[i0][i1]=c%KK; c/=KK;
  };};
}

/* generate random cells in the center of arr1 */
rancen(arr1) int  arr1[AL][AL]; {int i0, i1, ia, ib, i, c;
i=0;
for (i0=0; i0<AL; i0++) {
for (i1=0; i1<AL; i1++) {
  arr1[i0][i1]=0;
  };};
ia=AL/3; ib=2*(AL/3);
for (i0=ia; i0<ib; i0++) {
for (i1=ia; i1<ib; i1++) {
  if (i%8 == 0) c=rand(); i++;
  arr1[i0][i1]=c%KK; c/=KK;
  };};
}

/* generate a random rule */
ranrul() {int i0, i, c;
i=0;
for (i0=0; i0<TS; i0++) {
  if (i%8 == 0) c=rand(); i++;
  trule[i0]=c%KK+'0'; c/=KK;
};
}

/* limit j to interval (i,k) */
int lim(i,j,k) int i, j, k;
    {if (i>=j) return i; if (k<=j) return k; return j;}

/* write a string in graphics mode */
scrstr(s) char *s;
  {for (; *s != '\0'; s++) videoputc(*s,1); }

/* keyboard status */
int kbdst() {return(bdos(11) & 0xFF);}

/* direct keyboard input, no echo */
kbdin() {int c;
  if ((c = bdos(7) & 0xFF) == '\0') c = (bdos(7) & 0xFF) | 0x80;
  return(c);
}

/* read number from keyboard */
int numin(n) int n; {char c;
  c=kbdin();
  if (c>='0'&&c<='9') return(numin(10*n+(c-'0'))); else return(n);
}

/* end */
