
/* brui21.c						*/
/* program for hexl21 option 'd'			*/
/* display de Bruijn diagram - 1st, 2nd, & 3rd gen	*/
/* Harold V. McIntosh, 20 February 1988			*/
/* 21 February 1988 - add j0=i0; to Pass 4's [HVM]	*/

/*    Copyright (C) 1987		*/
/*    Copyright (C) 1988		*/
/*    Copyright (C) 1996		*/
/*    Harold V. McIntosh		*/

# define XC    8  	/* maximum cycle length		*/

# import  "nxlcau33.h"
# import  <dpsclient/psops.h>
# import  <appkit/graphics.h>
# include <math.h>

int  ww[QWN][QWN];	/* connection matrix for de Bruijn nucleus  */
int  xx[QWN][QWN];
int  zz[QWN][QWN];	/* distance map for de Bruijn nucleus	    */
int  oc[QWN][QWN];	/* work array for finding de Bruijn nucleus */
int  avl[QWN];		/* TRUE when right bits are both zero	*/
int  avr[QWN];		/* TRUE when left bits are both zero	*/
int  tox[QWN];		/* bit pattern of the top of the node	*/
int  box[QWN];		/* pattern of the bottom of the node	*/

int    qbno;		/* node number in de Bruijn diagram 	*/
int    qnmax;
int    ndst, xdst, nloo, xloo;
int    noloo;		/* not part of any loop			*/
int    unlat;		/* unilaterally connected only		*/
int    nolat;		/* disconnected = different components	*/

int  no, lk;		/* # nodes, links 		*/

double qcntr[QDN][2];

int  aa[KK][KK][KK][KK][KK][KK][KK][KK];
int  bb[KK][KK][KK][KK][KK][KK][KK][KK];

int  bdex(int i0, int i1, int i2, int i3, int i4, int i5);
void cluin();
void dbdraw(int s, int n, int z, int f, int  t);
void edrui(int c);
void pass1(int l);
void pass4(int z[KK][KK][KK][KK][KK][KK][KK][KK]);
void pass7(int l, int t);
void pass7a(int r[3][12], int  *i, int  *j, int  k, int  l, int  s, int  t);
void pass7x(int l, int  s, int  t);
int  tdex(int i0, int i1, int i2);

/* edit the de Bruijn diagram */
void edrui(c) int c; {int i, j;
  for (i=0; i<QWN; i++) for (j=0; j<QWN; j++) ww[i][j]=0;
  switch (c) {
    case 4: printf("(0,0) static:\n");  pass1(0); prnz(aa); break;
    case 5: printf("(0,1) gliders:\n"); pass1(1); prnz(aa); break;
    case 0: printf("(1,0) gliders:\n"); pass1(2); prnz(aa); break;
    case 1: printf("(1,1) gliders:\n"); pass1(3); prnz(aa); break;
    case 'e': printf("(z,z) vanish:\n");  pass1(4); prnz(aa); break;
    case 'f': printf("(z,z) ones  :\n");  pass1(5); prnz(aa); break;
    case 's': videoscroll(1,0,24,39,0,0); mprod(bb,aa,aa); prnz(bb); break;
    case 't': videoscroll(1,0,24,39,0,0); strp(aa); break;
    case 'A': pass7(1,1); qnmax=no; break;
    case 'B': pass7(2,1); qnmax=no; break;
    case 'C': pass7(3,1); qnmax=no; break;
    case 'D': pass7(4,1); qnmax=no; break;
    case 'E': pass7(5,1); qnmax=no; break;
    case 'F': pass7(6,1); qnmax=no; break;
    case 'G': pass7(7,1); qnmax=no; break;
//    case 'H': pass7(8,1); break;
    case 'M': pass7(1,0); qnmax=no; break;
    case 'N': pass7(2,0); qnmax=no; break;
    case 'O': pass7(3,0); qnmax=no; break;
    case 'P': pass7(4,0); qnmax=no; break;
    case 'Q': pass7(5,0); qnmax=no; break;
    case 'R': pass7(6,0); qnmax=no; break;
    case 'S': pass7(7,0); qnmax=no; break;
//    case 'T': pass7(8,0); break;
    default: break;
    };
}

/* THREE31.C */

/*	i5 i6 xx	*/
/*	i4 i0 i1	*/
/*	xx i3 i4	*/


/* Pass 1 marks sequences conforming to some shift */
void pass1(l) int l; {
int  i0, i1, i2, i3, i4, i5, i6;
int  k, m;
zerar(aa);
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++) {
  k=thrule[gc[(((((i0*KK+i1)*KK+i2)*KK+i3)*KK+i4)*KK+i5)*KK+i6]]-'0';
  switch (l) {
    case 0: m=i0; break;
    case 1: m=i1; break;
    case 2: m=i5; break;
    case 3: m=i6; break;
    case 4: m=0;  break;
    case 5: m=1;  break;
    default: break;}
  aa[i0][i3][i4][i5][i1][i2][i0][i6]=k==m?1:0;
  }
}

/*------------------------------------------*/
/* pass 7 - get the loops of length l       */
/*------------------------------------------*/
/*  t - 0 = isolated (0,0); 1 = cyclic      */
/*  t - 2 = Escher (Dirac)		    */
/*  z - array of links	(data) KK^6xKK^6    */
/*      eventually contains a serial number */
/*------------------------------------------*/

void pass7(l,t)
int  l, t; {
int  f, g, k, p, q;
// printf("width %1d.\n",l);
// printf("gen nodes links\n");
for (p=0; p<QWN; p++) for (q=0; q<QWN; q++) oc[p][q]=4;
k=0;
do {
  g=0; k++; no=0; lk=0;
  pass7x(l,0,t);
  for (p=0; p<QWN; p++) for (q=0; q<QWN; q++) {
    f=oc[p][q];
    if (f==7) no++;
    if (f==5||f==6) g=1;
    oc[p][q]=f==7?4:0;
    }
    printf("at pass %3d there are %5d nodes and %5d links \n",k,no,lk);
  } while (g==1);
// printf("list the links\n");
// pass7x(l,3,t);
if (no<QWN) {
  g=4;
  for (p=0; p<QWN; p++) for (q=0; q<QWN; q++) oc[p][q]=(oc[p][q]==4)?g++:0;
//  zerar(bb);
//  prng(1);
  pass7x(l,1,t);
  return;
  }
}

/* pass 7x - display the acceptable strings of length l */
/* z  = array of acceptable tiles	*/
/* l  = width of strip			*/
/* s  =	phase				*/
/* t  = 0 (zero boundary), 1 (periodic)	*/
void pass7x(l,s,t) 
int  l, s, t; {
int  i[6];		/* two-column boundary 		*/
int  j[6];		/* two initial columns		*/
int  r[3][12];		/* strip (max length 12)	*/
int  p, i0, i1, i2, i3, i4, i5;
if (l<1) return;
switch (t) {
  case 0:
    if (l>10) return;
    for (p=0; p<6; p++) i[p]=0;
    for (p=0; p<6; p++) j[p]=0;
      pass7a(r,i,j,0,l,s,t);
    break;
  case 1:
    if (l>XC) return;
    for (i0=0; i0<KK; i0++) {i[0]=i0;
    for (i1=0; i1<KK; i1++) {i[1]=i1;
    for (i2=0; i2<KK; i2++) {i[2]=i2;
    for (i3=0; i3<KK; i3++) {i[3]=i3;
    for (i4=0; i4<KK; i4++) {i[4]=i4;
    for (i5=0; i5<KK; i5++) {i[5]=i5;
      j[0]=i0; j[1]=i1; j[2]=i2; j[3]=i3; j[4]=i4; j[5]=i5;
      pass7a(r,i,j,0,l,s,t);
      }}}}}}
    break;
  }
}

/* pass 7a - recursive step in pass 7	*/
/* r[3][12]  = test strip		*/
/* i[6]  = two-column boundary spot	*/
/* j[6]  = 2-col moving domino spot 	*/
/* k  = position along the strip	*/
/* l  = width of strip			*/
/* s  = phase				*/
/* t  = 0 (zero boundary), 1 (periodic)	*/
void pass7a(r,i,j,k,l,s,t)
  int  *i, *j, k, l, s, t;
  int  r[3][12];
  {
int p, w, x, y, i0, i1, i2;
int jj0, jj1, jj2, jj3, jj4, jj5;
int p0, p1, p2, p3, p4, p5;
int q0, q1, q2, q3, q4, q5;
if (k<l) {
  jj0=j[0]; jj1=j[1]; jj2=j[2]; jj3=j[3]; jj4=j[4]; jj5=j[5];
  for (i0=0; i0<KK; i0++) {r[0][k]=i0;
  for (i1=0; i1<KK; i1++) {r[1][k]=i1;
  for (i2=0; i2<KK; i2++) {r[2][k]=i2;
    if (aa[jj4][jj5][jj1][jj0][i1][i2][jj4][jj3]!=0) {
      j[0]=jj3; j[1]=jj4; j[2]=jj5; j[3]=i0; j[4]=i1; j[5]=i2;
      pass7a(r,i,j,k+1,l,s,t);}
    }}}
  return;}
/* each time a full strip is found, take the following additional steps: */
/* 1) check whether the boundary condition has been satisfied		 */
for (p=0; p<6; p++) if (i[p]!=j[p]) return;
/* 2) code up the three rows as binary numbers				 */
	for (p=0, w=0; p<l; p++) w=w*KK+r[0][p];
	for (p=0, x=0; p<l; p++) x=x*KK+r[1][p];
	for (p=0, y=0; p<l; p++) y=y*KK+r[2][p];
    if (t==0) {w/=(KK*KK); x/=(KK*KK); y/=(KK*KK);}
/* 3) the top row pair has an out link, the bottom pair has an in link	 */
/*    what is done with this information depends on the parameter s	 */
/*    s=0 - transient discarding phase; link recorded in oc, count in lk */
/*    s=1 - the links are shown in a 64x64 matrix and recorded in ww	 */
/*    s=2 - but if the strip is too wide, make an alternative display 	 */
/*    s=3 - produce a printed listing of the linkages			 */		
if ((oc[w][x]>=4) && (oc[x][y]>=4)) {
  switch (s) {
    case 0:
//      adot(2*w,2*x,1); adot(2*w+1,2*x+1,1);
	  oc[w][x]|=1;
//      adot(2*x+1,2*y,2); adot(2*x,2*y+1,2);
	  oc[x][y]|=2;
          lk++; break;
    case 1:
      p5=oc[w][x]-4;
      q5=oc[x][y]-4;
//      adot(2*p5,2*q5,3); adot(2*p5+1,2*q5+1,3);
//      adot(2*p5+1,2*q5,3); adot(2*p5,2*q5+1,3);
//      luink(p5,q5,3);
      ww[p5][q5]=1;
      tox[p5]=w;
      box[p5]=x;
      if (t==0) {
	  avr[p5]=(r[0][  0]==0 && r[1][  0]==0);
	  avl[p5]=(r[0][l-3]==0 && r[1][l-3]==0);
	  }
      break;
    case 2:
      p5=oc[w][x]-4;
      q5=oc[x][y]-4;
//      adot(p5,q5,3);
      break;
    case 3:
      luink(p5,q5,3);
//      printf("      ");
//      if (l>1) for (p=0; p<l-1; p+=2)
//        printf("%1.1X",((r[0][p]*KK+r[1][p])*KK+r[0][p+1])*KK+r[1][p+1]);
//      if (l%2==1) printf("%1.1X",r[0][l-1]*KK+r[1][l-1]); printf(" ");
//      if (l>1) for (p=0; p<l-1; p+=2)
//	printf("%1.1X",((r[1][p]*KK+r[2][p])*KK+r[1][p+1])*KK+r[2][p+1]);
//      if (l%2==1) printf("%1.1X",r[1][l-1]*KK+r[2][l-1]); printf(" - ");
//      for (p=0; p<l; p++) printf("%1.1d",r[0][p]); printf(" ");
//      for (p=0; p<l; p++) printf("%1.1d",r[1][p]); printf(" ");
//      for (p=0; p<l; p++) printf("%1.1d",r[2][p]); printf("\n");
      break;
    default: break;
  }} 
}

/* multiply two matrices */
mprod(z,x,y)
int x[KK][KK][KK][KK][KK][KK][KK][KK];
int y[KK][KK][KK][KK][KK][KK][KK][KK];
int z[KK][KK][KK][KK][KK][KK][KK][KK];
{
int i0, i1, i2, i3;
int j0, j1, j2, j3;
int k0, k1, k2, k3;

for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)

for (k0=0; k0<KK; k0++)
for (k1=0; k1<KK; k1++)
for (k2=0; k2<KK; k2++)
for (k3=0; k3<KK; k3++) {

  z[i0][i1][i2][i3][k0][k1][k2][k3]=0; 

  for (j0=0; j0<KK; j0++)
  for (j1=0; j1<KK; j1++)
  for (j2=0; j2<KK; j2++)
  for (j3=0; j3<KK; j3++)

    z[i0][i1][i2][i3][k0][k1][k2][k3]+= 
    x[i0][i1][i2][i3][j0][j1][j2][j3]*y[j0][j1][j2][j3][k0][k1][k2][k3];

  }
}

/* fill an array with zeroes */
zerar(z)
int z[KK][KK][KK][KK][KK][KK][KK][KK];
{
int i0, i1, i2, i3;
int j0, j1, j2, j3;

for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)

  for (j0=0; j0<KK; j0++)
  for (j1=0; j1<KK; j1++)
  for (j2=0; j2<KK; j2++)
  for (j3=0; j3<KK; j3++)

    z[i0][i1][i2][i3][j0][j1][j2][j3]=0;

}

/* print the array z */
prnz(z) int z[KK][KK][KK][KK][KK][KK][KK][KK]; {
int i0, i1, i2, i3, i4, i5, i6, i7;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++) {
  for (i4=0; i4<KK; i4++)
  for (i5=0; i5<KK; i5++)
  for (i6=0; i6<KK; i6++)
  for (i7=0; i7<KK; i7++) {
    printf(" %d",z[i0][i1][i2][i3][i4][i5][i6][i7]);
    }
  printf("\n");
  }
}

/* print the diagonal of the array z in column 6*i */
prnd(z,i) int i, z[KK][KK][KK][KK][KK][KK][KK][KK]; {
int k, i0, i1, i2, i3;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++) {
    k=z[i0][i1][i2][i3][i0][i1][i2][i3];
    printf("%6u",k);
  }
}

/* number of cycles for six widths */
strp(z) int *z; {
int  x[KK][KK][KK][KK][KK][KK][KK][KK];
int  y[KK][KK][KK][KK][KK][KK][KK][KK];
mprod(x,z,z); prnd(x,0);
mprod(y,x,z); prnd(y,1);
mprod(x,y,z); prnd(x,2);
mprod(y,x,z); prnd(y,3);
mprod(x,y,z); prnd(x,4);
mprod(y,x,z); prnd(y,5);
}

/* fill the field with a configuration of periodicity obtained	*/
/* with the help of a de Bruijn diagram.			*/
/* bb is a connectivity matrix with FF's and coded nodes	*/
/* cc is an array for the sample configuration			*/
/* if (n==0) {screen(cc); printf("%3d",i); return;}		*/

filp(bb,cc,l) int *bb; char cc[QL][QL]; int l; {
int x[12*QWN], y[QWN], z[XC];
int i0, j0, i, j, k, m, n;
// cledi('m');
lnze(x,&n,bb);
// videocursor(0,1,34); printf("%5d",n);
if (n==0) return;
i=random()%n; m=x[i]; i0=m/QWN; j0=m%QWN; m=bb[m]; 
ecox(z,m%QWN,l); for (k=0; k<QL; k++) cc[QL/2][k]=z[k%l];
ecox(z,m/QWN,l); for (k=0; k<QL; k++) cc[QL/2+1][k]=z[k%l];
i=j0;
for (j=QL/2+2; j<QL; j++) {
  lnzr(y,&n,bb,i);
//   videocursor(0,1,28); printf("%5d",n);
  i=random()%n; i=y[i]; m=bb[i]/QWN; i%=QWN;
  ecox(z,m,l); for (k=0; k<QL; k++) cc[j][k]=z[k%l];
  }
i=i0;
for (j=QL/2-1; j>=0; j--) {
  lnzc(y,&n,bb,i);
//  videocursor(0,1,28); printf("%5d",n);
  i=random()%n; i=y[i]; m=bb[i]%QWN; i/=QWN;
  ecox(z,m,l); for (k=0; k<QL; k++) cc[j][k]=z[k%l];
  }
// frayme();
screen(cc);
}

/* locate and count the nonzero elements of the rth row of bb */
lnzr(x,n,bb,r)  int bb[QWN][QWN]; int x[QWN]; int *n, r; {int i, m;
for (i=0, m=0; i<QWN; i++) if (bb[r][i]!=-1) {x[m]=i+r*QWN; m++;} 
*n=m;}

/* locate and count the nonzero elements of the cth column of bb */
lnzc(x,n,bb,c)  int bb[QWN][QWN]; int x[QWN]; int *n, c; {int i, m;
for (i=0, m=0; i<QWN; i++) if (bb[i][c]!=-1) {x[m]=c+i*QWN; m++;}
*n=m;}

/* locate and count the nonzero elements within the matrix bb */
lnze(x,n,bb)  int bb[QWN][QWN]; int x[12*QWN]; int *n; {int i, j, m;
m=0; for (i=0; i<QWN; i++) for (j=0; j<QWN; j++) 
  if (bb[i][j]!=-1) {x[m]=j+i*QWN; m++;
     if (m>=12*QWN) {*n=m-1; printf("$"); return;}}
*n=m;}

/* decompose an index n into a series of l cell values */
ecox(x,n,l) int x[XC]; int n, l; {int i;
for (i=0; i<l; i++, n/=KK) x[l-i-1]=n%KK;}

/* draw the de Bruijn diagram */
/* links to the designated node are drawn in black		*/
/* if zero-links are admitted and we find one, draw it color 0	*/
/* if left-hugging is selected, draw left huggers in color 1	*/
/* if right-hugging is selected, draw right huggers in color 2	*/
/* if middle-of-the-roaders are selected, draw them in color 3	*/
/* if spanners are selected, draw them in color 4		*/
/* otherwise if there are no restrictions, color everything	*/
void dbdraw(s,n,z,f,t) int s, n, z, f, t; {int i, j;
  for (i=0; i<QWN; i++) for (j=0; j<QWN; j++) if (ww[i][j]!=0) {
    if (f && (i==qbno || j==qbno)) {luink(i,j,n,t,3); continue;}
    if (i==0 || j==0) {if (z) luink(i,j,n,t,0); continue;}
    switch (s) {
      case 0: if (avl[i]==1 && avl[j]==1) luink(i,j,n,t,1); break;
      case 1: if ((avl[i]==1 && avl[j]==1) && 
		  (avr[i]==1 && avr[j]==1)) luink(i,j,n,t,4); break;
      case 2: if (avr[i]==1 && avr[j]==1) luink(i,j,n,t,2); break;
      case 3: if ((avl[i]==0 && avl[j]==0) && 
		  (avr[i]==0 && avr[j]==0)) luink(i,j,n,t,4);
	      break;
      case 4:      if (avl[i]==1 && avl[j]==1) luink(i,j,n,t,1);
	      else if (avr[i]==1 && avr[j]==1) luink(i,j,n,t,2);
	      else    luink(i,j,n,t,5);
	      break;
      default: break;}
    }
}

/* insert a link into a de Bruijn diagram */
/* m - number of vertices */
/* u - initial vertex     */
/* v - terminal vertex    */
/* l - color of link	  */
void luink(u,v,j,k,l) int u, v, j, k, l; {
char   s[10];
double x, y, ex, wy, eks, wye, dx, dy, r, t;

	PSsetlinewidth(1.0);
   switch (l) {
     case 0:  PSsetrgbcolor(1.0,0.0,0.0); break;
     case 1:  PSsetrgbcolor(0.0,1.0,0.0); break;
     case 2:  PSsetrgbcolor(0.0,0.0,1.0); break;
     case 3:  PSsetgray(NX_BLACK); break;
     case 4:  PSsetgray(NX_LTGRAY); break;
     case 5:  PSsetrgbcolor(0.0,1.0,1.0); break;
     default: PSsetgray(NX_BLACK); break;}

	r=10.3;
	ex=qcntr[u][0];
	wy=qcntr[u][1];
	PSmoveto(ex,wy);
	x=qcntr[v][0];
	y=qcntr[v][1];
	dx=x-ex;
	dy=y-wy;
	t=atan2(dy,dx);
	eks=x-r*cos(t);
	wye=y-r*sin(t);
	PSlineto(eks,wye);
	PSlineto(eks-0.1*dx+0.05*dy,wye-0.1*dy-0.05*dx);
	PSlineto(eks-0.1*dx-0.05*dy,wye-0.1*dy+0.05*dx);
	PSlineto(eks,wye);
	PSmoveto(x+10.3,y);
	PSarc(x, y, 10.3, 0.0, 360.0);
	PSstroke();

   PSsetgray(NX_BLACK); 

   ex=qcntr[u][0];
   wy=qcntr[u][1];
   PSmoveto(ex-10.0,wy+5.0);
   switch (k) {
     case 0:  sprintf(s,"%4d",u); break;
     case 1:  sprintf(s,"%4d",tox[u]); break;
     case 2:  sprintf(s," "); break;
     case 3:  sprintf(s,"%d%d",avl[u],avr[u]); break;
     default: sprintf(s," "); break; }
   PSshow(s);

//   ex=qcntr[v][0];
//   wy=qcntr[v][1];
//   PSmoveto(ex-10.0,wy+5.0);
//	sprintf(s,"%3d,%d%d",v,avl[v],avr[v]);
//   PSshow(s);

   PSstroke();

   if (u==j) {
	PSstroke();
	x=qcntr[u][0];
	y=qcntr[u][1];
	PSsetlinewidth(3.0);
	PSmoveto(x+r,y);
	PSarc(x, y, 0.8*r, 0.0, 360.0);
        }
   PSstroke();
   PSsetlinewidth(0.0);
}

/* find index of nearest center to x, y (2-D de Bruijn) */
int proximate(x,y) double x, y; {int i, k; double ex, wy, r, p;
k=0; p=500000.0;
for (i=0; i<qnmax; i++) {
  ex=x-qcntr[i][0];
  wy=y-qcntr[i][1];
  r=ex*ex+wy*wy;
  if (r<p) {p=r; k=i;}
  }
return k;
}

/* place all the nodes forming a cycle around a ring */
nicering(k,r) int k; double r; {
int    i, j, kk, n, nn, p;
int    row[QWN], ring[12];
double x0, y0, en, rr, th, dt;
  kk=k;
  for (i=0, nn=0, x0=0.0, y0=0.0; i<12; i++) {
printf("nicering link %3d\n",kk);
    x0+=qcntr[kk][0];
    y0+=qcntr[kk][1];
for (j=0, p=0; j<QWN; j++) if (ww[kk][j]!=0) {row[p]=j; p++;}
//    lnzr(row,n,kk);
    j=0; if (p>1) j=random()%p;
    kk=ring[i]=row[j];
    nn++;
    if (kk==k) break;
    }
  en=(double)nn;
  x0/=en;
  y0/=en;
  dt=6.28/en;
  rr=(r*en)/6.28;
  for (i=0, th=0.0; i<nn; i++, th+=dt) {
    qcntr[ring[i]][0]=x0+r*cos(th);
    qcntr[ring[i]][1]=y0+r*sin(th);
    }
}

/* move the in-linked nodes into close proximity */

void inpull(gnod) int gnod; {
int    j;
double x, xx, y, yy;
  x = qcntr[gnod][0];
  y = qcntr[gnod][1];
  for (j=0; j<qnmax; j++) if (ww[gnod][j]!=0) {
    xx = qcntr[j][0];
    yy = qcntr[j][1];
    qcntr[j][0]=0.8*x+0.2*xx;
    qcntr[j][1]=0.8*y+0.2*yy;
    }
}

/* move the out-linked node into close proximity */

void outpull(gnod) int gnod; {
int    j;
double x, xx, y, yy;
  x = qcntr[gnod][0];
  y = qcntr[gnod][1];
  for (j=0; j<qnmax; j++) if (ww[j][gnod]!=0) {
    xx = qcntr[j][0];
    yy = qcntr[j][1];
    qcntr[j][0]=0.8*x+0.2*xx;
    qcntr[j][1]=0.8*y+0.2*yy;
    }
}

int inlinks(k) int k; {int i, n;
 for (i=0, n=0; i<qnmax; i++) if(ww[i][k]!=0) n++;
 return n;
}

int oulinks(k) int k; {int i, n;
 for (i=0, n=0; i<qnmax; i++) if(ww[k][i]!=0) n++;
 return n;
}

void printrail(ch,n,w,l) char *ch; int n, w; {
int i, p;
  for (i=0, p=n; i<w; i++, p/=KK) ch[i]='0'+p%KK;
  if (l) {ch[i]='\n'; i++;}
  ch[i]=0;
}

int topof(q) int q; {return tox[q];}
int bottm(q) int q; {return box[q];}

int fflip(q,w) int q, w; {int i, j, m, nt, nb;
  m=tox[q];
  for (i=0, nt=0; i<w-2; i++) {nt=nt*KK+m%KK; m/=KK;}
  m=box[q];
  for (i=0, nb=0; i<w-2; i++) {nb=nb*KK+m%KK; m/=KK;}
  for (i=0; i<no; i++) if (nb==box[i] && nt==tox[i]) break;
  return i;
}

void makemap() {
int i, j, k, n, l, dij, dji;
printf("starting makemap\n");
for (i=0; i<qnmax; i++) for (j=0; j<qnmax; j++) zz[i][j]=ww[i][j];
l=1;
do {
   n=0;
   l++;
   for (i=0; i<qnmax; i++) for (j=0; j<qnmax; j++) {
     xx[i][j]=0;
     for (k=0; k<qnmax; k++) xx[i][j]+=zz[i][k]*ww[k][j];
     }
   for (i=0; i<qnmax; i++) for (j=0; j<qnmax; j++) 
     if (zz[i][j]==0 && xx[i][j]!=0) {zz[i][j]=l; n=1;}
   } while (n==1);
   ndst=qnmax; xdst=0;
   unlat=0; nolat=0;
   for (i=0; i<qnmax; i++) for (j=0; j<qnmax; j++) {
     if (i==j) continue;
     dij=zz[i][j];
     dji=zz[j][i];
     if (dij==0) {if (dji==0) nolat++; else unlat++;}
     if (dij<ndst) ndst=dij;
     if (dij>xdst) xdst=dij;
     }
   nloo=qnmax; xloo=0;
   noloo=0;
   for (i=0; i<qnmax; i++) {
     dij=zz[i][i];
     if (dij==0) noloo++;
     if (dij<nloo) nloo=dij;
     if (dij>xloo) xloo=dij;
     }
// for (i=0; i<10; i++) {
//   for (j=0; j<10; j++) printf("%2d ",zz[i][j]);
//   printf("\n");
//   }
// printf("\n");
}

/* end nxbrui21.c */
