
/* HEXL21.C					*/
/* Two-Dimensional Hexagonal Life		*/
/* two-state cells with six neighbors		*/
/*						*/
/* Reference:					*/
/*						*/
/* Harold V. McIntosh, 16 May 1987		*/
/* 7 July 1987 - separate rules for each state	*/
/* 8 July 1988 - binary, 3 neighborhood classes	*/
/* 9 July 1988 - hexagonal lattice 		*/

# import  "nxlcau33.h"
# import  <dpsclient/psops.h>
# import  <appkit/graphics.h>

/* parameters related to arrays and automaton characteristics	*/

# define QL	    80  /* edge of square of evolution		*/
# define BPD	     7	/* degree of Bernstein polynomial	*/
# define GK        128	/* KK^BPD= total number neighborhoods	*/
# define GH      GK/KK	/* GK/KK= 1/KKth no. of neighborhoods	*/
# define NWOC	    13  /* number of symmetry classes w/o ctr	*/
# define TWIC	 NWOC*KK  /* number of symmetry classes w/ cntr	*/

/* coordinates of significant objects on the display screen 	*/

# define NRR	     3  /* row coordinate for neighborhood disp	*/
# define NRC	    25	/* column coordinate for neighborhood d	*/		
# define TRR	     2  /* row coordinate for totalistic rule	*/
# define TRC	     0	/* column coordinate for t'listic  rule	*/		
# define WRR	     0  /* row coordinate for Wolfram number	*/
# define WRC	     8	/* column coordinate Wolfram number	*/		
# define AUR	    64  /* row (from bottom) for planar array	*/
# define AUC	    60  /* column for planar array		*/
# define BER	     0  /* row (from bottom) for Bernstein p.	*/
# define BEC	   219  /* column for Bernstein Polynomial	*/
# define PLR	    60  /* row for planar array			*/
# define PLC	     0  /* column for planar array		*/

# define lcell(i,j) ar[li+i][lj-(j)]=1

extern char   ascrule[KK][KK][KK][KK][KK][KK][KK];
extern char   auxrule[KK][KK][KK][KK][KK][KK][KK];

char    thrule[TWIC];	/* evolutionary rule */
char    ahrule[TWIC];	/* auxiliary rule flags */
int     li, lj;		/* cursor in cell array */
int     nr[NWOC];	/* nbrhd representative of symm class */
int     rs[TWIC];	/* rule use statistics */
int	lf[KK];		/* number of live cells	   */
int     hxgus[2*NWOC];	/* golay use statistics    */
int     hxrus[DS];	/* rule use statistics     */
int     hxtus[TS];	/* totalistic use stats    */
int     gotb[DS];	/* Golay code key array    */
int     wc[BPD+1];	/* Bernstein coefficients */
double  bp[BPD+1];	/* Bernstein coefficients */

extern int     tc[DS];	/* totalistic key array	   */

mainhexl(c) char c; {
int  i, j, ln;
int  ar[AL];

    li=QL/2; lj=QL/2;

    for (i=0; i<TWIC; i++) rs[i]=0;
    for (i=0; i<TWIC; i++) ahrule[i]=' ';

  golta();
  for (j=1; j<KK; j++) for (i=0; i<GH; i++) gotb[i+j*GH]=gotb[i]+j*NWOC;
  for (i=0; i<NWOC; i++) bp[i]=((double)wc[i]);

    conlif();

    hscrrul();
    wscrrul();
    hrmenu();
    phrame(3);
    rancen(ar);
    screen(ar);
    phrame(3);

    while (0<1) {			/* set up one run */
    hrmenu();
    hscrrul();
    wscrrul();
    if (c=='q') break;
    switch (c) {
	case 'g':			/* one cycle of evolution */
	    phrame (1);
	    onehexgen(ar);
	    screen(ar);
	    break;
        case ':':			/* multiple cycles of evolution */
	    i=0;
	    rstat();
	    phrame (1);
            do {
	      i++;
	      videocursor(0,8,0);
	      printf("%5d",i);
	      onehexgen(ar);
	      screen(ar);
	      rstat();
	      } while (!kbdst());
            break;
    case 'M':				/* record rule usage */
      phrame (1);
      videoscroll(18,0,24,25,0,0);
      for (ln=0; ln<40; ln++) {
        for (i=0; i<TWIC; i++) rs[i]=0;
        videocursor(0,8,0);
        printf("%5d",ln+1);
        onehexgen(ar);
        screen(ar);
        rinci(40-ln);
        if (kbdst()) {kbdin(); break;};
	};
      break;
	case 's':
	    rstat();
	    break;
	case 't':			/* probability plot */
	    for (i=0; i<BPD+1; i++) bp[i]=0.0;
	    if (thrule[0]=='1') bp[0]+=1.0;
	    if (thrule[1]=='1') bp[1]+=6.0;
	    if (thrule[2]=='1') bp[2]+=6.0;
	    if (thrule[3]=='1') bp[2]+=6.0;
	    if (thrule[4]=='1') bp[2]+=3.0;
	    if (thrule[5]=='1') bp[3]+=6.0;
	    if (thrule[6]=='1') bp[3]+=12.0;
	    if (thrule[7]=='1') bp[3]+=2.0;
	    if (thrule[8]=='1') bp[4]+=6.0;
	    if (thrule[9]=='1') bp[4]+=6.0;
	    if (thrule[10]=='1') bp[4]+=3.0;
	    if (thrule[11]=='1') bp[5]+=6.0;
	    if (thrule[12]=='1') bp[6]+=1.0;
	    if (thrule[13]=='1') bp[1]+=1.0;
	    if (thrule[14]=='1') bp[2]+=6.0;
	    if (thrule[15]=='1') bp[3]+=6.0;
	    if (thrule[16]=='1') bp[3]+=6.0;
	    if (thrule[17]=='1') bp[3]+=3.0;
	    if (thrule[18]=='1') bp[4]+=6.0;
	    if (thrule[19]=='1') bp[4]+=12.0;
	    if (thrule[20]=='1') bp[4]+=2.0;
	    if (thrule[21]=='1') bp[5]+=6.0;
	    if (thrule[22]=='1') bp[5]+=6.0;
	    if (thrule[23]=='1') bp[5]+=3.0;
	    if (thrule[24]=='1') bp[6]+=6.0;
	    if (thrule[25]=='1') bp[7]+=1.0;
	    hxgfram(100,3);
	    hxgdiag(100,3);
	    bhxgraf(BPD,100);
	    break;
	case 'l':				/* edit cell array */
	    edhline(ar);
	    break;
        case 'r':				/* edit rule */	
	    edhrule();
	    videocursor(0,4,0);
	    break;
	case 'd':				/* de Bruijn diagram */
//	    edrui();
	    break;
       case 'u':				/* sparse init arry */
//	    zarry(ar);
//	    ar[QL/4][QL/4]=1;
//	    ar[QL/2+1][QL/2]=1;
//	    ar[QL/2-1][QL/2]=1;
//	    ar[QL/2][QL/2+1]=1;
//	    ar[QL/2][QL/2-1]=1;
//	    screen(ar);
	    break;
	case 'v':					/* Golay's Life */
	    conlif();
	    break;
	case 'X':					/* random rule */
	    ranhrul();
	    break;
	case 'x':					/* random rule */
	    for (i=0, j=0; j<TWIC; j++) {
	      if (i==0) i=rand();
	      if (ahrule[j]==' ') {thrule[j]='0'+i%KK; i/=KK;};
	      };
	    break;
	case 'y':					/* random line */
	    rancen(ar);
	    screen(ar);
	    break;
	case 'Y':					/* random line */
	    ranarr(ar);
	    screen(ar);
	    break;
/*	case '=':
/*	    for (i=1; i<8;  i++) {
/*	    for (j=0; j<SL; j++) ar[SL*i+j]=ar[j];};
/*	    lmenu();
/*	    break;
/*	case '~':
/*	    for (i=1; i<16;  i++) {
/*	    for (j=0; j<20; j++) ar[20*i+j]=ar[j];};
/*	    lmenu();
/*	    break; */
        default: break;
        }; /* end switch */
    };	   /* end while */
}	   /* end main */

void ascintohex() {int i0, i1, i2, i3, i4, i5, i6;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
  for (i4=0; i4<KK; i4++)
  for (i5=0; i5<KK; i5++)
  for (i6=0; i6<KK; i6++)
    thrule[i0+KK*(i1+KK*(i2+KK*(i3+KK*(i4+KK*(i5+KK*i6)))))] = 
	ascrule[i0][i1][i2][i3][i4][i5][i6];
}

void hexintoasc() {int i0, i1, i2, i3, i4, i5, i6;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
  for (i4=0; i4<KK; i4++)
  for (i5=0; i5<KK; i5++)
  for (i6=0; i6<KK; i6++)
    ascrule[i0][i1][i2][i3][i4][i5][i6] = 
	thrule[i0+KK*(i1+KK*(i2+KK*(i3+KK*(i4+KK*(i5+KK*i6)))))];
}

/* calculate one generation of evolution */
onehexgen(aa,bb,m,n) char aa[], bb[]; int m, n; {int i0, i1, gn;

/* count neighbors of center square */

for (i0=1; i0<m-1; i0++)
for (i1=1; i1<n-1; i1++) {
  gn=aa[n*(i0-1)+i1];
  gn=KK*gn+aa[n*(i0-1)+i1-1];
  gn=KK*gn+aa[n*i0+i1-1];
  gn=KK*gn+aa[n*(i0+1)+i1];
  gn=KK*gn+aa[n*(i0+1)+i1+1];
  gn=KK*gn+aa[n*i0+i1+1];
  gn+=GH*aa[n*i0+i1];
  bb[n*i0+i1]=gn;
  }

/* count neighbors for left edge (i1=0) */

for (i0=1; i0<m-1; i0++) {
  gn=aa[n*(i0-1)];
  gn=KK*gn+aa[n*(i0-1)+n-1];
  gn=KK*gn+aa[n*i0+n-1];
  gn=KK*gn+aa[n*(i0+1)];
  gn=KK*gn+aa[n*(i0+1)+1];
  gn=KK*gn+aa[n*i0+1];
  gn+=GH*aa[n*i0];
  bb[n*i0]=gn;
  }

/* count neighbors for right edge (i1=n-1) */

for (i0=1; i0<m-1; i0++) {
  gn=aa[n*(i0-1)+(n-1)];
  gn=KK*gn+aa[n*(i0-1)+(n-1)-1];
  gn=KK*gn+aa[n*i0+(n-1)-1];
  gn=KK*gn+aa[n*(i0+1)+(n-1)];
  gn=KK*gn+aa[n*(i0+1)];
  gn=KK*gn+aa[n*i0];
  gn+=GH*aa[n*i0+(n-1)];
  bb[n*i0+(n-1)]=gn;
  }

/* count neighbors for top edge (i0=0) */

for (i1=1; i1<n-1; i1++) {
  gn=aa[n*(m-1)+i1];
  gn=KK*gn+aa[n*(m-1)+i1-1];
  gn=KK*gn+aa[i1-1];
  gn=KK*gn+aa[n+i1];
  gn=KK*gn+aa[n+i1+1];
  gn=KK*gn+aa[i1+1];
  gn+=GH*aa[i1];
  bb[i1]=gn;
  }

/* count neighbors of bottom edge (i0=m-1) */

for (i1=1; i1<n-1; i1++) {
  gn=aa[n*((m-1)-1)+i1];
  gn=KK*gn+aa[n*((m-1)-1)+i1-1];
  gn=KK*gn+aa[n*(m-1)+i1-1];
  gn=KK*gn+aa[i1];
  gn=KK*gn+aa[i1+1];
  gn=KK*gn+aa[n*(m-1)+i1+1];
  gn+=GH*aa[n*(m-1)+i1];
  bb[n*(m-1)+i1]=gn;
  }

/* count neighbors for upper left hand corner */
/* i0 = 0, i1 = 0 			      */

  gn=aa[n*(m-1)];
  gn=KK*gn+aa[n*m-1];
  gn=KK*gn+aa[n-1];
  gn=KK*gn+aa[n];
  gn=KK*gn+aa[n+1];
  gn=KK*gn+aa[1];
  gn+=GH*aa[0];
  bb[0]=gn;

/* count neighbors for upper right hand corner */
/* i0 = 0, i1 = n-1 			       */

  gn=aa[n*m-1];
  gn=KK*gn+aa[n*(m-1)+n-2];
  gn=KK*gn+aa[n-2];
  gn=KK*gn+aa[2*n-1];
  gn=KK*gn+aa[2*n];
  gn=KK*gn+aa[n];
  gn+=GH*aa[n-1];
  bb[n-1]=gn;

/* count neighbors for bottom left hand corner */
/* i0 = m-1, i1 = 0 			       */

  gn=aa[n*(m-2)];
  gn=KK*gn+aa[n*(m-2)+n-1];
  gn=KK*gn+aa[n*(m-1)+n-1];
  gn=KK*gn+aa[0];
  gn=KK*gn+aa[1];
  gn=KK*gn+aa[n*(m-1)+1];
  gn+=GH*aa[n*(m-1)];
  bb[n*(m-1)]=gn;

/* count neighbors for bottom right hand corner */
/* i0 = m-1, i1 = n-1 			        */

  gn=aa[n*(m-2)+n-1];
  gn=KK*gn+aa[n*(m-2)+n-2];
  gn=KK*gn+aa[n*(m-1)+n-2];
  gn=KK*gn+aa[(n-1)];
  gn=KK*gn+aa[0];
  gn=KK*gn+aa[n*(m-1)];
  gn+=GH*aa[n*m-1];
  bb[n*m-1]=gn;

/* phase to update cells */
for (i0=0; i0<KK; i0++)   lf[i0] =0;
for (i0=0; i0<TS; i0++)   hxtus[i0]=0;
for (i0=0; i0<DS; i0++)   hxrus[i0]=0;
for (i0=0; i0<2*NWOC; i0++) hxgus[i0]=0;
for (i0=0; i0<m; i0++) for (i1=0; i1<n; i1++) {
  gn=bb[n*i0+i1];
  aa[n*i0+i1]=thrule[gn]-'0';
  lf[aa[n*i0+i1]]+=1;
//  hxgus[gotb[gn]]+=1;
  hxtus[tc[gn]]+=1;
  hxrus[gn]++;
  }

/* phase to show results */
}	/* end onehexgen */
    
/*find fraction of live cells in arry[m][n] */
void hxstatecount(aa,m,n) char *aa; int m, n; {int i0, i1;
  for (i0=0; i0<KK; i0++) lf[i0]=0;
  for (i0=0; i0<m; i0++) for (i1=0; i1<n; i1++) lf[aa[n*i0+i1]]+=1;
}

/* display a screen of evolution */
screen(ar,m,n) char ar[], m, n; {int i0, i1;
  for (i1=0; i1<n; i1++) for (i0=0; i0<m; i0++) hxbdot(i0,i1,ar[n*i0+i1]);}

/* display a frame around the cell array */
phrame(l) int l; {int j;
for (j=0; j<QL; j++) {
  adot(0,j,l);
  adot(QL-1,j,l);
  adot(j,0,l);
  adot(j,QL-1,l);
  }
}

/* make a frame for the fixpoint graph  */
/* n     = dimension of arframe         */
/* l     = color of fiducial marks      */
hxgfram(n,l) int n, l; {int i;
for (i=0; i<=n; i++) {hxbdot(i,0,0);hxbdot(i,n,0);hxbdot(n,i,0);hxbdot(0,i,0);};
for (i=0; i<=n; i+=10) {hxbdot(i,0,l);hxbdot(i,n,l);hxbdot(n,i,l);hxbdot(0,i,l);};
}

/* put a diagonal in the fixpoint graph */
hxgdiag(n,l) int n, l; {int i; for (i=0; 2*i<n; i++) hxbdot(2*i,2*i,l);}

/* graph incidence of rule usage  */
/* beware limited count capacity  */
rinci(j) int j; {int i; for (i=0; i<TWIC; i++) sdot(i,j,rs[i]==0?1:3);}

/* graph statistics of rule usage */
/* beware limited count capacity  */
/* large counts look negative     */
rstat() {int i, j, m, n; double s, t;
for (i=0, s=0.0; i<TWIC; i++) s+=(double)rs[i];
if  (s<1.0) {for (i=0; i<TWIC; i++) rs[i]=0; return;};
for (i=0; i<TWIC; i++) {
  t=333.3*(((double)rs[i])/s); /* full scale = 1/10 population */
  if (t>32000.0) t=50.0;
  n=(int)t;
  sdot(i,33,0);
  m=33; if (n>33) {n=33; m=34;};
  if (n>0) for (j=0; j<n; j++) sdot(i,j,1);
  for (j=n; j<m; j++) sdot(i,j,2);
  rs[i]=0;
  };
}

/* generate the Golay code table */
golta() {
int i, j, k;
int i0, i1, i2, i3, i4, i5;
k=0;
for (i5=0; i5<KK; i5++)
for (i4=0; i4<KK; i4++)
for (i3=0; i3<KK; i3++)
for (i2=0; i2<KK; i2++)
for (i1=0; i1<KK; i1++)
for (i0=0; i0<KK; i0++) {
  i=binq(i0,i1,i2,i3,i4,i5);
  j=binq(i1,i2,i3,i4,i5,i0);
  if (j<i) i=j;
  j=binq(i2,i3,i4,i5,i0,i1);
  if (j<i) i=j;
  j=binq(i3,i4,i5,i0,i1,i2);
  if (j<i) i=j;
  j=binq(i4,i5,i0,i1,i2,i3);
  if (j<i) i=j;
  j=binq(i5,i0,i1,i2,i3,i4);
  if (j<i) i=j;
  j=binq(i5,i4,i3,i2,i1,i0);
  if (j<i) i=j;
  j=binq(i4,i3,i2,i1,i0,i5);
  if (j<i) i=j;
  j=binq(i3,i2,i1,i0,i5,i4);
  if (j<i) i=j;
  j=binq(i2,i1,i0,i5,i4,i3);
  if (j<i) i=j;
  j=binq(i1,i0,i5,i4,i3,i2);
  if (j<i) i=j;
  j=binq(i0,i5,i4,i3,i2,i1);
  if (j<i) i=j;
  j=binq(i0,i1,i2,i3,i4,i5);
  if (i==j) {gotb[i]=k; nr[k]=j; k++;} else {gotb[j]=gotb[i];};
  /*
  printf("  %2X, gotb[%3d]=%3d, k=%3d, (%d,%d)\n",j,j,i,k,i0+i2+i4,i1+i3+i5);
  */
  }
  for (i=1; i<KK; i++) for (j=0; j<DS/KK; j++) gotb[i*(DS/KK)+j]=gotb[j];
printf("no. hex classes = %3d\n",k);
// for (i=0; i<DS; i++) printf("  %d",gotb[i]);
// printf("\n");
}

/* calculate binary equivalent */
int binq(i0,i1,i2,i3,i4,i5)
int i0, i1, i2, i3, i4, i5; {
return i0+KK*(i1+KK*(i2+KK*(i3+KK*(i4+KK*i5))));}

/* calculate Golay class of a cell */
int golcl(i0,i1,ar,m,n) int  i0, i1, m, n; char ar[]; {int gn;
  gn=ar[m*(i0-1)+i1];
  gn=KK*gn+ar[m*(i0-1)+i1-1];
  gn=KK*gn+ar[m*(i0)+i1-1];
  gn=KK*gn+ar[m*(i0+1)+i1];
  gn=KK*gn+ar[m*(i0+1)+i1+1];
  gn=KK*gn+ar[m*(i0)+i1+1];
  if (ar[m*(i0)+i1]==1) gn+=GH;
  gn=gotb[gn];
  return gn;
}

/* calculate the image of a neighborhood */
int golcln(i0,i1,i2,i3,i4,i5,i6) int  i0, i1, i2, i3, i4, i5, i6;
{return thrule[gotb[(((((i0*KK+i1)*KK+i2)*KK+i3)*KK+i4)*KK+i5)*KK+i6]]-'0';}

/* turn the golay rule number into an ascii rule number */
goltoasc() {
int i0, i1, i2, i3, i4, i5, i6;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
  for (i4=0; i4<KK; i4++)
  for (i5=0; i5<KK; i5++)
  for (i6=0; i6<KK; i6++)
    ascrule[i0][i1][i2][i3][i4][i5][i6]=golcln(i0,i1,i2,i3,i4,i5,i6)+'0';
}

/* generate the de Bruijn diagram of s strip */
rijn() {int i, j, i0, i1, i2, i3, i4, i5, i6;
i=0; j=0;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++) {
  i++;
  if (golcln(i0,i1,i2,i3,i4,i5,i6)==i0) {
    j++;
    videocursor(0,8,3);
    printf("%X-%X\n",binq(i0,i3,i4,i5,0,0),binq(i1,i2,i0,i6,0,0));
    };
}
videocursor(0,9,3);
printf("%d links amongst %d max",j,i);
}

/* graph Bernstein polynomial */
bhxgraf(k,n) int k, n; {int i; double hexbern(), en, p;
en=(double)(n);
for (i=0; i<n; i++) {
  p=((double)(i))/en;
  hxbdot(i,(int)(en*hexbern(p,k)),1);
  }
}

/* evaluate the nth degree Bernstein polynomial at point p */
double hexbern(p,n) double p; int n; {double q, s, x, r; int i;
if (p>0.99) return bp[n];
q=1.0-p; r=p/q; s=0.0; x=1.0;
for (i=0; i<n;  i++) x*=q;
for (i=0; i<=n; i++, x*=r) s+=bp[i]*x;
s*=0.9998;
return(s+0.0001);
}

/* adot(i,j,l) - place dot on automaton screen at (i,j) in color l */
adot(i,j,l) int i, j, l; {videodot(199-PLR-(QL-1-j),PLC+(QL-1-j)+2*i,l);}

/* hxbdot(i,j,l) - place dot on Bernstein screen at (i,j) in color l */
hxbdot(i,j,l) int i, j, l; {videodot(j,i,l);}

/* hxbline(x,y,l) - place line on Bernstein screen to (x,y); l=first */
hxbline(x,y,l) double x, y; int l; 
	{if (l==0) PSmoveto(y,150.0-x); else PSlineto(y,150.0-x);}

/* sdot(i,j,l) - place dot on statistics screen at (i,j) in color l */
sdot(i,j,l) int i, j, l; {videodot(199-10-j,25+4*i,l);}

/* clear the cell array */
zarry(ar,m,n) char  ar[]; int m, n; {int i0, i1;
for (i0=0; i0<m; i0++) for (i1=0; i1<n; i1++) ar[m*i0+i1]=0;}

/* display rule menu */
hrmenu() 
{videocursor(0,0,33); scrstr("HEXL21"); scrrul(); videocursor(0,TRR,TRC);}

/* display Wolfram rule number of planar cross section */
/* calculate LCAU21 wru from a HEXL21 argument rule    */
void whxsscrrul(c,ru) char *c, ru[KK][KK][KK][KK][KK][KK][KK]; {
char ur[KK][KK][KK]; int i0, i1, i2;
  for (i0=0; i0<KK; i0++) for (i1=0; i1<KK; i1++) for (i2=0; i2<KK; i2++)
    ur[i0][i1][i2]=ru[i0][i1][i2][i2][i1][i0][i1];
  wrru21(c,ur);
  }

/* display rule number by sum in color */
scrrulx() {int i, j; 
for (j=0; j<KK; j++)
for (i=0; i<NWOC; i++) {
  videocursor(0,TRR+j,TRC+8+i);
  resch(j*NWOC+i);
  }
}

/* display rule number as a hexadecimal */
hscrrul() {int i; long int k;
  for (i=0, k=0; i<TWIC; i++) k=KK*k+thrule[i]-'0';
  videocursor(0,WRR,WRC);
  printf("        ");
  videocursor(0,WRR,WRC);
  printf("%7.7lX",k);
  videocursor(0,0,0);
  }

/* display Wolfram rule number of cross section */
wscrrul() {int i;
  i=thrule[25]-'0';
  i=KK*i+thrule[20]-'0';
  i=KK*i+thrule[10]-'0';
  i=KK*i+thrule[2] -'0';
  i=KK*i+thrule[20]-'0';
  i=KK*i+thrule[18]-'0';
  i=KK*i+thrule[2] -'0';
  i=KK*i+thrule[0] -'0';
  videocursor(0,WRR,WRC+9);
  printf("     ");
  videocursor(0,WRR,WRC+9);
  printf("[%2d]",i);
  videocursor(0,0,0);
  }

/* edit the rule */
edhrule() {char c; int i, j;
  j=0; i=0;
  while (0<1) {
    hscrrul();
    wscrrul();
    shon(j,nr[i]);
    videocursor(0,TRR+j,TRC+i+8);
    videoputc(thrule[j*NWOC+i],3);
    videocursor(0,TRR+j,TRC+i+8);
    c=kbdin();
      if (c=='\015') break;				/* carriage return */
      switch (c) {
        case '0': case '1': 				/* state = 0,1 */
          thrule[j*NWOC+i]=c;
	  resch(j*NWOC+i);
          i++; if (i==NWOC) {i=0; j++; if (j==KK) j=0;};
	  break;
        case ' ': case '\315':				/* space = advance */
	  resch(j*NWOC+i);
          i++; if (i==NWOC) {i=0; j++; if (j==KK) j=0;};
	  break;
        case '\010': case '\313':			/* backspace */
	  resch(j*NWOC+i);
          if (i>0) i--; else if(j>0) {i=NWOC-1; j--;};
	  break;
	case '<':					/* cursor to front */
	  resch(j*NWOC+i);
	  i=0; j=0;
	  break;
	case '*':					/* cursor to center */
	  resch(j*NWOC+i);
	  i=0; j=1;
	  break;
	case '>':					/* cursor to end */
	  resch(j*NWOC+i);
	  i=NWOC-1; j=KK-1;
	  break;
	case '\322':					/* insert */
	  ahrule[j*NWOC+i]='+';
	  resch(j*NWOC+i);
          i++; if (i==NWOC) {i=0; j++; if (j==KK) j=0;};
	  break;
	case '\323':					/* delete */
	  ahrule[j*NWOC+i]=' ';
	  resch(j*NWOC+i);
          i++; if (i==NWOC) {i=0; j++; if (j==KK) j=0;};
	  break;
	case '\013': case '\310':			/* up arrow */
	  resch(j*NWOC+i);
          if (j==0) j=KK; else j--;
	  break;
	case '\012': case '\320':			/* down arrow */
	  resch(j*NWOC+i);
          j++; if (j==KK) j=0;
	  break;
	default:
	  break;
          };
    };
eran();
}

/* restore character in edhrule */
resch(i) int i; {videoputc(thrule[i],(ahrule[i]==' ')?1:2);}

/* show neighborhood in edhrule */
/* i = center cell */
/* j = Golay code  */
shon(i,j) int i, j; {int k;
  videocursor(0,NRR,NRC);
  videoputc('0'+i,1);
  videocursor(0,NRR,NRC+2);
  k=j; videoputc('0'+k%KK,1);
  videocursor(0,NRR-1,NRC+1);
  k=k/KK; videoputc('0'+k%KK,1);
  videocursor(0,NRR-1,NRC-1);
  k=k/KK; videoputc('0'+k%KK,1);
  videocursor(0,NRR,NRC-2);
  k=k/KK; videoputc('0'+k%KK,1);
  videocursor(0,NRR+1,NRC-1);
  k=k/KK; videoputc('0'+k%KK,1);
  videocursor(0,NRR+1,NRC+1);
  videoputc('0'+k/KK,1);
}

/* erase the neighborhood in edhrule */
eran() {
  videocursor(0,NRR,NRC); videoputc(' ',1);
  videocursor(0,NRR,NRC+2); videoputc(' ',1);
  videocursor(0,NRR-1,NRC+1); videoputc(' ',1);
  videocursor(0,NRR-1,NRC-1); videoputc(' ',1);
  videocursor(0,NRR,NRC-2); videoputc(' ',1);
  videocursor(0,NRR+1,NRC-1); videoputc(' ',1);
  videocursor(0,NRR+1,NRC+1); videoputc(' ',1);
}

/* edit the plane of cells */
edhline(ar) int ar[QL][QL]; {
char c;
int  i, j, k, arrr2[QL][QL];

for (i=0; i<QL; i++) for (j=0; j<QL; j++) arrr2[i][j]=0;

videoscroll(18,0,24,24,0,0);
videocursor(0,19,0);
scrstr("insert states with 0,1,.");
videocursor(0,20,0);
scrstr("move cursor using arrows");
videocursor(0,21,0);
scrstr("NSEW R CGO fgmMxz *<>^_");
videocursor(0,22,0);
scrstr("carriage return exits");

while (0<1) {
  adot(li,lj,3);
  videocursor(0,0,1);
  videoputc('?',1);
  c=kbdin();
  videocursor(0,0,1);
  videoputc(c,1);
  if (c == '\015') break;
  switch (c) {
    case '0': case '1':	case '.':	/* enter state */
      if (c=='.') c='1';
      i=c-'0';
      ar[li][lj]=i;
      adot(li,lj,i);
      break;
    case '\012': case '\320':		/* down    */
      adot(li,lj,ar[li][lj]);
      lj++; if (lj==QL) lj=0;
      break;
    case '\013': case '\310':		/* up      */
      adot(li,lj,ar[li][lj]);
      if (lj==0) lj=QL; lj--;
      break;
    case '\014': case '\315': case ' ':	/* forward */
      adot(li,lj,ar[li][lj]);
      li++; if (li==QL) li=0;
      break;
    case '\010': case '\313':		/* back    */
      adot(li,lj,ar[li][lj]);
      if (li==0) li=QL; li--;
      break;
    case '<':				/* left margin  */
      adot(li,lj,ar[li][lj]);
      li=0;
      break;
    case '>':				/* right margin */
      adot(li,lj,ar[li][lj]);
      li=QL-1;
      break;
    case '^':				/* top margin   */
      adot(li,lj,ar[li][lj]);
      lj=0;
      break;
   case '_':				/* bottom margin  */
      adot(li,lj,ar[li][lj]);
      lj=QL-1;
      break;
   case '*':				/* center cursor */
      adot(li,lj,ar[li][lj]);
      li=QL/2; 
      lj=QL/2;
      break;
    case 'a':				/* insert glider */
      lcell(0,2);
      lcell(1,1); lcell(1,3);
      lcell(3,2);
      lcell(4,0); lcell(4,1);
      screen(ar);
      break; 
    case 'b':				/* insert glider */
      lcell(0,0);
      lcell(1,0);
      lcell(2,-1);
      lcell(3,0); lcell(3,-1);
      lcell(4,-2);
      screen(ar);
      break; 
     case 'f':				/* generate a frame */
      phrame (2);
      break;
    case 'g':				/* one cycle of evolution */
      phrame (1);
      onehexgen(ar);
      screen(ar);
      break;
 case 'm':				/* mark neighborhood */
      ahrule[golcl(li,lj,ar)]='+';
      videocursor(0,TRR,TRC+8);
      scrrul();
      break;
    case 'M':				/* mark neighborhood over a region */
      if (li<10||li>QL-11) break;
      if (lj<10||lj>QL-11) break;
      for (i=li-10; i<li+10; i++) {
      for (j=lj-10; j<lj+10; j++) {
        ahrule[golcl(i,j,ar)]='+';
        };};
      videocursor(0,TRR,TRC+8);
      scrrul();
      break;
    case 'x':				/* uncommitted transitions */
      for (k=0; k<TWIC; k++) ahrule[k]=' ';
      videocursor(0,TRR,TRC+8);
      scrrul();
      break;
    case 'z':				/* clear array  */
      zarry(ar);
      screen(ar);
      break;
    case 'N':				/* rotate array up */
      for (i=0; i<QL; i++) {
        k=ar[i][0]; 
        for (j=1; j<QL; j++) ar[i][j-1]=ar[i][j];
	ar[i][QL-1]=k;
	};
      if (lj==0) lj=QL; lj--;
      screen(ar);
      break;
    case 'S':				/* rotate array down */
      for (i=0; i<QL; i++) {
	k=ar[i][QL-1];
        for (j=1; j<QL; j++) ar[i][QL-j]=ar[i][QL-j-1];
	ar[i][0]=k;
	};
      lj++; if (lj==QL) lj=0;
      screen(ar);
      break;
    case 'E':				/* rotate array right */
      for (j=0; j<QL; j++) {
	k=ar[QL-1][j];
        for (i=1; i<QL; i++) ar[QL-i][j]=ar[QL-i-1][j];
	ar[0][j]=k;
	};
      li++; if (li==QL) li=0;
      screen(ar);
      break;
    case 'W':				/* rotate array left */
      for (j=0; j<QL; j++) {
	k=ar[0][j];
        for (i=1; i<QL; i++) ar[i-1][j]=ar[i][j];
	ar[QL-1][j]=k;
	};
      if (li==0) li=QL; li--;
      screen(ar);
      break;
    case 'R':				/* reflect array in diagonal */
      for (j=1; j<QL; j++) {
      for (i=0; i<j;  i++) {
	k=ar[i][j];
	ar[i][j]=ar[j][i];
        ar[j][i]=k;
        };};
      k=li; li=lj; lj=k;
      screen(ar);
      break;
    case 'C':				/* save cell array */
      for (j=0; j<QL; j++) {
      for (i=0; i<QL; i++) {
	arrr2[i][j]=ar[i][j];
	};};
      break;
    case 'G':				/* recover saved cell array */
      for (j=0; j<QL; j++) {
      for (i=0; i<QL; i++) {
	ar[i][j]=arrr2[i][j];
	};};
      screen(ar);
      break;
    case 'O':				/* or saved cell array */
      for (j=0; j<QL; j++) {
      for (i=0; i<QL; i++) {
	ar[i][j]|=arrr2[i][j];
	};};
      screen(ar);
      break;
    default:
      break;
      };				/* end switch */
  };					/* end while */
adot(li,lj,ar[li][lj]);
videoscroll(18,0,24,24,0,0);
videocursor(0,0,1);
videoputc(' ',1);
}				        /* end edline */

/* generate the transitions for Golay's Life */
void conlif() {int i, i0, i1, i2, i3, i4, i5, i6;
printf("Golay's Life\n");
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
  for (i4=0; i4<KK; i4++)
  for (i5=0; i5<KK; i5++)
  for (i6=0; i6<KK; i6++) {
    i=i0+KK*(i1+KK*(i2+KK*(i3+KK*(i4+KK*(i5+KK*i6)))));
    if ( gotb[i]==2 || gotb[i]==3 )       /* || gotb[i]==15 || gotb[i]==16 */
          ascrule[i0][i1][i2][i3][i4][i5][i6] = '1';
    else  ascrule[i0][i1][i2][i3][i4][i5][i6] = '0';
    }
}

/* generate random cells in ar */
ranarr(ar,m,n) char ar[]; int m, n; {int i0, i1;
for (i0=0; i0<m; i0++) for (i1=0; i1<n; i1++) ar[m*(i0)+i1]=random()%KK;}

/* generate random cells in the center of ar */
rancen(ar,m,n) char ar[]; int m, n; {int i0, i1;
for (i0=0; i0<m; i0++) for (i1=0; i1<n; i1++) ar[m*(i0)+i1]=0;
for (i0=m/3; i0<2*(m/3); i0++)
for (i1=n/3; i1<2*(n/3); i1++) ar[m*(i0)+i1]=random()%KK;
}

/* generate a random rule */
ranhrul() {int i0; for (i0=0; i0<TWIC; i0++) thrule[i0]=random()%KK+'0';}

// # include "nxbrui21.c"

/* end nxhexl21.c */
