
/* nxlcau33.c				*/
/* (3,3) Linear Cellular Automaton	*/

/* Reference:				*/
/*					*/
/*    Kenneth E. Perry			*/
/*    Abstract Mathematical Art		*/
/*    BYTE				*/
/*    December, 1986			*/
/*    pages 181-192			*/

/*    Copyright (C) 1987		*/
/*    Copyright (C) 1988		*/
/*    Copyright (C) 1989		*/
/*    Copyright (C) 1994		*/
/*    Harold V. McIntosh		*/
/*    Gerardo Cisneros S.		*/

/* G. Cisneros, 4.3.87						*/
/* 10 April 1987 - modified for (4,2) [HVM]			*/
/* 26 April 1987 - Multiple menus [HVM]				*/
/* 28 April 1987 - back modified to (4,1) [HVM]			*/
/* 28 April 1987 - version for XVI Feria de Puebla [HVM]	*/
/* 14 May 1987 - modified for (3,1) and general rule [HVM]	*/
/* 19 May 1987 - modified for (2,2) [HVM]			*/
/* 20 May 1987 - modified for (2,3) [HVM]			*/
/* 21 May 1987 - Wolfram rule number [HVM]			*/
/* 8 June 1987 - general rule for (4,1) [HVM]		 	*/
/* 12 June 1987 - cartesian product of (2,1) rules [HVM]	*/
/* 12 June 1987 - (2,1) rule with memory  [HVM]			*/
/* 14 June 1987 - individual cycles of evolution [HVM]		*/
/* 17 June 1987 - p-adic representation in plane [HVM]		*/
/* 22 June 1987 - 2 speed gliders via 16 transitions [HVM]	*/
/* 26 June 1987 - push, pop the rule [HVM]			*/
/* 26 June 1987 - conserve position of rule cursor [HVM]	*/
/* 27 June 1987 - incremental rule construction [HVM]		*/
/* 29 June 1987 - conserve position of cell pointer [HVM]	*/
/* 30 June 1987 - mark & unmark transitions, xchg x&X [HVM]	*/
/* 25 July 1987 - display and edit de Bruijn diagrams [HVM]	*/
/* 27 July 1987 - graph of transition probabilities [HVM]	*/
/* 4 September 1987 - PROB21.C for option 't' [HVM]		*/
/* 21 October 1987 - program disks disappeared			*/
/* 20 December 1987 - program reconstructed from listings	*/
/* 20 February 1988 - RIJN21.C for option 'd' [HVM]		*/
/* 8 March 1989 - extended to third neighbors [HVM]		*/
/* 11 March 1989 - S option for semitotalistic rules [HVM]	*/
/* 11 March 1989 - x option in srule and trule [HVM]		*/
/* 16 March 1989 - *+- option to change screen color [HVM]	*/
/* 24 September 1989 - edline allows custom design [HVM]	*/
/* 30 July 1994 - adapted to NeXT [HVM]				*/

/* # include <bdos.h> */
# include "rec.h"
# include "rectbl.h"
# include "keybrd.h"
# include "nxlcau33.h"

# define LROW		11	/* first row of line panel */

char xrule[NX][KK][KK][KK][KK][KK][KK][KK];

char ixrule[NX][DS]= {

	"00010110011010010110100110010111000101100110100101101001100101100001001110101001110011000110001100101110000110101011001111000100",
		/*                      */
	"01101000001100110011001100110011100100001011001100000001011111001111110100110011001100110011001111001010100010100011100111000110"	
		/*                     	*/

	};

double blpr[KK];			/* probability from blocks	*/

char   hexrule[KK][KK][KK][KK][KK];
char   ascrule[KK][KK][KK][KK][KK][KK][KK];
char   auxrule[KK][KK][KK][KK][KK][KK][KK];
char   arr1[AL], arr2[AL], arr3[AL];
char   gf2rule[13];			/*   coefficients of GF2 rule	*/
char   strule[KK][TS-1];
char   sarule[KK][TS-1];
char   ttrule[TS];
char   tarule[TS];
int    binrule[KK][KK][KK][KK][KK][KK][KK];
int    stsum[KK];				/* state totals		     */
int    nhsum[KK][KK][KK][KK][KK][KK][KK];	/* neighborhood totals	     */
int    ru0, ru1, ru2, ru3, ru4, ru5, ru6;	/* cursor position in edrule */
int    ix0, iy0;				/* origin for pen moves      */
int    rij, rdi, rdj;				/* rec demo pointers 	     */
int    di, dj, dij;				/* demo pointer 	     */
int    li, lj;					/* cursor position in edline */
int    co, dn, dp, pa;				/* connectivity, dimension   */
int    pr, pl;					/* rule, line prob, mils   */
int    rp, rq;					/* source code in cstr	   */
int    bi;					/* bernstein index	   */
int    ct0, ct1, ct2, ct3;			/* general counters	   */
int    sw0, sw1, sw2, sw3;			/* general switches	   */
int    ex, wy;					/* hiplot coordinates	   */
double nu0, nu1, nu2, nu3;			/* general numeric values  */
double mf;					/* mouse increment	   */
double sm[TS];					/* selfconsistent nbrhd    */
double sn[TS];					/* selfconsistent nbrhd    */
double bx[BD];					/* bernstein factor        */
double bp[BD];
double pv[10];					/* polynomial vector 	   */
double fpa, fpb, fpc;				/* fixed points		   */

int    cc, dd;

Inst   rprg[384];			/* rec program array */
char   cstr[CLEN];			/* console rec program */
char   cprg[CLEN];			/* edit stored program */
Inst   xprg[10][XLEN];
char   idef[1][30]= { " (()) " };  //(z(!100!0r:;)1r2r(0r:;);) a" };

long int r_intpar;

# include "nxlcaukr.c"
# include "nxlcauk3.c"

nxlcau33main() {
int  more='r', sc=0;
int  i, j, i0, i1, i2, i3, i4, i5, i6;
char c;

for (i=0; i<TS; i++) ttrule[i]='0';
for (i=0; i<KK; i++) for (j=0; j<TS-1; j++) strule[i][j]='0';

ru0=0; ru1=0; ru2=0; ru3=0; ru4=0; ru5=0; ru6=0;
li=SL/2; lj=0;
pr=pl=500;					/* probability, mils   */
for (i=0; i<BD; i++) bx[i]=1.0;			/* bernstein factor    */
co=1;						/* normal connectivity */
dn=1;						/* dimension	       */
dp=0;						/* depth of 3-D	       */
pa=0;						/* pair type in ranpa  */
bi=0;						/* bernstein index     */
ex=wy=0;					/* hiplot coordinates  */
mf=0.05;
rij=2; rdi=0; rdj=1;
rp=rq=0; cstr[rp]=0;
ct0=ct1=ct2=ct3=0;				/* general counters  */
sw0=sw1=sw2=sw3=0;				/* switches	       */
nu0=nu1=nu2=nu3=0.0;				/* general constants */

for (i=0; i<NX; i++) {				/* copy to 3-index array */
i0=0; i1=0; i2=0; i3=0; i4=0; i5=0; i6=0;
for (j=0; j<DS; j++) {
  xrule[i][i0][i1][i2][i3][i4][i5][i6]=ixrule[i][j];
  i6++;
  if (i6>KK-1) {i6=0; i5++;}
  if (i5>KK-1) {i5=0; i4++;}
  if (i4>KK-1) {i4=0; i3++;}
  if (i3>KK-1) {i3=0; i2++;}
  if (i2>KK-1) {i2=0; i1++;}
  if (i1>KK-1) {i1=0; i0++;}
  if (i0>KK-1) {i0=0; }
}}


    videopalette(WHCYMAG);				/* white/cyan/magenta */

    tuto();
    while (!kbdst()) random();				/* wait for keypress */
    kbdin();						/* ignore it */
    videomode(T80X25);
    videoscroll(0,0,CR,71,0,3);				/* menu on blue background */
    videoscroll(CL+8,0,24,71,0,3);
    dij=random()%NX;
    di=dij%DROWS;
    dj=(dij/DROWS==1?DCOLS:1);
    xtoasc(i);
    ranlin();						/* random initial array */
    auxblnk();						/* uncomitted transitions */

    while (more!='n') {					/* execute multiple runs */
    rmenu();
    lmenu();
    while (0<1) {					/* set up one run */
    c=kbdin();
    if (c=='g') break;					/* go draw graph */
    if (c=='q') more='n';				/* quit for good */
    if (more=='n') break;
    switch (c) {
        case Kf1:					/* F1 = fetch stored rule */
//	    dij=dmenu(Kf1);
	    xtoasc(lim(0,dij,NX-1));
	    rmenu();
	    lmenu();
	    break;
        case Kf2:					/* F2 = rec program */
//	    rij=lim(1,recdem(Kf2),NY)-1;
//	    for (i=0; i<CLEN; i++) cstr[i]=recrule[rij][i];
	    rmenu();
	    lmenu();
	    break;
        case Kf3:					/* f3 = edit rec program */
//	    edrec(cstr); 
	    rmenu();
	    lmenu();
	    break;
        case Kf4:					/* f4 = execute rec program */
//	    if (rec_c(cstr,rprg,PLEN,dtbl)==1) rec_x(rprg);
	    break;
        case Kf5:					/* f5 = execute demo program */
//	    if (rec_c(&recrule[rdi],rprg,PLEN,dtbl)==1) rec_x(rprg);
	    break;

	case '*': sc=2; break;				/* white on red */
	case '+': sc=1; break;				/* white screen */
	case '-': sc=0; break;				/* blue screen */
	case '@':					/* numbered tot rule */
	    nutoto(numin(0));
	    totoasc();
	    rmenu();
	    videocursor(0,CM,0);
	    break;
	case '$':					/* dozen totalistics */
	    j=numin(0);
	    for (i=0; i<12; i++) {
	      nutoto(j+i);
	      totoasc();
	      ranlin();
	      evolve(0);
	      }
            videomode(T80X25);
	    rmenu();
	    lmenu();
	    break;
        case 'o': co=numin(0); break;			/* normal connectivity */
	case 'S':					/* semitotalistic rule */
	    xblnk();
	    smenu();
	    edsrule();
	    sttoasc();
	    videocursor(0,CM,0);
	    rmenu();
	    break;
	case 'T':					/* totalistic rule */
	    xblnk();
	    tmenu();
	    edtrule();
	    totoasc();
	    videocursor(0,CM,0);
	    rmenu();
	    break;
	case 't':					/* probability calculation */
	    edtri();
	    rmenu();
	    lmenu();
	    break;
        case 'r':					/* edit rule */	
	    xblnk();
	    edrule();
	    videocursor(0,CM,0);
	    rmenu();
	    break;
        case 'l':					/* edit cell string */
	    xblnk();
	    edline(8,40);
	    videocursor(0,CM,0);
	    lmenu();
	    break;
	case 'D':					/* run through samples */
	    for (i=0; i<NX; i++) {
	      xmenu(i);
	      xtoasc(i);
	      ranlin();
	      evolve(0);
	      }
	    videomode(T80X25);
	    rmenu();
	    break;
	case 'd':					/* de Bruijn diagram */
	    edijn();
	    rmenu();
	    lmenu();
	    break;
        case 'u':					/* sparse init arry */
	    xblnk();
	    for (i=0; i<AL; i++) arr1[i]=0;
	    arr1[AL/4]=1;
            arr1[AL/2]=1;
            arr1[(3*AL)/4]=1;
            arr1[(3*AL)/4+2]=1;
	    lmenu();
            break;
	case 'w':					/* Wolfram rulw # */
	    i=numin(0);
	    wmenu(i);
	    for (i0=0; i0<KK; i0++)
	    for (i1=0; i1<KK; i1++)
	    for (i2=0; i2<KK; i2++)
	    for (i3=0; i3<KK; i3++)
	    for (i4=0; i4<KK; i4++)
	    for (i5=0; i5<KK; i5++)
	    for (i6=0; i6<KK; i6++) {
	      ascrule[i0][i1][i2][i3][i4][i5][i6]='0'+i%KK;
	      i/=KK;
	      }
	    rmenu();
	    break;
	case 'x':					/* random rule */
	    xblnk();
	    for (i0=0; i0<KK; i0++)
	    for (i1=0; i1<KK; i1++)
	    for (i2=0; i2<KK; i2++)
	    for (i3=0; i3<KK; i3++)
	    for (i4=0; i4<KK; i4++)
	    for (i5=0; i5<KK; i5++)
	    for (i6=0; i6<KK; i6++) {
	      if (i == 0) i=random();
	      if (auxrule[i0][i1][i2][i3][i4][i5][i6]==' ') {
		ascrule[i0][i1][i2][i3][i4][i5][i6]='0'+i%KK;
		i/=KK;
		}
	      }
	    rmenu();
	    break;
	case 'X':					/* random rule */
	    xblnk();
	    ranrul();
	    rmenu();
	    break;
	case 'y':					/* random line */
	    xblnk();
	    ranlin();
            lmenu();
	    break;
	case 'Y':					/* symmetrize rule */
	    for (i0=0; i0<KK; i0++)
	    for (i1=0; i1<KK; i1++)
	    for (i2=0; i2<KK; i2++)
	    for (i3=0; i3<KK; i3++)
	    for (i4=0; i4<KK; i4++)
	    for (i5=0; i5<KK; i5++)
	    for (i6=0; i6<KK; i6++) {
	    ascrule[i6][i5][i4][i3][i2][i1][i0]=
		ascrule[i0][i1][i2][i3][i4][i5][i6];
	      }
	    break;
        case 'z':					/* zero initial arry */
	    xblnk();
	    for (i=0; i<AL; i++) arr1[i]=0;
	    lmenu();
            break;
	case '.':
	    asctobin();
	    onegen(AL);
	    lmenu();
	    break;
	case '=':
	    for (i=1; i<8;  i++) {
	    for (j=0; j<40; j++) arr1[40*i+j]=arr1[j];}
	    lmenu();
	    break;
	case '~':
	    for (i=1; i<16;  i++) {
	    for (j=0; j<20; j++) arr1[20*i+j]=arr1[j];}
	    lmenu();
	    break;
        default: break;
        }
    }
    if (more=='n') break;
    do {
    evolve(sc);
    videocursor(0,0,0);
    scrstr("?");
    videocursor(0,0,34);
    scrstr("y/n/cr");
    more=kbdin();
    } while (more=='\015');
    videomode(T80X25);					/* reset the screen */
    if (more=='n') break;
    }
  videomode(T80X25);}    

/* edit the rule */
edrule() {
char c;
int  i, i0, i1, i2, i3, i4;

i=6; i0=0; i1=0, i2=0; i3=0; i4=0;
asctohex();
while (0<1) {
  videocursor(0,CR,i);
  c=kbdin();
  if (c == '\015') break;				/* carriage return exits */
  switch (c) {
      case '0':  case '1': case '2': case '3':
      case '4':  case '5': case '6': case '7':
      case '8':  case '9': case 'a': case 'A':
      case 'b':  case 'B': case 'c': case 'C':
      case 'd':  case 'D': case 'e': case 'E':
      case 'f':  case 'F':				/* state */
	hexrule[i0][i1][i2][i3][i4] = c;
    	i4++;
	if (i4>KK-1) {i4=0; i3++;}
	if (i3>KK-1) {i3=0; i2++;}
	if (i2>KK-1) {i2=0; i1++;}
	if (i1>KK-1) {i1=0; i0++;}
	if (i0>KK-1) {i0=0; }
        videocattr(0,c,3,1);
        if (i<5+DS/4) i++; else i=6;
        break;
      case ' ': case '\315':				/* space = advance */
    	  i4++;
	  if (i4>KK-1) {i4=0; i3++;}
	  if (i3>KK-1) {i3=0; i2++;}
	  if (i2>KK-1) {i2=0; i1++;}
	  if (i1>KK-1) {i1=0; i0++;}
	  if (i0>KK-1) {i0=0; }
          if (i<5+DS/4) i++; else i=6;
          break;
      case '\010': case '\313':				/* backspace */
	  if (i4!=0) i4--; else {i4=KK-1;
	  if (i3!=0) i3--; else {i3=KK-1;
	  if (i2!=0) i2--; else {i2=KK-1;
	  if (i1!=0) i1--; else {i1=KK-1;
	  if (i0!=0) i0--; else {i0=KK-1;
	  }}}}}
          if (i>6) i--; else i=5+DS/4;
          break;
/* these won't normally be used */
    case '\013': case '\310':
	auxrule[ru0][ru1][ru2][ru3][ru4][ru5][ru6]='+'; break; /* up arrow */
    case '\012': case '\320':
	auxrule[ru0][ru1][ru2][ru3][ru4][ru5][ru6]=' '; break; /* down arrow */
      default: break;
      } /* end switch */
    }   /* end while  */
hextoasc();
}

/* change ascrule to hexrule */
asctohex() {int  i, i0, i1, i2, i3, i4;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++) {
  i=ascrule[i0][i1][i2][i3][i4][0][0]-'0';
  i=2*i+ascrule[i0][i1][i2][i3][i4][0][1]-'0';
  i=2*i+ascrule[i0][i1][i2][i3][i4][1][0]-'0';
  i=2*i+ascrule[i0][i1][i2][i3][i4][1][1]-'0';
  hexrule[i0][i1][i2][i3][i4]=(i<10?i+'0':i+'A'-10);      
  }
}

/* change hexrule to ascrule */
hextoasc() {
char c0, c1, c2, c3;
int  i0, i1, i2, i3, i4;
for (i0=0; i0<KK; i0++) {
for (i1=0; i1<KK; i1++) {
for (i2=0; i2<KK; i2++) {
for (i3=0; i3<KK; i3++) {
for (i4=0; i4<KK; i4++) {
  switch (hexrule[i0][i1][i2][i3][i4]) {
    case '0': c0='0'; c1='0'; c2='0'; c3='0'; break;
    case '1': c0='0'; c1='0'; c2='0'; c3='1'; break;
    case '2': c0='0'; c1='0'; c2='1'; c3='0'; break;
    case '3': c0='0'; c1='0'; c2='1'; c3='1'; break;
    case '4': c0='0'; c1='1'; c2='0'; c3='0'; break;
    case '5': c0='0'; c1='1'; c2='0'; c3='1'; break;
    case '6': c0='0'; c1='1'; c2='1'; c3='0'; break;
    case '7': c0='0'; c1='1'; c2='1'; c3='1'; break;
    case '8': c0='1'; c1='0'; c2='0'; c3='0'; break;
    case '9': c0='1'; c1='0'; c2='0'; c3='1'; break;
    case 'a': case 'A': c0='1'; c1='0'; c2='1'; c3='0'; break;
    case 'b': case 'B': c0='1'; c1='0'; c2='1'; c3='1'; break;
    case 'c': case 'C': c0='1'; c1='1'; c2='0'; c3='0'; break;
    case 'd': case 'D': c0='1'; c1='1'; c2='0'; c3='1'; break;
    case 'e': case 'E': c0='1'; c1='1'; c2='1'; c3='0'; break;
    case 'f': case 'F': c0='1'; c1='1'; c2='1'; c3='1'; break;
    default: break;
    }
  ascrule[i0][i1][i2][i3][i4][0][0]=c0;
  ascrule[i0][i1][i2][i3][i4][0][1]=c1;
  ascrule[i0][i1][i2][i3][i4][1][0]=c2;
  ascrule[i0][i1][i2][i3][i4][1][1]=c3;
  }}}}}
}

/* edit semitotalistic rule */
edsrule() {char c; int  i, j, k;
for (j=0; j<KK; j++) {
  i=0;
  while (i<TS-1) {
    c=strule[j][i];
    videocursor(0,CR-2+j,57+i);
    videocattr(0,c,3,1);
    c=kbdin();
    if (c=='\015') break;
    switch (c) {
      case '0': case '1':			/* state */
        strule[j][i]=c;
        videocattr(0,c,CR,1);
        i++;
        break;
      case ' ': case '\014': case '\315':	/* space = advance */
        i++;
        break;
      case '\010': case '\313':			/* backspace */
        if (i!=0) i--;
	break;
      case '\013': case '\310': if (j==0) j=KK; j--; break; /* up arrow */
      case '\012': case '\320': j++; if (j==KK) j=0; break; /* down arrow */
      case 'x':					/* random selection */
	for (i=0; i<TS-1; i++) {
	  if (i%8==0) k=random();
	  strule[j][i]='0'+k%KK; k/=KK;}
	i=0;
	sscrrul(j);
	break;
      default: break;
      } /* end switch */
  }     /* end while  */
}       /* end for    */
}

/* edit totalistic rule */
edtrule() {char c; int  i, k;
  i=0;
  while (i<TS) {
    c=ttrule[i];
    videocursor(0,3,56+i);
    videocattr(0,c,3,1);
    c=kbdin();
    if (c=='\015') break;
    switch (c) {
      case '0': case '1':				/* state */
        ttrule[i]=c;
        videocattr(0,c,CR,1);
        i++;
        break;
      case ' ': case '\014': case '\315':	/* space = advance */
        i++;
        break;
      case '\010': case '\313':			/* backspace */
        if (i!=0) i--;
	break;
      case 'x':					/* random selection */
	for (i=0; i<TS; i++) {
	  if (i%8==0) k=random();
	  ttrule[i]='0'+k%KK; k/=KK;}
	i=0;
	tscrrul();
	break;
      default: break;
      }
  }
}

/* edit the line of cells */
edline(nr,nc) int nr, nc; {
char c;
int  i, j, k, l, ll, m;
int  aa, bb, cc, dd, ee, ff, gg;
int  tt, uu, vv, ww, xx, yy, zz;

videocursor(0,20,0);
scrstr("insert states with 0, 1; move cursor");
videocursor(0,21,0);
scrstr("with keyboard arrows, space or backspace.");
videocursor(0,22,0);
scrstr("carriage return exits");
while (0<1) {
  m=0;					/* 0/1 = refresh/keep line */
  ll=lj;
  videocursor(0,lj+LROW,li);
  c=kbdin();
  if (c == '\015') {videoscroll(20,0,24,70,0,3); break;}
  switch (c) {
  case '0': case '1':					/* enter state */
    arr1[li+nc*lj]=c-'0';
    li++; if (li==nc) {li=0; lj++;} if (lj==nr) lj=0;
    break;
  case '\012': case '\320':				/* downwards */
    lj++; if (lj==nr) lj=0;
    break;
  case '\013': case '\310':				/* upwards */
    if (lj>0) lj--; else lj=nr-1;
    break;
  case '\014': case '\315': case ' ':			/* forward */
    li++; if (li==nc) {li=0; lj++;} if (lj==nr) lj=0;
    break;
  case '\010': case '\313':				/* backward */
    if (li>0) li--;
      else {li=nc-1; if (lj>0) lj--; else lj=nr-1;}
    break;
  case '<':						/* left margin */
    li=0;
    break;
  case '>':						/* right margin */
    li=nc-1;
    break;
  case 'z':						/* clear single row */
    for (k=0; k<nc; k++) arr1[k+nc*lj]=0;
    break;
  case 'Z':						/* clear full array */
    for (k=0; k<AL; k++) arr1[k]=0;
    lmenu();
    break;
  case '^':						/* copy line above */
    for (k=0; k<nc; k++)
      arr1[nc*lj+k]=arr1[nc*(lj-1)+k];
    break;
  case 'x':						/* unmark transitions */
    auxblnk();
    m=1;
    break;				
  case 'q':						/* clean up the area */
    ulmenu(3);
    m=1;
    break;
  case '=':						/* insert transition */
    if (lj==0) break;
    ww=li+nc*(lj-1);
    if ((li==0)||(li==1)||(li==2)) tt=ww+nc-3; else tt=ww-3;
    if ((li==0)||(li==1)) uu=ww+nc-2; else uu=ww-2;
    if (li==0) vv=ww+nc-1; else vv=ww-1;
    if (li==nc-1) xx=ww-nc+1; else xx=ww+1;
    if ((li==nc-1)||(li==nc-2)) yy=ww-nc+2; else yy=ww+2;
    if ((li==nc-1)||(li==nc-2)||(li==nc-3)) zz=ww-nc+3; else zz=ww+3;
    c=arr1[ww+nc]+'0';
    ascrule[arr1[tt]][arr1[uu]][arr1[vv]][arr1[ww]][arr1[xx]][arr1[yy]][arr1[zz]]=c;
    videocursor(0,lj+LROW,li);
    videocattr(0,c=='0'?'.':c,3,1);
    m=1;
    break;
  case '+':		/* set marker for transition in auxrule */
    if (lj==0) break;
    ww=li+nc*(lj-1);
    if ((li==0)||(li==1)||(li==2)) tt=ww+nc-3; else tt=ww-3;
    if ((li==0)||(li==1)) uu=ww+nc-2; else uu=ww-2;
    if (li==0) vv=ww+nc-1; else vv=ww-1;
    if (li==nc-1) xx=ww-nc+1; else xx=ww+1;
    if ((li==nc-1)||(li==nc-2)) yy=ww-nc+2; else yy=ww+2;
    if ((li==nc-1)||(li==nc-2)||(li==nc-3)) zz=ww-nc+3; else zz=ww+3;
    auxrule[arr1[tt]][arr1[uu]][arr1[vv]][arr1[ww]][arr1[xx]][arr1[yy]][arr1[zz]]='+';
    m=1;
    break;
  case '-':		/* remove marker for transition from auxrule */
    if (lj==0) break;
    ww=li+nc*(lj-1);
    if ((li==0)||(li==1)||(li==2)) tt=ww+nc-3; else tt=ww-3;
    if ((li==0)||(li==1)) uu=ww+nc-2; else uu=ww-2;
    if (li==0) vv=ww+nc-1; else vv=ww-1;
    if (li==nc-1) xx=ww-nc+1; else xx=ww+1;
    if ((li==nc-1)||(li==nc-2)) yy=ww-nc+2; else yy=ww+2;
    if ((li==nc-1)||(li==nc-2)||(li==nc-3)) zz=ww-nc+3; else zz=ww+3;
    auxrule[arr1[tt]][arr1[uu]][arr1[vv]][arr1[ww]][arr1[xx]][arr1[yy]][arr1[zz]]=' ';
    break;
  case '*':		/* insert transitions for the whole line */
    if (lj==0) break;
    for (i=0; i<nc; i++) {
      ww=i+nc*(lj-1);
      if ((i==0)||(i==1)||(i==2)) tt=ww+nc-3; else tt=ww-3;
      if ((i==0)||(i==1)) uu=ww+nc-2; else uu=ww-2;
      if (i==0) vv=ww+nc-1; else vv=ww-1;
      if (i==nc-1) xx=ww-nc+1; else xx=ww+1;
      if ((i==nc-1)||(i==nc-2)) yy=ww-nc+2; else yy=ww+2;
      if ((i==nc-1)||(i==nc-2)||(i==nc-3)) zz=ww-nc+3; else zz=ww+3;
      c=arr1[ww+nc]+'0';
      ascrule[arr1[tt]][arr1[uu]][arr1[vv]][arr1[ww]][arr1[xx]][arr1[yy]][arr1[zz]]=c;
      auxrule[arr1[tt]][arr1[uu]][arr1[vv]][arr1[ww]][arr1[xx]][arr1[yy]][arr1[zz]]=c;
      };
    m=1;
    break;
  case '.':		/* point evolution */
    if (lj==0) break;
    ww=li+nc*(lj-1);
    if ((li==0)||(li==1)||(li==2)) tt=ww+nc-3; else tt=ww-3;
    if ((li==0)||(li==1)) uu=ww+nc-2; else uu=ww-2;
    if (li==0) vv=ww+nc-1; else vv=ww-1;
    if (li==nc-1) xx=ww-nc+1; else xx=ww+1;
    if ((li==nc-1)||(li==nc-2)) yy=ww-nc+2; else yy=ww+2;
    if ((li==nc-1)||(li==nc-2)||(li==nc-3)) zz=ww-nc+3; else zz=ww+3;
    c=arr1[ww+nc]+'0';
    arr1[ww+nc]=ascrule[arr1[tt]][arr1[uu]][arr1[vv]][arr1[ww]][arr1[xx]][arr1[yy]][arr1[zz]]-'0';
    break;
  case '?':		/* evolution of the whole line */
    if (lj==nr-1) {for (i=0; i<AL-nc; i++) arr1[i]=arr1[i+nc]; lj--; lmenu(); };
    lj++; ll=lj;
    for (i=0; i<nc; i++) {
      ww=i+nc*(lj-1);
      if ((i==0)||(i==1)||(i==2)) tt=ww+nc-3; else tt=ww-3;
      if ((i==0)||(i==1)) uu=ww+nc-2; else uu=ww-2;
      if (i==0) vv=ww+nc-1; else vv=ww-1;
      if (i==nc-1) xx=ww-nc+1; else xx=ww+1;
      if ((i==nc-1)||(i==nc-2)) yy=ww-nc+2; else yy=ww+2;
      if ((i==nc-1)||(i==nc-2)||(i==nc-3)) zz=ww-nc+3; else zz=ww+3;
      c=arr1[ww+nc]+'0';
      arr1[ww+nc]=ascrule[arr1[tt]][arr1[uu]][arr1[vv]][arr1[ww]][arr1[xx]][arr1[yy]][arr1[zz]]-'0';
      };
    break;
  case '/':		/* conditional evolution */
    if (lj==nr-1) {for (k=0; k<AL-nc; k++) arr1[k]=arr1[k+nc]; lj--; lmenu(); };
    lj++; ll=lj;
    for (i=0; i<nc; i++) {
      j=0;
      ww=i+nc*(lj-1);
      if ((i==0)||(i==1)||(i==2)) tt=ww+nc-3; else tt=ww-3;
      if ((i==0)||(i==1)) uu=ww+nc-2; else uu=ww-2;
      if (i==0) vv=ww+nc-1; else vv=ww-1;
      if (i==nc-1) xx=ww-nc+1; else xx=ww+1;
      if ((i==nc-1)||(i==nc-2)) yy=ww-nc+2; else yy=ww+2;
      if ((i==nc-1)||(i==nc-2)||(i==nc-3)) zz=ww-nc+3; else zz=ww+3;
      c=arr1[ww+nc]+'0';
      aa=arr1[tt]; if (aa>KK-1) j=KK;
      bb=arr1[uu]; if (bb>KK-1) j=KK;
      cc=arr1[vv]; if (cc>KK-1) j=KK;
      dd=arr1[ww]; if (dd>KK-1) j=KK;
      ee=arr1[xx]; if (ee>KK-1) j=KK;
      ff=arr1[yy]; if (ff>KK-1) j=KK;
      gg=arr1[zz]; if (gg>KK-1) j=KK;
      if (auxrule[aa][bb][cc][dd][ee][ff][gg]==' ') j=KK;
      if (j==0) j=ascrule[aa][bb][cc][dd][ee][ff][gg]-'0';
      arr1[ww+nc]=j;
      };
    break;
  case 'c':		/* test consistency everywhere */
    for (i=0; i<nc; i++) {
      videocursor(0,LROW,i);
      k=arr1[i];
      videocattr(0,k==0?'.':k+'0',7,1);
      }
    for (j=1; j<nr;  j++) {
    for (i=0; i<nc; i++) {
      ww=i+nc*(j-1);
      if ((i==0)||(i==1)||(i==2)) tt=ww+nc-3; else tt=ww-3;
      if ((i==0)||(i==1)) uu=ww+nc-2; else uu=ww-2;
      if (i==0) vv=ww+nc-1; else vv=ww-1;
      if (i==nc-1) xx=ww-nc+1; else xx=ww+1;
      if ((i==nc-1)||(i==nc-2)) yy=ww-nc+2; else yy=ww+2;
      if ((i==nc-1)||(i==nc-2)||(i==nc-3)) zz=ww-nc+3; else zz=ww+3;
      c=arr1[ww+nc]+'0';
      aa=arr1[tt];
      bb=arr1[uu];
      cc=arr1[vv];
      dd=arr1[ww];
      ee=arr1[xx];
      ff=arr1[yy];
      gg=arr1[zz];
      k=arr1[ww+nc];
      videocursor(0,j+LROW,i);
      videocattr(0,k==0?'.':k+'0',k==ascrule[aa][bb][cc][dd][ee][ff][gg]-'0'?7:4,1);
      }}
    m=1;
    break;
  case 'C':		/* test consistency for marked neighborhoods */
    for (i=0; i<nc; i++) {
      videocursor(0,LROW,i);
      k=arr1[i];
      videocattr(0,k==0?'.':k+'0',7,1);
      }
    for (j=1; j<nr;  j++) {
    for (i=0; i<nc; i++) {
      ww=i+nc*(j-1);
      if ((i==0)||(i==1)||(i==2)) tt=ww+nc-3; else tt=ww-3;
      if ((i==0)||(i==1)) uu=ww+nc-2; else uu=ww-2;
      if (i==0) vv=ww+nc-1; else vv=ww-1;
      if (i==nc-1) xx=ww-nc+1; else xx=ww+1;
      if ((i==nc-1)||(i==nc-2)) yy=ww-nc+2; else yy=ww+2;
      if ((i==nc-1)||(i==nc-2)||(i==nc-3)) zz=ww-nc+3; else zz=ww+3;
      c=arr1[ww+nc]+'0';
      aa=arr1[tt];
      bb=arr1[uu];
      cc=arr1[vv];
      dd=arr1[ww];
      ee=arr1[xx];
      ff=arr1[yy];
      gg=arr1[zz];
      k=arr1[ww+nc];
      videocursor(0,j+LROW,i);
      videocattr(0,k==0?'.':k+'0',
        (k==ascrule[aa][bb][cc][dd][ee][ff][gg]-'0' && 
		auxrule[aa][bb][cc][dd][ee][ff][gg]!=' ')?7:4,1);
      }}
    m=1;
    break;
  default: break;
      } /* end switch */
  if (m==0) for (k=0; k<nc; k++) {
    videocursor(0,ll+LROW,k);
    l=arr1[k+nc*ll]+'0';
    videoputc(l=='0'?'.':l,1);
    }
  }     /* end while */
}       /* end edline */

/* calculate index, or rather the neighbors, in edline. */
linix(xx,i,j,nc) int xx[RR+1], i, j, nc; {int k, x;
  xx[(RR-1)/2]=x=i+nc*(j-1);
  if (i==0) xx[0]=x+nc-1; else xx[0]=x-1;
  if (i==nc-1) xx[2]=x-nc+1; else xx[2]=x+1;
  for (k=0; k<RR; k++) xx[k]=arr1[xx[k]];
  xx[RR]=arr1[x+nc];
}

/* display a screen of evolution */
void evolve(sc) int sc; {
int i, j;
  savearry();
  asctobin();
  for (j=0; j<200; j++) {		/* evolve for 200 generations */
    switch (sc) {
     case 0: for (i=0; i<AL; i++) /*serms((double)(2*i),(double)(2*j),arr1[i],3); */
            videodot(j,i,arr1[i]);
            break;
/*      case 1: for (i=0; i<AL; i++) videodot(j,i,arr1[i]*3); break; */
/*      case 2: for (i=0; i<AL; i++) videodot(j,i,arr1[i]+2); break; */
      default: break; }
    onegen(AL);
    }
  renewarry();
}

/* copy ascrule over to binrul */
asctobin() {int i0, i1, i2, i3, i4, i5, i6;
    for (i0=0; i0<KK; i0++)
    for (i1=0; i1<KK; i1++)
    for (i2=0; i2<KK; i2++)
    for (i3=0; i3<KK; i3++)
    for (i4=0; i4<KK; i4++)
    for (i5=0; i5<KK; i5++)
    for (i6=0; i6<KK; i6++)
      binrule[i0][i1][i2][i3][i4][i5][i6]=
	ascrule[i0][i1][i2][i3][i4][i5][i6]-'0';
}

/* ............................................................. */

/* clear auxrule to blanks */
auxblnk() {int i0, i1, i2, i3, i4, i5, i6;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++)
  auxrule[i0][i1][i2][i3][i4][i5][i6]=' ';
}

/* clear auxrule to zeroes */
void auxzero() {int i0, i1, i2, i3, i4, i5, i6;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++)
  auxrule[i0][i1][i2][i3][i4][i5][i6]=0;
}

/* clear ascrule to ascii zeroes */
void asczero() {int i0, i1, i2, i3, i4, i5, i6;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++)
  ascrule[i0][i1][i2][i3][i4][i5][i6]='0';
}

ranpred() {if (random()%100>49) return 0; else return 1;}

/* generate a random line of cells in arr1 */
ranlin() {int i, j;
for (i=0; i<AL; i++) {if (i%8 == 0) j=random(); arr1[i]=j%KK; j/=KK;}}

/* generate line of cells in arr1 with n mils of 1's */
rprob(n) int n; {int i; for (i=0; i<AL; i++) arr1[i]=(random()%1000<n)?1:0;}

/* generate line of n cells in character array a with p mils of 1's */
ranchay(a,p,n) char *a; int p, n; {int i;
  for (i=0; i<n; i++) a[i]=(random()%1000<p)?'1':'0';}

/* generate line of n cells in character array a with p mils of 1's */
ranblan(a,n) char *a; int n; {int i; for (i=0; i<n; i++) a[i]=' ';}

/* generate a random rule in ascrule */
ranrul() {int i, i0, i1, i2, i3, i4, i5, i6;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
  for (i4=0; i4<KK; i4++) {
    if ((KK*(KK*(KK*(KK*i0+i1)+i2)+i3)+i4)%4 == 0) i=random();
  for (i5=0; i5<KK; i5++) {
  for (i6=0; i6<KK; i6++) {
    ascrule[i0][i1][i2][i3][i4][i5][i6]='0'+i%KK;
    i/=KK;
  }} }
}

/* tutorial and Help screen */
tuto() {
    videomode(T80X25);
    videocursor(0,2,0);
    scrstr("<Copyright (C) 1987, 1988, 1989 - H.V. McIntosh, G. Cisneros S.>");
    videocursor(0,4,0);
    scrstr("           ******* LIFE in One Dimension *******");
    videocursor(0,6,0);
    scrstr("Two States - Black(0), Cyan(1).");
    videocursor(0,8,0);
    scrstr("Third neighbors - three on each side, seven altogether.");
    videocursor(0,10,0);
    scrstr("Complete transition rule - random, edited, or stored.");
    videocursor(0,12,0);
    scrstr("Initial Cellular Array - random, edited, or patterned.");
    videocursor(0,14,0);
    scrstr("Submenus in options t(probabilities) and d(de Bruijn)");
    videocursor(0,15,0);
    scrstr("will be displayed in response to typing ?");
    videocursor(0,17,0);
    scrstr("Use any key to terminate a display in progress.");
    videocursor(0,21,0);
    scrstr("Now, ... press any key to continue.");
}

/* rule menu */
rmenu() {
    videocursor(0,CR-5,0);
    scrstr("      00000000000000001111111111111111");
    videocursor(0,CR-4,0);
    scrstr("      00000000111111110000000011111111");
    videocursor(0,CR-3,0);
    scrstr("      00001111000011110000111100001111");
    videocursor(0,CR-2,0);
    scrstr("      00110011001100110011001100110011");
    videocursor(0,CR-1,0);
    scrstr("      01010101010101010101010101010101");
    videocursor(0,CR,0);
    scrstr("Rule: ");
    scrrul();
    videocursor(0,CR,42);
    printf("         ");
    videocursor(0,CR,42);
    if (istot()==1) printf("(T:%2d)",totonu(0));
    videocursor(0,CR+1,42);
    printf("         ");
    videocursor(0,CR+1,42);
    if (isstot()==1) printf("(S:%2d,%2d)",stotonu(0,0),stotonu(0,1));
    videocursor(0,CM,0);
    scrstr("    r(rule), l(line), #nn(stored), g(graph), q(quit), t(prob),");
    videocursor(0,CM+1,0);
    scrstr("         x(rand rule), y(rand line), u(unit line), d(deBruijn)");
    videocursor(0,CM+2,0);
    scrstr("         @nn(tot/rule), $nn(12 tot/rules), T(ed tot/rule).");
    videocursor(0,CM,0);
    }

/* semitotalistic rule menu*/
smenu() {
    videocursor(0,CR-3,50); scrstr("       0123456");
    videocursor(0,CR-2,50); scrstr("born:  "); sscrrul(0);
    videocursor(0,CR-1,50); scrstr("live:  "); sscrrul(1);
    videocursor(0,CR-2,57);
    }

/* totalistic rule menu*/
tmenu() {
    videocursor(0,CR-3,50); scrstr("      01234567");
    videocursor(0,CR-2,50); scrstr("rule: "); tscrrul();
    videocursor(0,CR-2,56);
    }

/* line menu */
lmenu() {int i, j; char c;
  for (j=0; 40*j<YL; j++) {
    videocursor(0,CL+j,0);
    for (i=0; i<40; i++) {
      c=arr1[40*j+i]+'0';
      videoputc(c=='0'?'.':c,1);
      }
    }
  videocursor(0,CM,0);
  }

/* report Wolfram rule number or variant */
int wolnum(c) char c; {
int  k, i0, i1, i2, i3, i4, i5, i6;
  k=0;
  switch (c) {
    case 'w':
      for (i6=KK-1; i6>=0; i6--)
      for (i5=KK-1; i5>=0; i5--)
      for (i4=KK-1; i4>=0; i4--)
      for (i3=KK-1; i3>=0; i3--)
      for (i2=KK-1; i2>=0; i2--)
      for (i1=KK-1; i1>=0; i1--)
      for (i0=KK-1; i0>=0; i0--)
        k=2*k+(ascrule[i0][i1][i2][i3][i4][i5][i6]-'0');
      break;
    case 'c':
      for (i6=KK-1; i6>=0; i6--)
      for (i5=KK-1; i5>=0; i5--)
      for (i4=KK-1; i4>=0; i4--)
      for (i3=KK-1; i3>=0; i3--)
      for (i2=KK-1; i2>=0; i2--)
      for (i1=KK-1; i1>=0; i1--)
      for (i0=KK-1; i0>=0; i0--)
        k=2*k+((ascrule[(i0+1)%KK][(i1+1)%KK][(i2+1)%KK][(i3+1)%KK][(i4+1)%KK][(i5+1)%KK][(i6+1)%KK]-'0')+1)%KK;
      break;
    case 'r':
      for (i6=KK-1; i6>=0; i6--)
      for (i5=KK-1; i5>=0; i5--)
      for (i4=KK-1; i4>=0; i4--)
      for (i3=KK-1; i3>=0; i3--)
      for (i2=KK-1; i2>=0; i2--)
      for (i1=KK-1; i1>=0; i1--)
      for (i0=KK-1; i0>=0; i0--)
        k=2*k+(ascrule[i6][i5][i4][i3][i2][i1][i0]-'0');
      break;
    case 'b':
      for (i6=KK-1; i6>=0; i6--)
      for (i5=KK-1; i5>=0; i5--)
      for (i4=KK-1; i4>=0; i4--)
      for (i3=KK-1; i3>=0; i3--)
      for (i2=KK-1; i2>=0; i2--)
      for (i1=KK-1; i1>=0; i1--)
      for (i0=KK-1; i0>=0; i0--)
        k=2*k+((ascrule[(i6+1)%KK][(i5+1)%KK][(i4+1)%KK][(i3+1)%KK][(i2+1)%KK][(i1+1)%KK][(i0+1)%KK]-'0')+1)%KK;
      break;
    default: k=0; break;}
        return k;
}

/* display Wolfram rule number */
wmenu(n) int n;
{char xx[4]; int i, nn;  
    nn=sprintf(xx,"%3d",n);
    videocursor(0,2,40);
    for (i=0; i<nn; i++) videoputc(xx[i],1);
    videocursor(0,2,40); }

/* display rule number */
xmenu(n) int n;
{char xx[4]; int i, nn;  
    nn=sprintf(xx,"%3d",n);
    videocursor(0,1,40);
    for (i=0; i<nn; i++) videoputc(xx[i],1);
    videocursor(0,1,40); }

/* clear screen space for rule numbers */
xblnk() {
    videocursor(0,CR-5,40);
    scrstr("           ");
    videocursor(0,CR-4,40);
    scrstr("           ");
    videocursor(0,CR-3,40);
    scrstr("                ");
    videocursor(0,CR,0); }

/* display the rule number on the screen in glorious technicolor */
cscrrul() {int i, i0, i1, i2, i3, i4, i5, i6;
  i=6;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
  for (i4=0; i4<KK; i4++)
  for (i5=0; i5<KK; i5++)
  for (i6=0; i6<KK; i6++) {
  videocursor(0,3,i++);
  videocattr(0,ascrule[i0][i1][i2][i3][i4][i5][i6],
	auxrule[i0][i1][i2][i3][i4][i5][i6]==' '?3:6,1);
  } }

/* uniform line menu */
ulmenu(l) int l; {int i, j, k;
  for (j=0; j*SL<YL; j++) {
  for (i=0; i<SL;    i++) {
    videocursor(0,LROW+j,i);
    k=arr1[j*SL+i];
    videocattr(0,k==0?'.':k+'0',l,1);
    }}
  }

/* copy saved rule #n into active rule */
xtoasc(n) int n; {int i0, i1, i2, i3, i4, i5, i6;
  xmenu(n+1);
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
  for (i4=0; i4<KK; i4++)
  for (i5=0; i5<KK; i5++)
  for (i6=0; i6<KK; i6++)
  ascrule[i0][i1][i2][i3][i4][i5][i6]=xrule[n][i0][i1][i2][i3][i4][i5][i6];		/* random sample rule */
}

/* change semitotalistic rule to general rule */
sttoasc() {int i0, i1, i2, i3, i4, i5, i6;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++)
ascrule[i0][i1][i2][i3][i4][i5][i6]=strule[i3][i0+i1+i2+i4+i5+i6];
}

/* change totalistic rule to general rule */
totoasc() {int i0, i1, i2, i3, i4, i5, i6;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++)
ascrule[i0][i1][i2][i3][i4][i5][i6]=ttrule[i0+i1+i2+i3+i4+i5+i6];
}

/* find the totalistic rule number of a known ascii totalistic rule */
void asctotot() {int i0, i1, i2, i3, i4, i5, i6;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++)
      ttrule[i0+i1+i2+i3+i4+i5+i6] = ascrule[i0][i1][i2][i3][i4][i5][i6];
}

/* change decimal totalistic rule to sum values */
nutoto(x) int x; {int i; for (i=0; i<TS; i++) {ttrule[i]=x%KK+'0'; x/=KK;}}

/* change sum values to decimal semitotalistic rule */
int stotonu(i,j) int i, j; {int r;
 r=0; if (i<TS-1) r=(strule[j][i]-'0')+KK*stotonu(i+1,j);
  return r; }

/* change sum values to decimal totalistic rule */
int totonu(i) int i; {int r;
 r=0; if (i<TS) r=(ttrule[i]-'0')+KK*totonu(i+1);
  return r; }

/* test whether a rule is semi totalistic */
int isstot() {
char sr[KK][TS-1];
int  i0, i1, i2, i3, i4, i5, i6, l;
    l=1;
    sr[0][0]=ascrule[0][0][0][0][0][0][0];
    sr[0][1]=ascrule[1][0][0][0][0][0][0];
    sr[0][2]=ascrule[1][1][0][0][0][0][0];
    sr[0][3]=ascrule[1][1][1][0][0][0][0];
    sr[0][4]=ascrule[1][1][1][0][1][0][0];
    sr[0][5]=ascrule[1][1][1][0][1][1][0];
    sr[0][6]=ascrule[1][1][1][0][1][1][1];
    sr[1][0]=ascrule[0][0][0][1][0][0][0];
    sr[1][1]=ascrule[1][0][0][1][0][0][0];
    sr[1][2]=ascrule[1][1][0][1][0][0][0];
    sr[1][3]=ascrule[1][1][1][1][0][0][0];
    sr[1][4]=ascrule[1][1][1][1][1][0][0];
    sr[1][5]=ascrule[1][1][1][1][1][1][0];
    sr[1][6]=ascrule[1][1][1][1][1][1][1];
    for (i0=0; i0<KK; i0++)
    for (i1=0; i1<KK; i1++)
    for (i2=0; i2<KK; i2++)
    for (i3=0; i3<KK; i3++)
    for (i4=0; i4<KK; i4++)
    for (i5=0; i5<KK; i5++)
    for (i6=0; i6<KK; i6++)
    if (ascrule[i0][i1][i2][i3][i4][i5][i6]!=sr[i3][i0+i1+i2+i4+i5+i6]) l=0;
    return l; }

/* test whether a rule is totalistic */
int istot() {
char tr[TS];
int  i0, i1, i2, i3, i4, i5, i6, l;
    l=1;
    tr[0]=ascrule[0][0][0][0][0][0][0];
    tr[1]=ascrule[1][0][0][0][0][0][0];
    tr[2]=ascrule[1][1][0][0][0][0][0];
    tr[3]=ascrule[1][1][1][0][0][0][0];
    tr[4]=ascrule[1][1][1][1][0][0][0];
    tr[5]=ascrule[1][1][1][1][1][0][0];
    tr[6]=ascrule[1][1][1][1][1][1][0];
    tr[7]=ascrule[1][1][1][1][1][1][1];
    for (i0=0; i0<KK; i0++)
    for (i1=0; i1<KK; i1++)
    for (i2=0; i2<KK; i2++)
    for (i3=0; i3<KK; i3++)
    for (i4=0; i4<KK; i4++)
    for (i5=0; i5<KK; i5++)
    for (i6=0; i6<KK; i6++)
    if (ascrule[i0][i1][i2][i3][i4][i5][i6]!=tr[i0+i1+i2+i3+i4+i5+i6]) l=0;      
    return l; }

/* generate a random core for a planar array */
ranplacor() {int i, j;
  for (i=60; i<69; i++) for (j=60; j<69; j++)  arr1[ix2(i,j)] = random()%KK;}

/* generate a random core for a solid array */
ransolcor() {int i, j, k;
  for (i=14; i<19; i++) for (j=14; j<19; j++) for (k=7; k<10; k++) 
	 arr1[ix3(i,j,k)] = random()%KK;}

/* make all of arr1 equal to zero */
void zerline() {int i; for (i=0; i<AL; i++) arr1[i]=0;}

/* put a single i-cell on a line of j-cells */
void putionj(i,j) int i, j; 
{int k; for (k=0; k<AL; k++) arr1[k]=j; arr1[AL/2]=i;}

/* limit j to interval (i,k) */
int lim(i,j,k) int i, j, k;
    {if (i>=j) return i; if (k<=j) return k; return j;}

/* display the rule number on the screen */
scrrul() {int i, i0, i1, i2, i3, i4, i5, i6;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
  for (i4=0; i4<KK; i4++) {
    i=0;
    for (i5=0; i5<KK; i5++) {
    for (i6=0; i6<KK; i6++) {
    i=KK*i+ascrule[i0][i1][i2][i3][i4][i5][i6]-'0';
    }} 
  videoputc(i<10?i+'0':i+'A'-10,1);      
  } }

/* turn ascrule into its Wolfram number and place on screen at (i,j) */
woruno(i,j) int i, j; {
char xx[12];
int  k, l, i0, i1, i2, i3, i4, i5, i6;
  k=0;
  for (i0=KK-1; i0>=0; i0--)
  for (i1=KK-1; i1>=0; i1--)
  for (i2=KK-1; i2>=0; i2--)
  for (i3=KK-1; i3>=0; i3--)
  for (i4=KK-1; i4>=0; i4--)
  for (i5=KK-1; i5>=0; i5--)
  for (i6=KK-1; i6>=0; i6--)
    k=KK*k+(ascrule[i0][i1][i2][i3][i4][i5][i6]-'0');
  k=sprintf(xx,"%10d",k);
  videocursor(0,i,j);
  for (l=0; l<k; l++) videoputc(xx[l],1);
}

void rule33(){
    ranrul();
    ranlin();					/* random initial array */
}

void savearry() {int i; for (i=0; i<AL; i++) arr3[i]=arr1[i];}
void renewarry() {int i; for (i=0; i<AL; i++) arr1[i]=arr3[i];}

int  bdos(){}

/* end nxlcau33.c */
