
/* RECT31.C					*/
/* Two-Dimensional Anisotropic Square Lattice	*/
/* three-state cells with seven neighbors	*/
/*						*/
/* Reference:					*/
/*						*/
/* Harold V. McIntosh, 16 May 1987		*/
/* 7 July 1987 - separate rules for each state	*/
/* 8 July 1988 - binary, 3 neighborhood classes	*/
/* 9 July 1988 - hexagonal lattice 		*/

# import  "nxlcau33.h"
# include <math.h>
# import  <dpsclient/psops.h>
# import  <appkit/graphics.h>

/* parameters related to arrays and automaton characteristics	*/

# define GH	 DS/KK  /* "half" the number of rules		*/
# define QL	    80  /* edge of square of evolution		*/
# define BPD	     7	/* degree of Bernstein polynomial	*/
# define NWOC	    13  /* number of symmetry classes w/o ctr	*/
# define TWIC	 NWOC*KK  /* number of symmetry classes w/ cntr	*/

/* coordinates of significant objects on the display screen 	*/

extern char   ascrule[KK][KK][KK][KK][KK][KK][KK];
extern char   auxrule[KK][KK][KK][KK][KK][KK][KK];
extern  int   binrule[KK][KK][KK][KK][KK][KK][KK];

char    trtrule[DS];	/* evolutionary rule       */
char    artrule[DS];	/* auxiliary rule flags    */
int     nr[NWOC];	/* nbrhd representative of symm class */
int     rs[TWIC];	/* rule use statistics     */
int	rtlf[KK];	/* number of live cells	   */
int     rtstus[KK*(TS-1)];/* s-totalistic use stats  */
int     rtgus[2*NWOC];	/* golay use statistics    */
int     rtrus[DS];	/* rule use statistics     */
int     rttus[TS];	/* totalistic use stats    */
int     srttus[TS];	/* stotalistic use stats   */
int     gotb[DS];	/* Golay code key array    */
int     wc[BPD+1];	/* Bernstein coefficients */
double  bp[BPD+1];	/* Bernstein coefficients */

extern int     tc[DS];	/* totalistic key array	   */
extern int     stc[DS];	/* s-totalistic key array  */

mainrect(c) char c; {}	   /* end main */

void ascintorect() {int i0, i1, i2, i3, i4, i5, i6;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
  for (i4=0; i4<KK; i4++)
  for (i5=0; i5<KK; i5++)
  for (i6=0; i6<KK; i6++)
    trtrule[i0+KK*(i1+KK*(i2+KK*(i3+KK*(i4+KK*(i5+KK*i6)))))] = 
	ascrule[i0][i1][i2][i3][i4][i5][i6];
}

void rectintoasc() {int i0, i1, i2, i3, i4, i5, i6;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
  for (i4=0; i4<KK; i4++)
  for (i5=0; i5<KK; i5++)
  for (i6=0; i6<KK; i6++)
    ascrule[i0][i1][i2][i3][i4][i5][i6] = 
	trtrule[i0+KK*(i1+KK*(i2+KK*(i3+KK*(i4+KK*(i5+KK*i6)))))];
}

/* 	     layout for cell indexing		*/
/*						*/
/*		 |				*/
/*		 0    				*/
/*      - -  1 4 6 5 3 - -  	       		*/
/*	         2				*/
/*		 |				*/
/*						*/

/* calculate one generation of evolution */
onerectgen(ar,br,m,n) char ar[]; int br[]; int m, n; {int i0, i1, gn;

/* count neighbors of center square */

ascintorect();

for (i0=0; i0<m; i0++) for (i1=0; i1<n; i1++) {
	    gn=      ar[indx2(i0-1, i1,   m,n)];
	    gn=KK*gn+ar[indx2(i0,   i1-2, m,n)];
	    gn=KK*gn+ar[indx2(i0+1, i1,   m,n)];
	    gn=KK*gn+ar[indx2(i0,   i1+2, m,n)];
	    gn=KK*gn+ar[indx2(i0,   i1-1, m,n)];
	    gn=KK*gn+ar[indx2(i0,   i1+1, m,n)];
	    gn+=  GH*ar[indx2(i0,   i1,   m,n)];
  br[indx2(i0,i1,m,n)]=gn;
  }

/* phase to update cells */
for (i0=0; i0<KK; i0++)        rtlf[i0] =0;
for (i0=0; i0<DS; i0++)        rtrus[i0]=0;
// for (i0=0; i0<TS; i0++)        rttus[i0]=0;
// for (i0=0; i0<KK*(TS-1); i0++) rtstus[i0]=0;
// for (i0=0; i0<2*NG; i0++)   rtgus[i0]=0;
for (i0=0; i0<m; i0++) for (i1=0; i1<n; i1++) {
  gn=br[indx2(i0,i1,m,n)];
  ar[indx2(i0,i1,m,n)]=trtrule[gn]-'0';
  rtlf[ar[indx2(i0,i1,m,n)]]+=1;
//  rtstus[stc[gn]]+=1;
//  gus22[gotb[gn]]+=1;
//  rttus[tc[gn]]+=1;
  rtrus[gn]++;
  }
/* phase to show results */
}	/* end onerectgen */
    
int indx2(i,j,m,n) int i, j, m, n; {return((n+j)%n+n*((m+i)%m));}

/*find fraction of live cells in arry[m][n] */
void rtstatecount(aa,m,n) char *aa; int m, n; {int i0, i1;
  for (i0=0; i0<KK; i0++) rtlf[i0]=0;
  for (i0=0; i0<m; i0++) for (i1=0; i1<n; i1++) rtlf[aa[n*i0+i1]]+=1;
}

/* display a screen of evolution */
void rtscreen(ar,m,n) char ar[], m, n; {int i0, i1;
  for (i1=0; i1<n; i1++) for (i0=0; i0<m; i0++) rtbdot(i0,i1,ar[n*i0+i1]);}

/* display a screen of evolution */
void rtboolcomp(ar,m,n) char ar[], m, n; {int i0, i1;
  for (i1=0; i1<n; i1++) for (i0=0; i0<m; i0++) ar[n*i0+i1]=(ar[n*i0+i1]+1)%2;}

/* make a frame for the fixpoint graph  */
/* n     = dimension of arframe         */
/* l     = color of fiducial marks      */
rtgfram(n,l) int n, l; {int i;
for (i=0; i<=n; i++) {rtbdot(i,0,0);rtbdot(i,n,0);rtbdot(n,i,0);rtbdot(0,i,0);};
for (i=0; i<=n; i+=10) {rtbdot(i,0,l);rtbdot(i,n,l);rtbdot(n,i,l);rtbdot(0,i,l);};
}

/* put a diagonal in the fixpoint graph */
rtgdiag(n,l) int n, l; {int i; for (i=0; 2*i<n; i++) rtbdot(2*i,2*i,l);}

/* graph statistics of rule usage */
/* beware limited count capacity  */
/* large counts look negative     */
rrtstat() {int i, j, m, n; double s, t;
for (i=0, s=0.0; i<TWIC; i++) s+=(double)rs[i];
if  (s<1.0) {for (i=0; i<TWIC; i++) rs[i]=0; return;};
for (i=0; i<TWIC; i++) {
  t=333.3*(((double)rs[i])/s); /* full scale = 1/10 population */
  if (t>32000.0) t=50.0;
  n=(int)t;
  rtrule(i,33,0);
  m=33; if (n>33) {n=33; m=34;};
  if (n>0) for (j=0; j<n; j++) rtrule(i,j,1);
  for (j=n; j<m; j++) rtrule(i,j,2);
  rs[i]=0;
  };
}

/* generate the Golay code table */
rtgolta() {
int i, j, k;
int i0, i1, i2, i3, i4, i5;
k=0;
for (i5=0; i5<KK; i5++)
for (i4=0; i4<KK; i4++)
for (i3=0; i3<KK; i3++)
for (i2=0; i2<KK; i2++)
for (i1=0; i1<KK; i1++)
for (i0=0; i0<KK; i0++) {
  i=binq(i0,i1,i2,i3,i4,i5);
  j=binq(i1,i2,i3,i4,i5,i0);
  if (j<i) i=j;
  j=binq(i2,i3,i4,i5,i0,i1);
  if (j<i) i=j;
  j=binq(i3,i4,i5,i0,i1,i2);
  if (j<i) i=j;
  j=binq(i4,i5,i0,i1,i2,i3);
  if (j<i) i=j;
  j=binq(i5,i0,i1,i2,i3,i4);
  if (j<i) i=j;
  j=binq(i5,i4,i3,i2,i1,i0);
  if (j<i) i=j;
  j=binq(i4,i3,i2,i1,i0,i5);
  if (j<i) i=j;
  j=binq(i3,i2,i1,i0,i5,i4);
  if (j<i) i=j;
  j=binq(i2,i1,i0,i5,i4,i3);
  if (j<i) i=j;
  j=binq(i1,i0,i5,i4,i3,i2);
  if (j<i) i=j;
  j=binq(i0,i5,i4,i3,i2,i1);
  if (j<i) i=j;
  j=binq(i0,i1,i2,i3,i4,i5);
  if (i==j) {gotb[i]=k; nr[k]=j; k++;} else {gotb[j]=gotb[i];};
  /*
  printf("  %2X, gotb[%3d]=%3d, k=%3d, (%d,%d)\n",j,j,i,k,i0+i2+i4,i1+i3+i5);
  */
  }
  for (i=1; i<KK; i++) for (j=0; j<DS/KK; j++) gotb[i*(DS/KK)+j]=gotb[j];
printf("no. hex classes = %3d\n",k);
// for (i=0; i<DS; i++) printf("  %d",gotb[i]);
// printf("\n");
}

/* calculate binary equivalent */
int rtbinq(i0,i1,i2,i3,i4,i5)
int i0, i1, i2, i3, i4, i5; {
return i0+KK*(i1+KK*(i2+KK*(i3+KK*(i4+KK*i5))));}

/* calculate Golay class of a cell */
int rtgolcl(i0,i1,ar,m,n) int  i0, i1, m, n; char ar[]; {int gn;
  gn=ar[m*(i0-1)+i1];
  gn=KK*gn+ar[m*(i0-1)+i1-1];
  gn=KK*gn+ar[m*(i0)+i1-1];
  gn=KK*gn+ar[m*(i0+1)+i1];
  gn=KK*gn+ar[m*(i0+1)+i1+1];
  gn=KK*gn+ar[m*(i0)+i1+1];
  if (ar[m*(i0)+i1]==1) gn+=GH;
  gn=gotb[gn];
  return gn;
}

/* calculate the image of a neighborhood */
int rtgolcln(i0,i1,i2,i3,i4,i5,i6) int  i0, i1, i2, i3, i4, i5, i6;
{return trtrule[gotb[(((((i0*KK+i1)*KK+i2)*KK+i3)*KK+i4)*KK+i5)*KK+i6]]-'0';}

/* turn the golay rule number into an ascii rule number */
rtgoltoasc() {
int i0, i1, i2, i3, i4, i5, i6;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
  for (i4=0; i4<KK; i4++)
  for (i5=0; i5<KK; i5++)
  for (i6=0; i6<KK; i6++)
    ascrule[i0][i1][i2][i3][i4][i5][i6]=golcln(i0,i1,i2,i3,i4,i5,i6)+'0';
}

/* graph Bernstein polynomial */
brtgraf(k,n) int k, n; {int i; double rectbern(), en, p;
en=(double)(n);
for (i=0; i<n; i++) {
  p=((double)(i))/en;
  rtbdot(i,(int)(en*rectbern(p,k)),1);
  }
}

/* evaluate the nth degree Bernstein polynomial at point p */
double rectbern(p,n) double p; int n; {double q, s, x, r; int i;
if (p>0.99) return bp[n];
q=1.0-p; r=p/q; s=0.0; x=1.0;
for (i=0; i<n;  i++) x*=q;
for (i=0; i<=n; i++, x*=r) s+=bp[i]*x;
s*=0.9998;
return(s+0.0001);
}

/* rtbdot(i,j,l) - place dot on Bernstein screen at (i,j) in color l */
rtbdot(i,j,l) int i, j, l; {videodot(j,i,l);}

/* rtbline(x,y,l) - place line on Bernstein screen to (x,y); l=first */
rtbline(x,y,l) double x, y; int l; 
	{if (l==0) PSmoveto(y,150.0-x); else PSlineto(y,150.0-x);}

/* rtsdot(i,j,l) - place dot on statistics screen at (i,j) in color l */
rtrule(i,j,l) int i, j, l; {videodot(199-10-j,25+4*i,l);}

/* display Wolfram rule number of planar cross section */
/* calculate LCAU21 wru from a HEXL21 argument rule    */
void wrtsscrrul(c,ru) char *c, ru[KK][KK][KK][KK][KK][KK][KK]; {
char ur[KK][KK][KK]; int i0, i1, i2;
  for (i0=0; i0<KK; i0++) for (i1=0; i1<KK; i1++) for (i2=0; i2<KK; i2++)
    ur[i0][i1][i2]=ru[i0][i1][i2][i2][i1][i0][i1];
  wrru21(c,ur);
  }

/* 	     layout for cell indexing		*/
/*						*/
/*	     a			  |		*/
/*	 b c d e f	   	  0    		*/
/*   g h i j k l m n o   - -  1 4 6 5 3 - -  	*/
/*	 p q r s t  	          2		*/
/*	     u			  |	 	*/
/*						*/

/* enumeration of idempotent RECT rules */
void idemp() {
int na, nb, nc, nd, ne, nf, ng, nh, ni, nj;
int nk, nl, nm, nn, no, np, nq, nr, ns, nt, nu;
int jj[RR];
int i, j, kk;
double all, base;
double ndx[RR], idx[RR], sum[RR], sqr[RR];
double avsum[RR], avsqr[RR], devia[RR]; 
base=0.0;
for (i=0; i<RR; i++) {sum[i]=0.0; sqr[i]=0.0;}
// for (j=0; j<256; j++) {
for (j=101; j<134; j++) {
  base+=1.0;
  nutoto(j);
  totoasc();
  asctobin();
  all=0.0;
for (i=0; i<RR; i++) idx[i]=0.0;
for (na=0; na<KK; na++) 
for (nb=0; nb<KK; nb++) 
for (nc=0; nc<KK; nc++) 
for (nd=0; nd<KK; nd++) 
for (ne=0; ne<KK; ne++) 
for (nf=0; nf<KK; nf++) 
for (ng=0; ng<KK; ng++) 
for (nh=0; nh<KK; nh++) 
for (ni=0; ni<KK; ni++) 
for (nj=0; nj<KK; nj++) 
for (nk=0; nk<KK; nk++) 
for (nl=0; nl<KK; nl++) 
for (nm=0; nm<KK; nm++) 
for (nn=0; nn<KK; nn++) 
for (no=0; no<KK; no++) 
for (np=0; np<KK; np++) 
for (nq=0; nq<KK; nq++) 
for (nr=0; nr<KK; nr++) 
for (ns=0; ns<KK; ns++) 
for (nt=0; nt<KK; nt++) 
for (nu=0; nu<KK; nu++) {
	jj[0]=binrule[na][nb][nk][nf][nc][ne][nd];
	jj[1]=binrule[nb][ng][np][nk][nh][nj][ni];
	jj[2]=binrule[nc][nh][nq][nl][ni][nk][nj];
	jj[3]=binrule[nd][ni][nr][nm][nj][nl][nk];
	jj[4]=binrule[ne][nj][ns][nn][nk][nm][nl];
	jj[5]=binrule[nf][nk][nt][no][nl][nn][nm];
	jj[6]=binrule[nk][np][nu][nt][nq][ns][nr];
	      kk=binrule[jj[0]][jj[1]][jj[6]][jj[5]][jj[2]][jj[4]][jj[3]];
	all+=1.0;
	for (i=0; i<RR; i++) if (kk==jj[i]) idx[i]+=1.0;
        }
for (i=0; i<RR; i++) {
  ndx[i]=idx[i]/all;
  sum[i]+=ndx[i];
  sqr[i]+=ndx[i]*ndx[i];
  }
printf("idempotency indices for RECT totalistic rule %3d:\n",j);
for (i=0; i<RR; i++) printf("index %d = %6.3f, ",i,ndx[i]);
printf("\n");
  }
for (i=0; i<RR; i++) {
  avsum[i]=sum[i]/base;
  avsqr[i]=sqr[i]/base;
  devia[i]=sqrt(avsqr[i]-avsum[i]*avsum[i]);
  }
printf("\n");
for (i=0; i<RR; i++) printf("%d: average = %6.3f, deviation = %6.3f\n",
	i,avsum[i],devia[i]);
printf("\n");
}

/* end nxrect21.c */
