
/* nxrijn33.c						*/
/* program for lcau23 option 'd'			*/
/* display de Bruijn diagram - 1st, 2nd, & 3rd gen	*/
/* Harold V. McIntosh, 20 February 1988			*/
/* 21 February 1988 - add j0=i0; to Pass 4's		*/
/* 3 March 1988 - adapted from lca21.c to lca22.c	*/
/* 12 March 1989 - adapted from lca22.c to lca23.c	*/
/* 26 September 1989 - show evolution of periodic conf	*/
/* 2 October 1989 - cycl33.c incorporated into rijn33.c	*/
/* 20 November 1989 - loop in kwait eliminated		*/
/* 30 July 1994 - adapted to NeXT			*/

/*  Copyright (C) 1987	*/
/*  Copyright (C) 1988	*/
/*  Copyright (C) 1989	*/
/*  Copyright (C) 1994	*/
/*  Harold V. McIntosh	*/

# include <math.h>
# include "nxlcau33.h"
# include "nxrijn33.h"
# import  <dpsclient/psops.h>
# import  <appkit/graphics.h>

extern NXColor colst[KK];

# define LUR		 1	/* left panel upper left row */
# define LUC		 0	/* left panel upper left col */
# define LLR		17	/* left panel lower right row */
# define LLC		15	/* left panel lower right col */
# define RUR		 1	/* right panel upper left row */
# define RUC		16	/* right panel upper left col */
# define RLR		21	/* right panel lower right row */
# define RLC		39	/* right panel lower right col */

# define CUR		 1	/* cycle panel upper left row */
# define CUC		 0	/* cycle panel upper left col */
# define CLR		24	/* cycle panel lower right row */
# define CLC		39	/* cycle panel lower right col */

# define EROW		23	/* row for sample evolution */

# define JX  0.50  /* x-center, de Bruijn diagram	*/
# define JY  0.50  /* y-center, de Bruijn diagram	*/
# define RI 0.400  /* inner radius, de Bruijn diagram	*/
# define RO 0.425  /* outer radius, de Bruijn diagram	*/

extern char arr1[AL];
extern char ascrule[KK][KK][KK][KK][KK][KK][KK];
extern  int binrule[KK][KK][KK][KK][KK][KK][KK];

char               aarry[KK][KK][KK][KK][KK][KK][KK];
char   barry[KK][KK][KK][KK][KK][KK][KK][KK][KK][KK][KK][KK][KK];

extern int    dd;		/* # points in de Bruijn ring	*/
extern int    dbno;		/* node in de Bruijn diagram	*/
extern double dcntr[][2];

/* edit the de Bruijn diagram		*/
/* g = number of generation (0==2nd)	*/
/* s = shift distance (g==0)		*/
void edijn(g,s,p,r) int g, s, p, r; {

  switch (g+'a') {

    case 'G': xpass1(s); break;
    case 'H': ypass1(s); break;

    case 'a': switch (s+'g') {
    case 'd': apass1(0); break;
    case 'e': apass1(1); break;
    case 'f': apass1(2); break;
    case 'g': apass1(3); break;
    case 'h': apass1(4); break;
    case 'i': apass1(5); break;
    case 'j': apass1(6); break;
	default: break;}

    case 'b': switch (s+'g') {
      case 'a': bpass1(0);  break;
      case 'b': bpass1(1);  break;
      case 'c': bpass1(2);  break;
      case 'd': bpass1(3);  break;
      case 'e': bpass1(4);  break;
      case 'f': bpass1(5);  break;
      case 'g': bpass1(6);  break;
      case 'h': bpass1(7);  break;
      case 'i': bpass1(8);  break;
      case 'j': bpass1(9);  break;
      case 'k': bpass1(10); break;
      case 'l': bpass1(11); break;
      case 'm': bpass1(12); break;
          default: break;}
    default: break;}
}

/* clear edijn screen and set up menu */
void clijn() {
  videomode(COLGRAF);
  videopalette(YELREGR);
  videocursor(0,0,0);
  scrrul();
  videocursor(0,3,0);
  printf("de Bruijn ring\n\n");
  printf("ab   -  0*,1*\n");
  printf("AB   -  full n*\n");
  printf("cdefghi - (1,x)\n");
  printf("CDEFG - (all)\n");
  printf("h.l.p - (2,x)\n");
  printf("y - cycles 1-9 \n");
  printf("1234 - diagram\n");
  printf("+-   - pallette\n");
  printf("?/   - clr scrn\n");
  printf("<cr> - exit\n\n");
}

/* edit the Pass 4 and Pass 5 shift displays */
void edshg(g,s) int g, s; {
    switch (shflev(g,s)) {
    case 0: apass6(); break;
    case 1: bpass6(); break;
    default: break;}
}

/* edit the Pass 4 and Pass 5 shift displays */
void edshf(g,s,c) int g, s, c; {
    switch (shflev(g,s)) {
    case 0: apass5(c); break;
    case 1: bpass5(c); break;
    default: break;}
}

/* edit the Pass 5 (sample loops) constant displays */
void edshc(g,c) int g, c; {
  switch(g) {
    case 0: apass5(c); break;
    case 1: bpass5(c); break;
    default: break;}
}

/* edit the Pass 4 and Pass 5 constant displays */
void edcst(s) int s; {
    switch(s) {
    case 0: apass6(); break;
    case 1: bpass6(); break;
    default: break;}
}

/* - - - - - - - - d e   B r u i j n - - - - - - - - - - - - - - - - - - */

/* generate a de Bruijn diagram with m nodes, n links per node, color l */
void ijn(m,n,l) int m, n, l; {
int    i, j;
double ii, jj, nn, x, y, t, h;
nn=(double)(n);
t=6.28318/((double)(m));
h=0.5*t;
for (i=0; i<m; i++) {
  ii=(double)(i);
  for (j=0; j<n; j++) {
    jj=(double)(j);
    x=JX-RI*sin(ii*t+h);
    y=JY-RI*cos(ii*t+h);
    videoline(x,y,0);
    x=JX-RO*sin((nn*ii+jj)*t+h);
    y=JY-RO*cos((nn*ii+jj)*t+h);
    videoline(x,y,l);
    }
  }
}

/* insert a link into a de Bruijn diagram */
/* u - initial vertex     */
/* v - terminal vertex    */
/* l - color of link	  */

void lijnk(u,v,l) int u, v, l; {
char   s[10];
double x, y, ex, wy, eks, wye, dx, dy, r, w, t, sx, sy;

	if (l<KK) NXSetColor(colst[l]); else PSsetgray(NX_BLACK);
	r=10.3;
	w=3.33;
	ex=dcntr[u][0];
	wy=dcntr[u][1];
	x=dcntr[v][0];
	y=dcntr[v][1];
	dx=x-ex;
	dy=y-wy;
	t=atan2(dy,dx);
	sx=-w*sin(t);
	sy= w*cos(t);
	eks=x-r*cos(t)+sx;
	wye=y-r*sin(t)+sy;
	PSmoveto(ex+sx,wy+sy);
	PSlineto(eks,wye);
	PSlineto(eks-0.1*dx+0.05*dy,wye-0.1*dy-0.05*dx);
	PSlineto(eks-0.1*dx-0.05*dy,wye-0.1*dy+0.05*dx);
	PSlineto(eks,wye);
	PSstroke();

	PSmoveto(x+10.3,y);
	NXSetColor(colst[v%KK]);
	PSarc(x, y, 10.3, 0.0, 360.0);
	PSstroke();

   PSsetgray(NX_BLACK); 

   ex=dcntr[u][0];
   wy=dcntr[u][1];
   PSmoveto(ex-10.0,wy+5.0);
	sprintf(s,"%3d",u);
   PSshow(s);

   ex=dcntr[v][0];
   wy=dcntr[v][1];
   PSmoveto(ex-10.0,wy+5.0);
	sprintf(s,"%3d",v);
   PSshow(s);
   PSstroke();

}

/* generate a de Bruijn diagram showing ancestors of state k in color l */
void sijn(k,l) int k, l; {
int    i, j, i0, i1, i2, i3, i4, i5, i6;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++) {
  i=((((i0*KK+i1)*KK+i2)*KK+i3)*KK+i4)*KK+i5;
  j=((((i1*KK+i2)*KK+i3)*KK+i4)*KK+i5)*KK+i6;
  if (k==ascrule[i0][i1][i2][i3][i4][i5][i6]-'0') lijnk(i,j,l);
  }
}

/* generate a de Bruijn diagram showing the (1,k) links in color l */
void zijn(m,k,l) int m, k, l; {
int    i, i0, i1, i2, i3, i4, i5, i6;
double th, x, y, t, h;
t=6.28318/((double)(m));
h=0.5*t;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++) {
  i=ascrule[i0][i1][i2][i3][i4][i5][i6]-'0';
  if ((k==0&&i==i0)||(k==1&&i==i1)||(k==2&&i==i2)||(k==3&&i==i3)||(k==4&&i==i4)) {
    th=((double)(((i0*KK+i1)*KK+i2)*KK+i3)*t+h);
    x=JX-RI*sin(th);
    y=JY-RI*cos(th);
    videoline(x,y,0);
    th=((double)(((i1*KK+i2)*KK+i3)*KK+i4)*t+h);
    x=JX-RO*sin(th);
    y=JY-RO*cos(th);
    videoline(x,y,l);
    }	         /* end if    */
  }              /* end for's */
}	         /* end zijn  */

/* plot graph on video screen 		   */
/* move pen from present position to (x,y) */
/* raised if l=0; color l if l>0	   */
void videoline(x,y,l) double x, y; int l; {
if (l==0) PSmoveto(400.0*x,400.0*y);
else PSlineto(400.0*x,400.0*y);
}

/* find the level of de Bruijn diagram appropriate to g, s  */
int shflev(g,s) int g, s; {int  i; i=s<0?-s:s; return (g+(i-1)/(3*(g+1)));}

/* increment ix */ 
void incix(ix,n) int *ix, n; {int i; 
  for (i=0; i<n; i++) {
    ix[i]++;
    if (ix[i]>=KK) ix[i]=0; else return;
    }
}

/* selected node in ix */ 
void nodix(ix,n) int *ix, n; {int i, j; 
  j=dbno;
  for (i=0; i<n; i++) {ix[n-i-1]=j%KK; j/=KK;}
}

/* random number in ix */ 
void ranix(ix,n) int *ix, n; {int i, j; 
  j=random();
  for (i=0; i<n; i++) {ix[i]=j%KK; j/=KK;}
}

/* clear ix */ 
void zerix(ix,n) int *ix, n; {int i; for (i=0; i<n; i++) ix[i]=0;}

/* find index of nearest center to x, y (de Bruijn) */
int nearest(x,y) double x, y; {int i, k; double ex, wy, r, p;
k=0; p=500000.0;
for (i=0; i<dd; i++) {
  ex=x-dcntr[i][0];
  wy=y-dcntr[i][1];
  r=ex*ex+wy*wy;
  if (r<p) {p=r; k=i;}
  }
return k;
}

/* ONE33.C */

/* Pass 1a analyzes all the configurations which fulfil (-1+l,1) */
void apass1(l) int l; {int  m, ix[RR];
asctobin();
for (ix[0]=0; ix[0]<KK; ix[0]++)
for (ix[1]=0; ix[1]<KK; ix[1]++)
for (ix[2]=0; ix[2]<KK; ix[2]++)
for (ix[3]=0; ix[3]<KK; ix[3]++)
for (ix[4]=0; ix[4]<KK; ix[4]++)
for (ix[5]=0; ix[5]<KK; ix[5]++)
for (ix[6]=0; ix[6]<KK; ix[6]++) {
  switch (l) {
    case 0: m=ix[0]; break;
    case 1: m=ix[1]; break;
    case 2: m=ix[2]; break;
    case 3: m=ix[3]; break;
    case 4: m=ix[4]; break;
    case 5: m=ix[5]; break;
    case 6: m=ix[6]; break;
    default: break;}
  aarry[ix[0]][ix[1]][ix[2]][ix[3]][ix[4]][ix[5]][ix[6]]=
    binrule[ix[0]][ix[1]][ix[2]][ix[3]][ix[4]][ix[5]][ix[6]]==m?'Y':'N';
  }
apass2i();
apass2o();
}

/* Pass 1x analyzes all the configurations mapping into a constant */
void xpass1(l) int l; {int  ix[RR];
asctobin();
// printf(" Pass1a\015");
for (ix[0]=0; ix[0]<KK; ix[0]++)
for (ix[1]=0; ix[1]<KK; ix[1]++)
for (ix[2]=0; ix[2]<KK; ix[2]++)
for (ix[3]=0; ix[3]<KK; ix[3]++)
for (ix[4]=0; ix[4]<KK; ix[4]++)
for (ix[5]=0; ix[5]<KK; ix[5]++)
for (ix[6]=0; ix[6]<KK; ix[6]++) {
  aarry[ix[0]][ix[1]][ix[2]][ix[3]][ix[4]][ix[5]][ix[6]]=
    binrule[ix[0]][ix[1]][ix[2]][ix[3]][ix[4]][ix[5]][ix[6]]==l?'Y':'N';
  }
apass2i();
apass2o();
}

/* Pass 2i flags all links with an incoming arrow */
/* Pass 2o flags all links with an outgoing arrow */
/* Then pass 3 discards all unflagged links */
/* Passes 2 and 3 alternate until no change is observed */

void apass2i() {int i0, i1, i2, i3, i4, i5, i6, m;
do {
// printf(" Pass2i\015");
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++) {
  if ((aarry[i0][i1][i2][i3][i4][i5][i6]&0x5F)=='Y')
    {for (m=0; m<KK; m++) aarry[i1][i2][i3][i4][i5][i6][m]|=0x20;}
    } } while (apass3()!=0); }

void apass2o() {int i0, i1, i2, i3, i4, i5, i6, m;
do {
// printf(" Pass2o\015");
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++) {
  if ((aarry[i0][i1][i2][i3][i4][i5][i6]&0x5F)=='Y')
    {for (m=0; m<KK; m++) aarry[m][i0][i1][i2][i3][i4][i5]|=0x20;}
     } } while (apass3()!=0); }

/* Pass 3 - erase flags, mark survivors, detect changes */

int apass3() {int i0, i1, i2, i3, i4, i5, i6, l;
l=0;
// printf(" Pass3 \015");
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++) {
  switch (aarry[i0][i1][i2][i3][i4][i5][i6]) {
    case 'y': aarry[i0][i1][i2][i3][i4][i5][i6]='Y'; break;
    case 'Y': aarry[i0][i1][i2][i3][i4][i5][i6]='N'; l=1; break;
    case 'n': case 'N': aarry[i0][i1][i2][i3][i4][i5][i6]='N'; break;
    default: break; }
}
return l;
}

/* Pass 4 - print loops which remain */
void apass4() {
int i0, i1, i2, i3, i4, i5, i6;
int j0, j1, j2, j3, j4, j5, j6;
int k, l, m;
printf(" pass4 \015");
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++) {
j0=i0; j1=i1; j2=i2; j3=i3; j4=i4; j5=i5; j6=i6;
l=0; m=0;
do {
        if (aarry[0][j1][j2][j3][j4][j5][j6]=='Y')
    {aarry[0][j1][j2][j3][j4][j5][j6]='y';
    k=j4; j4=j3; j3=j2; j2=j1; j1=0; m=1;}
  else {if (aarry[1][j1][j2][j3][j4][j5][j6]=='Y')
    {aarry[1][j1][j2][j3][j4][j5][j6]='y';
    k=j6; j6=j5; j5=j4; j4=j3; j3=j2; j2=j1; j1=1; m=1;}
  else {l=1; if (m==1) {j0=j1; j1=j2; j2=j3; j3=j4; j4=j5; j5=j6; j6=k;} }}
  } while (l==0);
l=0; m=0;
do {
        if (aarry[j0][j1][j2][j3][j4][j5][0]=='y')
   {aprf(j0,j1,j2,j3,j4,j5,0);
   aarry[j0][j1][j2][j3][j4][j5][0]='N';
   j0=j1; j1=j2; j2=j3; j3=j4; j4=j5; j6=0; m=1;}
  else {if (aarry[j0][j1][j2][j3][j4][j5][1]=='y')
   {aprf(j0,j1,j2,j3,j4,j5,1);
   aarry[j0][j1][j2][j3][j4][j5][1]='N';
   j0=j1; j1=j2; j2=j3; j3=j4; j4=j5; j5=1; m=1;}
  else {l=1;}}
  } while (l==0);
l=0;
do {
        if (aarry[j0][j1][j2][j3][j4][j5][0]=='Y')
   {aprf(j0,j1,j2,j3,j4,j5,0);
   aarry[j0][j1][j2][j3][j4][j5][0]='N';
   j0=j1; j1=j2; j2=j3; j3=j4; j4=j5; j5=0; m=1;}
  else {if (aarry[j0][j1][j2][j3][j4][j5][1]=='Y')
   {aprf(j0,j1,j2,j3,j4,j5,1);
   aarry[j0][j1][j2][j3][j4][j5][1]='N';
   j0=j1; j1=j2; j2=j3; j3=j4; j4=j5; j5=1; m=1;}
  else {l=1; }}
  } while (l==0);
}
}

/* print one link of the de Bruijn diagram */
aprf(i0,i1,i2,i3,i4,i5,i6) int i0, i1, i2, i3, i4, i5, i6; {
// kwait(1);
printf("%1d",i0);
printf("%1d",i1);
printf("%1d",i2);
printf("%1d",i3);
printf("%1d",i4);
printf("%1d",i5);
printf("-");
printf("%1d",i6);
printf("  ");
lijnk(((i0*KK+i1)*KK+i2)*KK+i3,((i1*KK+i2)*KK+i3)*KK+i4,3);
}

/* Pass 5 - show evolution of sample loops */
void apass5(c) char c; {
int i0, i1, i2, i3, i4, i5, i6;
int j1, j2, j3, j4, j5, j6;
int hl;
int b[KK], irry[RR], i, j, k, l;
  switch (c) {
    case 'z': zerix(irry,RR); break;
    case 'r': ranix(irry,RR); break;
    case 'n': nodix(irry,RR-1);
  i0=irry[0]; i1=irry[1]; i2=irry[2]; i3=irry[3]; i4=irry[4]; i5=irry[5];
	for (i=j=0; j<KK; j++)
		if (aarry[i0][i1][i2][i3][i4][i5][j]=='Y') {b[i]=j; i++;}
	if (i==0) return;
	l=0; if (i>1) l=random()%i;
	irry[RR-1]=b[l];
	break;
    case 'i': incix(irry,RR); break;
    default:  zerix(irry,RR); break;}
  i0=irry[0]; i1=irry[1]; i2=irry[2]; i3=irry[3]; i4=irry[4]; i5=irry[5]; i6=irry[6];
for (k=0; k<81; k++) {
  if (aarry[i0][i1][i2][i3][i4][i5][i6]=='Y') break;
  incix(irry,RR);
  i0=irry[0]; i1=irry[1]; i2=irry[2]; i3=irry[3]; i4=irry[4]; i5=irry[5]; i6=irry[6];
  }
hl=AL/2;
arr1[hl-3]=i0;
arr1[hl-2]=i1;
arr1[hl-1]=i2;
arr1[hl]  =i3;
arr1[hl+1]=i4;
arr1[hl+2]=i5;
arr1[hl+3]=i6;
j1=i1; j2=i2; j3=i3; j4=i4; j5=i5; j6=i6;
for (k=hl-4; k>=0; k--) {
  for (i=j=0; j<KK; j++) if (aarry[j][i0][i1][i2][i3][i4][i5]=='Y') {b[i]=j; i++;}
  if (i==0) return;
  l=0; if (i>1) l=random()%i;
  i6=i5; i5=i4; i4=i3; i3=i2; i2=i1; i1=i0; i0=arr1[k]=b[l];
  }
i1=j1; i2=j2; i3=j3; i4=j4; i5=j5; i6=j6;
for (k=hl+4; k<AL; k++) {
  for (i=j=0; j<KK; j++) if (aarry[i1][i2][i3][i4][i5][i6][j]=='Y') {b[i]=j; i++;}
  if (i==0) return;
  l=0; if (i>1) l=random()%i;
  i1=i2; i2=i3; i3=i4; i4=i5; i5=i6; i6=arr1[k]=b[l];
  }
}

void apass6() {int i, j, i0, i1, i2, i3, i4, i5, i6;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++) {
  i=((((i0*KK+i1)*KK+i2)*KK+i3)*KK+i4)*KK+i5;
  j=((((i1*KK+i2)*KK+i3)*KK+i4)*KK+i5)*KK+i6;
  if(aarry[i0][i1][i2][i3][i4][i5][i6]=='Y') lijnk(i,j,3);
  }
printf("apass6 done, dd=%3d\n",dd);
}

/* TWO33.C */
/* arry occupies 8,192 bytes */

/* mark sequences conforming to (-2+l,2) */
void bpass1(l) int l; {
int ix[2*RR-1], jx[RR], kx;
int i, m, n;
asctobin();
// printf(" Pass1 \015");
for (ix[0]=0;  ix[0]<KK;  ix[0]++)
for (ix[1]=0;  ix[1]<KK;  ix[1]++)
for (ix[2]=0;  ix[2]<KK;  ix[2]++)
for (ix[3]=0;  ix[3]<KK;  ix[3]++)
for (ix[4]=0;  ix[4]<KK;  ix[4]++)
for (ix[5]=0;  ix[5]<KK;  ix[5]++)
for (ix[6]=0;  ix[6]<KK;  ix[6]++)
for (ix[7]=0;  ix[7]<KK;  ix[7]++)
for (ix[8]=0;  ix[8]<KK;  ix[8]++)
for (ix[9]=0;  ix[9]<KK;  ix[9]++)
for (ix[10]=0; ix[10]<KK; ix[10]++)
for (ix[11]=0; ix[11]<KK; ix[11]++)
for (ix[12]=0; ix[12]<KK; ix[12]++) {
  for (i=0; i<7; i++) jx[i]=binrule[ix[i]][ix[i+1]][ix[i+2]]
				[ix[i+3]][ix[i+4]][ix[i+5]][ix[i+6]];
  kx=binrule[jx[0]][jx[1]][jx[2]][jx[3]][jx[4]][jx[5]][jx[6]];
switch (l) {
  case 0:  m=ix[0]; n=kx; break;
  case 1:  m=ix[1]; n=kx; break;
  case 2:  m=ix[2]; n=kx; break;
  case 3:  m=ix[3]; n=kx; break;
  case 4:  m=ix[4]; n=kx; break;
  case 5:  m=ix[5]; n=kx; break;
  case 6:  m=ix[6]; n=kx; break;
  case 7:  m=ix[7]; n=kx; break;
  case 8:  m=ix[8]; n=kx; break;
  case 9:  m=ix[9]; n=kx; break;
  case 10: m=ix[10]; n=kx; break;
  case 11: m=ix[11]; n=kx; break;
  case 12: m=ix[12]; n=kx; break;
  case 13: m=ix[12]; n=jx[3]; break;
  case 14: m=ix[11]; n=jx[3]; break;
  case 15: m=ix[10]; n=jx[3]; break;
  case 16: m=ix[0]; n=jx[3]; break;
  case 17: m=ix[1]; n=jx[3]; break;
  case 18: m=ix[2]; n=jx[3]; break;
  default: break;}
barry[ix[0]][ix[1]][ix[2]][ix[3]][ix[4]][ix[5]][ix[6]]
    [ix[7]][ix[8]][ix[9]][ix[10]][ix[11]][ix[12]]=m==n?'Y':'N';
}
bpass2i();
bpass2o();
}

/* Pass 1y analyzes all the configurations mapping into a constant */
void ypass1(l) int l; {
int  i, ix[2*RR-1], jx[RR], kx;
asctobin();
for (ix[0]=0;   ix[0]<KK;  ix[0]++)
for (ix[1]=0;   ix[1]<KK;  ix[1]++)
for (ix[2]=0;   ix[2]<KK;  ix[2]++)
for (ix[3]=0;   ix[3]<KK;  ix[3]++)
for (ix[4]=0;   ix[4]<KK;  ix[4]++)
for (ix[5]=0;   ix[5]<KK;  ix[5]++)
for (ix[6]=0;   ix[6]<KK;  ix[6]++)
for (ix[7]=0;   ix[7]<KK;  ix[7]++)
for (ix[8]=0;   ix[8]<KK;  ix[8]++)
for (ix[9]=0;   ix[9]<KK;  ix[9]++)
for (ix[10]=0; ix[10]<KK; ix[10]++)
for (ix[11]=0; ix[11]<KK; ix[11]++)
for (ix[12]=0; ix[12]<KK; ix[12]++) {
  for (i=0; i<RR; i++) 
	jx[i] = binrule[ix[i]][ix[i+1]][ix[i+2]]
		       [ix[i+3]][ix[i+4]][ix[i+5]][ix[i+6]];
  kx = binrule[jx[0]][jx[1]][jx[2]][jx[3]][jx[4]][jx[5]][jx[6]];
  barry[ix[0]][ix[1]][ix[2]][ix[3]][ix[4]][ix[5]][ix[6]]
       [ix[7]][ix[8]][ix[9]][ix[10]][ix[11]][ix[12]] = kx==l?'Y':'N';
  }
bpass2i();
bpass2o();
}

/* Passs 2i flags links which have an incoming arrow */
void bpass2i() {
int i0, i1, i2, i3, i4, i5, i6, i7, i8, i9, ia, ib, ic, m;
do {
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++)
for (i7=0; i7<KK; i7++)
for (i8=0; i8<KK; i8++)
for (i9=0; i9<KK; i9++)
for (ia=0; ia<KK; ia++)
for (ib=0; ib<KK; ib++)
for (ic=0; ic<KK; ic++) {
if ((barry[i0][i1][i2][i3][i4][i5][i6][i7][i8][i9][ia][ib][ic]&0x5F)=='Y')
 {for (m=0; m<KK; m++) 
	barry[i1][i2][i3][i4][i5][i6][i7][i8][i9][ia][ib][ic][m]|=0x20;}
}
} while (bpass3()!=0); }

/* Passs 2o flags links which have an outgoing arrow */
void bpass2o() {
int i0, i1, i2, i3, i4, i5, i6, i7, i8, i9, ia, ib, ic, m;
do {
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++)
for (i7=0; i7<KK; i7++)
for (i8=0; i8<KK; i8++)
for (i9=0; i9<KK; i9++)
for (ia=0; ia<KK; ia++)
for (ib=0; ib<KK; ib++)
for (ic=0; ic<KK; ic++) {
if ((barry[i0][i1][i2][i3][i4][i5][i6][i7][i8][i9][ia][ib][ic]&0x5F)=='Y')
 {for (m=0; m<KK; m++) 
	barry[m][i0][i1][i2][i3][i4][i5][i6][i7][i8][i9][ia][ib]|=0x20;}
}
} while (bpass3()!=0); }

/*  Pass 3 erases flags, marks survivors, counts channges */
int bpass3() {
int i0, i1, i2, i3, i4, i5, i6, i7, i8, i9, ia, ib, ic, l;
l=0;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++)
for (i7=0; i7<KK; i7++)
for (i8=0; i8<KK; i8++)
for (i9=0; i9<KK; i9++)
for (ia=0; ia<KK; ia++)
for (ib=0; ib<KK; ib++)
for (ic=0; ic<KK; ic++) {
switch (barry[i0][i1][i2][i3][i4][i5][i6][i7][i8][i9][ia][ib][ic]) {
    case 'y': barry[i0][i1][i2][i3][i4][i5][i6][i7][i8][i9][ia][ib][ic]='Y'; break;
    case 'Y': barry[i0][i1][i2][i3][i4][i5][i6][i7][i8][i9][ia][ib][ic]='N'; l=1; break;
    case 'n': case 'N': barry[i0][i1][i2][i3][i4][i5][i6][i7][i8][i9][ia][ib][ic]='N'; break;
    default: break; }
}
return l;
}

/* Pass 4 prints loops which remain */
void bpass4() {
int i0, i1, i2, i3, i4, i5, i6, i7, i8, i9, ia, ib, ic;
int j0, j1, j2, j3, j4, j5, j6, j7, j8, j9, ja, jb, jc;
int k, l, m;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++)
for (i7=0; i7<KK; i7++)
for (i8=0; i8<KK; i8++)
for (i9=0; i9<KK; i9++)
for (ia=0; ia<KK; ia++)
for (ib=0; ib<KK; ib++)
for (ic=0; ic<KK; ic++) {
j0=i0; j1=i1; j2=i2; j3=i3; j4=i4; j5=i5; j6=i6; j7=i7; 
j8=i8; j9=i9; ja=ia; jb=ib; jc=ic; l=0; m=0;
do {
        if (barry[0][j1][j2][j3][j4][j5][j6][j7][j8][j9][ja][jb][jc]=='Y')
    {barry[0][j1][j2][j3][j4][j5][j6][j7][j8][j9][ja][jb][jc]='y';
    k=j8; j8=j7; j7=j6; j6=j5; j5=j4; j4=j3; j3=j2; j2=j1; j1=0; m=1;}
  else {if (barry[1][j1][j2][j3][j4][j5][j6][j7][j8][j9][ja][jb][jc]=='Y')
    {barry[1][j1][j2][j3][j4][j5][j6][j7][j8][j9][ja][jb][jc]='y';
    k=j8; j8=j7; j7=j6; j6=j5; j5=j4; j4=j3; j3=j2; j2=j1; j1=1; m=1;}
  else {l=1; if (m==1) {j0=j1; j1=j2; j2=j3; j3=j4; j4=j5; j5=j6; j6=j7; j7=j8; j8=k;} }}
  } while (l==0);
l=0; 
m=0;
do {
        if (barry[j0][j1][j2][j3][j4][j5][j6][j7][j8][j9][ja][jb][0]=='y')
   {bprf(j0,j1,j2,j3,j4,j5,j6,j7,0);
   barry[j0][j1][j2][j3][j4][j5][j6][j7][j8][j9][ja][jb][0]='N';
   j0=j1; j1=j2; j2=j3; j3=j4; j4=j5; j5=j6; j6=j7; j7=0; m=1;}
  else {if (barry[j0][j1][j2][j3][j4][j5][j6][j7][j8][j9][ja][jb][1]=='y')
   {bprf(j0,j1,j2,j3,j4,j5,j6,j7,1);
   barry[j0][j1][j2][j3][j4][j5][j6][j7][j8][j9][ja][jb][1]='N';
   j0=j1; j1=j2; j2=j3; j3=j4; j4=j5; j5=j6; j6=j7; j7=1; m=1;}
  else {l=1;}}
  } while (l==0);
l=0;
do {
        if (barry[j0][j1][j2][j3][j4][j5][j6][j7][j8][j9][ja][jb][0]=='Y')
   {bprf(j0,j1,j2,j3,j4,j5,j6,j7,0);
   barry[j0][j1][j2][j3][j4][j5][j6][j7][j8][j9][ja][jb][0]='N';
   j0=j1; j1=j2; j2=j3; j3=j4; j4=j5; j5=j6; j6=j7; j7=0; m=1;}
  else {if (barry[j0][j1][j2][j3][j4][j5][j6][j7][j8][j9][ja][jb][1]=='Y')
   {bprf(j0,j1,j2,j3,j4,j5,j6,j7,1);
   barry[j0][j1][j2][j3][j4][j5][j6][j7][j8][j9][ja][jb][1]='N';
   j0=j1; j1=j2; j2=j3; j3=j4; j4=j5; j5=j6; j6=j7; j7=1; m=1;}
  else {l=1; }}
  } while (l==0);
}
}

/* print one of the links in a chain */
bprf(i0,i1,i2,i3,i4,i5,i6,i7,i8,i9,ia) 
int i0, i1, i2, i3, i4, i5, i6, i7, i8, i9, ia; {
int i, j;
// kwait(1);
printf("%1d",i0);
printf("%1d",i1);
printf("%1d",i2);
printf("%1d",i3);
printf("%1d",i4);
printf("%1d",i5);
printf("%1d",i6);
printf("%1d",i7);
printf("-");
printf("%1d",i8);
printf("%1d",i9);
printf("%1d",ia);
printf(" ");
i=((((((i0*KK+i1)*KK+i2)*KK+i3)*KK+i4)*KK+i5)*KK+i6)*KK+i7;
j=((((((i1*KK+i2)*KK+i3)*KK+i4)*KK+i5)*KK+i6)*KK+i7)*KK+i8;
lijnk(i,j,3);
}

/* Pass 5 - show evolution of sample loops */
void bpass5(c) char c; {
int i0, i1, i2, i3, i4, i5, i6, i7, i8, i9, ia, ib, ic;
int j1, j2, j3, j4, j5, j6, j7, j8, j9, ja, jb, jc;
int hl;
int b[KK], irry[2*RR-1], i, j, k, l;
  switch (c) {
    case 'z': zerix(irry,2*RR-1); break;
    case 'r': ranix(irry,2*RR-1); break;
    case 'n': nodix(irry,2*RR-2);
	i0=irry[0]; i1=irry[1]; i2=irry[2]; i3=irry[3]; 
	i4=irry[4]; i5=irry[5]; i6=irry[6]; i7=irry[7]; 
	i8=irry[8]; i9=irry[9]; ia=irry[10]; ib=irry[11]; 
	for (i=j=0; j<KK; j++) 
	if (barry[i0][i1][i2][i3][i4][i5][i6][i7][i8][i9][ia][ib][j]=='Y') 
		{b[i]=j; i++;}
	if (i==0) return;
	l=0; if (i>1) l=random()%i;
	irry[2*RR-2]=b[l];
	break;
    case 'i': incix(irry,2*RR-1); break;
    default:  zerix(irry,2*RR-1); break;}
for (k=0; k<1024; k++) {
if (barry[irry[0]]
	 [irry[1]]
	 [irry[2]]
	 [irry[3]]
	 [irry[4]]
	 [irry[5]]
	 [irry[6]]
	 [irry[7]]
	 [irry[8]]
	 [irry[9]]
	 [irry[10]]
	 [irry[11]]
	 [irry[12]]=='Y') break;
  incix(irry,2*RR-1);
  }
   i0=irry[0]; i1=j1=irry[1]; i2=j2=irry[2]; i3=j3=irry[3]; i4=j4=irry[4];
i5=j5=irry[5]; i6=j6=irry[6]; i7=j7=irry[7]; i8=j8=irry[8]; i9=j9=irry[9];
ia=ja=irry[10]; ib=jb=irry[11]; ic=jc=irry[12]; 
hl=AL/2;
arr1[hl-6]=i0;
arr1[hl-5]=i1;
arr1[hl-4]=i2;
arr1[hl-3]=i3;
arr1[hl-2]=i4;
arr1[hl-1]=i5;
arr1[hl]  =i6;
arr1[hl+1]=i7;
arr1[hl+2]=i8;
arr1[hl+3]=i9;
arr1[hl+4]=ia;
arr1[hl+5]=ib;
arr1[hl+6]=ic;
for (k=hl-7; k>=0; k--) {
  for (i=j=0; j<KK; j++) 
    if (barry[j][i0][i1][i2][i3][i4][i5][i6][i7][i8][i9][ia][ib]=='Y') 
	{b[i]=j; i++;}
  if (i==0) return;
  if (i==1) l=i-1; else l=random()%i;
  ib=ia; ia=i9; i9=i8; i8=i7; i7=i6; i6=i5; i5=i4; i4=i3; i3=i2; i2=i1; 
	 i1=i0; i0=arr1[k]=b[l];
  }
printf("bpass5: left half done\n");
i1=j1; i2=j2; i3=j3; i4=j4; i5=j5; i6=j6; i7=j7; i8=j8; i9=j9; ia=ja; 
       ib=jb; ic=jc; 
for (k=hl+7; k<AL; k++) {
  for (i=j=0; j<KK; j++) 
    if (barry[i1][i2][i3][i4][i5][i6][i7][i8][i9][ia][ib][ic][j]=='Y') 
	{b[i]=j; i++;}
  if (i==0) return;
  if (i==1) l=i-1; else l=random()%i;
  i1=i2; i2=i3; i3=i4; i4=i5; i5=i6; i6=i7; i7=i8; i8=i9; i9=ia; ia=ib; 
	ib=ic; ic=arr1[k]=b[l];
  }
printf("bpass5: right half done\n");
}

void bpass6() {int i, j, i0, i1, i2, i3, i4, i5, i6, i7, i8, i9, ia, ib, ic;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++)
for (i7=0; i7<KK; i7++)
for (i8=0; i8<KK; i8++)
for (i9=0; i9<KK; i9++)
for (ia=0; ia<KK; ia++)
for (ib=0; ib<KK; ib++)
for (ic=0; ic<KK; ic++) {
  i=((((((((((i0*KK+i1)*KK+i2)*KK+i3)*KK+i4)*KK+i5)*KK+i6)*KK+i7)*KK+i8)*KK+i9)*KK+ia)*KK+ib;
  j=((((((((((i1*KK+i2)*KK+i3)*KK+i4)*KK+i5)*KK+i6)*KK+i7)*KK+i8)*KK+i9)*KK+ia)*KK+ib)*KK+ic;
  if(barry[i0][i1][i2][i3][i4][i5][i6][i7][i8][i9][ia][ib][ic]=='Y') lijnk(i,j,3);
  }
}

/* end nxrijn33.c */
