
/* SQAR31.C					*/
/* Three-Dimensional Square Lattice 		*/
/* trinary cells, first neighbors		*/
/* ---- von Neumann neighborhoods ----		*/
/*						*/
/* Reference:					*/
/*						*/
/* Harold V. McIntosh, 16 May 1987		*/
/* 7 July 1987 - separate rules for each state	*/
/* 8 July 1988 - binary, 3 neighborhood classes	*/
/* 9 July 1988 - hexagonal lattice 		*/
/* 20 July 1988 - von Neumann neighborhgood	*/

# import  "nxlcau33.h"
# include <math.h>
# import  <dpsclient/psops.h>
# import  <appkit/graphics.h>

# define GH    (DS/KK)	/* DS/KK= half number of neighborhoods	*/
# define QS	   128  /* number of symmetry classes		*/
# define NF	    50	/* lines in live frequency graph	*/
# define NG	    10	/* number of Golay Classes		*/

char    golRule[KK*NG];	/* evolutionary rule       */
char    sqrule[DS];	/* evolutionary rule       */
char    qarule[DS];	/* auxiliary rule flags    */
int     li, lj;		/* cursor in cell array    */
int	lf[KK];		/* number of live cells	   */
int     stus[KK*(TS-1)];/* s-totalistic use stats  */
int     gus[KK*NG];	/* golay use statistics    */
int     rus[DS];	/* rule use statistics     */
int     tus[TS];	/* totalistic use stats    */
int     gc[DS];		/* Golay code key array    */
int     tc[DS];		/* totalistic key array    */
int     stc[DS];	/* s-totalistic key array  */
double  bp[TS];		/* Bernstein coefficients  */

extern char   ascrule[KK][KK][KK][KK][KK][KK][KK];
extern  int   binrule[KK][KK][KK][KK][KK][KK][KK];

mainsquar() {}	   /* end main */

void asctoqar() {int i0, i1, i2, i3, i4, i5, i6;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
  for (i4=0; i4<KK; i4++)
  for (i5=0; i5<KK; i5++)
  for (i6=0; i6<KK; i6++)
    sqrule[i0+KK*(i1+KK*(i2+KK*(i3+KK*(i4+KK*(i5+KK*i6)))))] = 
	ascrule[i0][i1][i2][i3][i4][i5][i6];
}

void qartoasc() {int i0, i1, i2, i3, i4, i5, i6;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
  for (i4=0; i4<KK; i4++)
  for (i5=0; i5<KK; i5++)
  for (i6=0; i6<KK; i6++)
    ascrule[i0][i1][i2][i3][i4][i5][i6] = 
	sqrule[i0+KK*(i1+KK*(i2+KK*(i3+KK*(i4+KK*(i5+KK*i6)))))];
}

/* 		     layout for cell indexing				*/
/*									*/
/*		 |				 |			*/
/*		 z   y				 5   4			*/
/*	       	 | / 			       	 | / 			*/
/*      - - -x - c - x - - -	        - -  2 - 6 - 3 - - -		*/
/*	       / |  			       / |  			*/
/*	    -y  -z			     1   0			*/
/*		 |				 |			*/
/*									*/

/* calculate one generation of evolution    */
/* aa[l][m][n] = original and final plane   */
/* bb[l][m][n] = intermediate scratch plane */
/* l = range of first index		    */
/* m = range of second index  		    */
/* n = range of third index   		    */
cuonegen(aa,bb,l,m,n) char *aa; int *bb; int l, m, n; {int i0, i1, i2, gn;

/* count neighbors of center cube */

for (i0=1; i0<l-1; i0++)
for (i1=1; i1<m-1; i1++) 
for (i2=1; i2<n-1; i2++) {
  gn=aa[i0+l*(i1+m*(i2-1))];
  gn=KK*gn+aa[i0+l*((i1-1)+m*i2)];
  gn=KK*gn+aa[(i0-1)+l*(i1+m*i2)];
  gn=KK*gn+aa[(i0+1)+l*(i1+m*i2)];
  gn=KK*gn+aa[i0+l*((i1+1)+m*i2)];
  gn=KK*gn+aa[i0+l*(i1+m*(i2+1))];
  gn+=GH*aa[i0+l*(i1+m*i2)];
  bb[i0+l*(i1+m*i2)]=gn;
  }

/* count neighbors for left face (i0=0) */

for (i1=1; i1<m-1; i1++) 
for (i2=1; i2<n-1; i2++) {
  gn=aa[0+l*(i1+m*(i2-1))];
  gn=KK*gn+aa[0+l*((i1-1)+m*i2)];
  gn=KK*gn+aa[(l-1)+l*(i1+m*i2)];
  gn=KK*gn+aa[(0+1)+l*(i1+m*i2)];
  gn=KK*gn+aa[0+l*((i1+1)+m*i2)];
  gn=KK*gn+aa[0+l*(i1+m*(i2+1))];
  gn+=GH*aa[0+l*(i1+m*i2)];
  bb[0+l*(i1+m*i2)]=gn;
  }

/* count neighbors for right face (i0=l-1) */

for (i1=1; i1<m-1; i1++) 
for (i2=1; i2<n-1; i2++) {
  gn=aa[(l-1)+l*(i1+m*(i2-1))];
  gn=KK*gn+aa[(l-1)+l*((i1-1)+m*i2)];
  gn=KK*gn+aa[((l-1)-1)+l*(i1+m*i2)];
  gn=KK*gn+aa[(0)+l*(i1+m*i2)];
  gn=KK*gn+aa[(l-1)+l*((i1+1)+m*i2)];
  gn=KK*gn+aa[(l-1)+l*(i1+m*(i2+1))];
  gn+=GH*aa[(l-1)+l*(i1+m*i2)];
  bb[(l-1)+l*(i1+m*i2)]=gn;
  }

/* count neighbors for front face (i1=0) */

for (i0=1; i0<l-1; i0++)
for (i2=1; i2<n-1; i2++) {
  gn=aa[i0+l*(0+m*(i2-1))];
  gn=KK*gn+aa[i0+l*((m-1)+m*i2)];
  gn=KK*gn+aa[(i0-1)+l*(0+m*i2)];
  gn=KK*gn+aa[(i0+1)+l*(0+m*i2)];
  gn=KK*gn+aa[i0+l*((0+1)+m*i2)];
  gn=KK*gn+aa[i0+l*(0+m*(i2+1))];
  gn+=GH*aa[i0+l*(0+m*i2)];
  bb[i0+l*(0+m*i2)]=gn;
  }

/* count neighbors for back face (i1=m-1) */

for (i0=1; i0<l-1; i0++)
for (i2=1; i2<n-1; i2++) {
  gn=aa[i0+l*((m-1)+m*(i2-1))];
  gn=KK*gn+aa[i0+l*(((m-1)-1)+m*i2)];
  gn=KK*gn+aa[(i0-1)+l*((m-1)+m*i2)];
  gn=KK*gn+aa[(i0+1)+l*((m-1)+m*i2)];
  gn=KK*gn+aa[i0+l*((0)+m*i2)];
  gn=KK*gn+aa[i0+l*((m-1)+m*(i2+1))];
  gn+=GH*aa[i0+l*((m-1)+m*i2)];
  bb[i0+l*((m-1)+m*i2)]=gn;
  }

/* count neighbors of bottom face (i2=0) */

for (i0=1; i0<l-1; i0++)
for (i1=1; i1<m-1; i1++) {
  gn=aa[i0+l*(i1+m*(n-1))];
  gn=KK*gn+aa[i0+l*((i1-1)+m*0)];
  gn=KK*gn+aa[(i0-1)+l*(i1+m*0)];
  gn=KK*gn+aa[(i0+1)+l*(i1+m*0)];
  gn=KK*gn+aa[i0+l*((i1+1)+m*0)];
  gn=KK*gn+aa[i0+l*(i1+m*(0+1))];
  gn+=GH*aa[i0+l*(i1+m*0)];
  bb[i0+l*(i1+m*0)]=gn;
  }

/* count neighbors of top face (i2=n-1) */

for (i0=1; i0<l-1; i0++)
for (i1=1; i1<m-1; i1++) {
  gn=aa[i0+l*(i1+m*((n-1)-1))];
  gn=KK*gn+aa[i0+l*((i1-1)+m*(n-1))];
  gn=KK*gn+aa[(i0-1)+l*(i1+m*(n-1))];
  gn=KK*gn+aa[(i0+1)+l*(i1+m*(n-1))];
  gn=KK*gn+aa[i0+l*((i1+1)+m*(n-1))];
  gn=KK*gn+aa[i0+l*(i1+m*(0))];
  gn+=GH*aa[i0+l*(i1+m*(n-1))];
  bb[i0+l*(i1+m*(n-1))]=gn;
  }

/* count neighbors for all twelve edges */

for (i0=1; i0<l-1; i0++) safecount(aa,bb,i0,  0,  0,l,m,n);
for (i0=1; i0<l-1; i0++) safecount(aa,bb,i0,  0,n-1,l,m,n);
for (i0=1; i0<l-1; i0++) safecount(aa,bb,i0,m-1,  0,l,m,n);
for (i0=1; i0<l-1; i0++) safecount(aa,bb,i0,m-1,n-1,l,m,n);

for (i1=1; i1<m-1; i1++) safecount(aa,bb,  0,i1,  0,l,m,n);
for (i1=1; i1<m-1; i1++) safecount(aa,bb,  0,i1,n-1,l,m,n);
for (i1=1; i1<m-1; i1++) safecount(aa,bb,l-1,i1,  0,l,m,n);
for (i1=1; i1<m-1; i1++) safecount(aa,bb,l-1,i1,n-1,l,m,n);

for (i2=1; i2<n-1; i2++) safecount(aa,bb,  0,  0,i2,l,m,n);
for (i2=1; i2<n-1; i2++) safecount(aa,bb,  0,m-1,i2,l,m,n);
for (i2=1; i2<n-1; i2++) safecount(aa,bb,l-1,  0,i2,l,m,n);
for (i2=1; i2<n-1; i2++) safecount(aa,bb,l-1,m-1,i2,l,m,n);

/* count neighbors for all eight corners */
  safecount(aa,bb,0,  0,  0,  l,m,n);
  safecount(aa,bb,0,  0,  n-1,l,m,n);
  safecount(aa,bb,0,  m-1,0,  l,m,n);
  safecount(aa,bb,0,  m-1,n-1,l,m,n);
  safecount(aa,bb,l-1,0,  0,  l,m,n);
  safecount(aa,bb,l-1,0,  n-1,l,m,n);
  safecount(aa,bb,l-1,m-1,0,  l,m,n);
  safecount(aa,bb,l-1,m-1,n-1,l,m,n);

/* phase to update cells */
for (i0=0; i0<KK; i0++)        lf[i0]=0;
for (i0=0; i0<DS; i0++)        rus[i0]=0;
for (i0=0; i0<TS; i0++)        tus[i0]=0;
// for (i0=0; i0<KK*NG; i0++)     gus[i0]=0;
// for (i0=0; i0<KK*(TS-1); i0++) stus[i0]=0;
for (i0=0; i0<l; i0++) for (i1=0; i1<m; i1++) for (i2=0; i2<n; i2++) {
  gn=bb[i0+l*(i1+m*i2)];
  aa[i0+l*(i1+m*i2)]=sqrule[gn]-'0';
  lf[aa[i0+l*(i1+m*i2)]]+=1;
//   stus[stc[gn]]+=1;
//   gus[gc[gn]]+=1;
  tus[tc[gn]]+=1;
  rus[gn]++;
  }

/* phase to show results */
}	 /* end cuonegen */

/* do counting which is reliable at the boundaries of the parallelopiped */
void safecount(aa,bb,i0,i1,i2,l,m,n) 
char *aa; int *bb; int i0, i1, i2, l, m, n; {int gn;
  gn=aa[i0+l*(i1+m*((n+i2-1)%n))];
  gn=KK*gn+aa[i0+l*(((m+i1-1)%m)+m*i2)];
  gn=KK*gn+aa[((l+i0-1)%l)+l*(i1+m*i2)];
  gn=KK*gn+aa[((l+i0-1)%l)+l*(i1+m*i2)];
  gn=KK*gn+aa[i0+l*(((m+i1-1)%m)+m*i2)];
  gn=KK*gn+aa[i0+l*(i1+m*((n+i2+1)%n))];
  gn+=GH*aa[i0+l*(i1+m*i2)];
  bb[i0+l*(i1+m*i2)]=gn;
}


/* find fraction of live cells in arry[l][m][n] */
void statecount(aa,l,m,n) char *aa; int l, m, n; {int i0, i1, i2;
  for (i0=0; i0<KK; i0++) lf[i0]=0;
  for (i0=0; i0<l; i0++)
  for (i1=0; i1<m; i1++)
  for (i2=0; i2<n; i2++)
    lf[aa[i0+l*(i1+m*i2)]]+=1;
}

/* display a screen of evolution   */
/* aa[] array of binary cells      */
/* l,m,n the three dimensions      */
/* q chosen plane, third direction */
arscreen(aa,l,m,n,q) char *aa; int l, m, n, q; {int i, j;
  for (j=0; j<m; j++) for (i=0; i<l; i++) videodot(j,i,aa[i+l*(j+m*(q%n))]);}

/* make a frame for the fixpoint graph  */
/* n     = dimension of arframe         */
/* l     = color of fiducial marks      */
argfram(n,l) int n, l; {int i;
for (i=0; i<=n; i++) {arbdot(i,0,0);arbdot(i,n,0);arbdot(n,i,0);arbdot(0,i,0);};
for (i=0; i<=n; i+=10) {arbdot(i,0,l);arbdot(i,n,l);arbdot(n,i,l);arbdot(0,i,l);};
}

/* put a diagonal in the fixpoint graph */
argdiag(n,l) int n, l; {int i; for (i=0; 2*i<n; i++) arbdot(2*i,2*i,l);}

/* graph statistics of rule usage */
/* beware limited count capacity  */
/* large counts look negative     */
arrstat() {int i, j, m, n; double s, t;
for (i=0, s=0.0; i<QS; i++) s+=(double)rus[i];
if  (s<1.0) {for (i=0; i<QS; i++) rus[i]=0; return;};
for (i=0; i<QS; i++) {
  t=333.3*(((double)rus[i])/s);	/* full scale = 1/10 population */
  if (t>=32000.0) t=50.0;	/* larger looks negative */
  n=(int)t;
  arsdot(i,33,0);
  m=33; if (n>33) {n=33; m=34;};
  if (n>0) for (j=0; j<n; j++) arsdot(i,j,1);
  for (j=n; j<m; j++) arsdot(i,j,2);
  rus[i]=0;
  }
}

/* graph fraction of live cells */
/* y = column for dot */
/* n = value to be graphed */
arlstat(y,n) int y, n; {int i, k;
  k=(int)(100.0*(((double)(n))/((double)(AL*AL))));
  if (k<=0) k=1; if (k>=100) k=99;
  for (i=0; i<k; i++) ldot(i,y,1);
  for (i=k; i<101; i++) ldot(i,y,2);
  for (i=0; i<101; i++) ldot(i,y+1,0);
  for (i=0; i<101; i+=10) ldot(i,y+1,3);
}

/* generate the totalistic code table */
totta() {int i0, i1, i2, i3, i4, i5, i6;
  for (i6=0; i6<KK; i6++)
  for (i5=0; i5<KK; i5++)
  for (i4=0; i4<KK; i4++)
  for (i3=0; i3<KK; i3++)
  for (i2=0; i2<KK; i2++)
  for (i1=0; i1<KK; i1++)
  for (i0=0; i0<KK; i0++)
    tc[i0+KK*(i1+KK*(i2+KK*(i3+KK*(i4+KK*(i5+KK*i6)))))]=i0+i1+i2+i3+i4+i5+i6;
}

/* generate the semitotalistic code table */
stotta() {int i0, i1, i2, i3, i4, i5, i6;
  for (i6=0; i6<KK; i6++)
  for (i5=0; i5<KK; i5++)
  for (i4=0; i4<KK; i4++)
  for (i3=0; i3<KK; i3++)
  for (i2=0; i2<KK; i2++)
  for (i1=0; i1<KK; i1++)
  for (i0=0; i0<KK; i0++)
    stc[i0+KK*(i1+KK*(i2+KK*(i3+KK*(i4+KK*(i5+KK*i6)))))] = 
	i0+i1+i2+i3+i4+i5+(TS-1)*i6;
}

/* generate the Golay code table */
argolta() {
int i, j, k;
int i0, i1, i2, i3;
int arbinq();
k=0;
for (i3=0; i3<KK; i3++)
for (i2=0; i2<KK; i2++)
for (i1=0; i1<KK; i1++)
for (i0=0; i0<KK; i0++) {
  i=arbinq(i0,i1,i2,i3);
  j=arbinq(i1,i2,i3,i0);
  if (j<i) i=j;
  j=arbinq(i2,i3,i0,i1);
  if (j<i) i=j;
  j=arbinq(i3,i0,i1,i2);
  if (j<i) i=j;
  j=arbinq(i0,i3,i2,i1);
  if (j<i) i=j;
  j=arbinq(i3,i2,i1,i0);
  if (j<i) i=j;
  j=arbinq(i2,i1,i0,i3);
  if (j<i) i=j;
  j=arbinq(i1,i0,i3,i2);
  if (j<i) i=j;
  j=arbinq(i0,i1,i2,i3);
  if (i==j) {gc[i]=k; k++;} else {gc[j]=gc[i];};
  }
/* printf("no. cl. = %3d",k); */
}

/* calculate binary equivalent */
int arbinq(i0,i1,i2,i3) int i0, i1, i2, i3; {return i0+KK*(i1+KK*(i2+KK*i3));}

/* calculate Golay class of a cell */
/* no check is made that cell isn't at array edge */
int argolcl(i0,i1,arr1) int  i0, i1, arr1[AL][AL]; {int gn;
gn=arr1[i0-1][i1];
gn=KK*gn+arr1[i0][i1-1];
gn=KK*gn+arr1[i0+1][i1];
gn=KK*gn+arr1[i0][i1+1];
if (arr1[i0][i1]==1) gn+=GH;
gn=gc[gn];
return gn;
}

/* plot self-consistent probability graph */
bpanel() {arburnc(); argfram(100); argdiag(100); bargraf(RR,100);}

/* generate Bernstein coefficients */
arburnc() {int i;
  for (i=0; i<RR+1; i++) bp[i]=0.0;
  if (golRule[0]=='1') bp[0]+=1.0;
  if (golRule[1]=='1') bp[1]+=4.0;
  if (golRule[2]=='1') bp[2]+=4.0;
  if (golRule[3]=='1') bp[2]+=2.0;
  if (golRule[4]=='1') bp[3]+=4.0;
  if (golRule[5]=='1') bp[4]+=1.0;
  if (golRule[6]=='1') bp[1]+=1.0;
  if (golRule[7]=='1') bp[2]+=4.0;
  if (golRule[8]=='1') bp[3]+=4.0;
  if (golRule[9]=='1') bp[3]+=2.0;
  if (golRule[10]=='1') bp[4]+=4.0;
  if (golRule[11]=='1') bp[5]+=1.0;
}

/* graph Bernstein polynomial */
bargraf(k,n) int k, n; {int i; double arburn(), en, p;
en=(double)(n);
for (i=0; i<n; i++) {
  p=((double)(i))/en;
  arbdot(i,(int)(en*arburn(p,k)),1);
  }
}

/* evaluate the nth degree Bernstein polynomial at point p */
double arburn(p,n) double p; int n; {double q, s, x, r; int i;
if (p>0.99) return bp[n];
q=1.0-p; r=p/q; s=0.0; x=1.0;
for (i=0; i<n;  i++) x*=q;
for (i=0; i<=n; i++, x*=r) s+=bp[i]*x;
s*=0.9998;
return(s+0.0001);
}

/* aradot(i,j,l) - place dot on automaton screen at (i,j) in color l */
aradot(i,j,l) int i, j, l; {videodot(199-(AL-1-j),60+i,l);}

/* arbdot(i,j,l) - place dot on Bernstein screen at (i,j) in color l */
arbdot(i,j,l) int i, j, l; {videodot(j,i,l);}

/* arbline(x,y,l) - place line on Bernstein screen to (x,y); l=first */
arbline(x,y,l) double x, y; int l; 
	{if (l==0) PSmoveto(y,150.0-x); else PSlineto(y,150.0-x);}

/* ldot(i,j,l) - place dot on live probability  screen at (i,j) in color l */
ldot(i,j,l) int i, j, l; {videodot(90-NF+j,i,l);}

/* arsdot(i,j,l) - place dot on statistics screen at (i,j) in color l */
arsdot(i,j,l) int i, j, l; {videodot(199-10-j,70+5*i,l);}

/* clear the cell array */
arzarry(arr1) int  arr1[AL][AL]; {int i0, i1;
for (i0=0; i0<AL; i0++) for (i1=0; i1<AL; i1++) arr1[i0][i1]=0;
}

/* display rule menu */
armenu() {}

/* display rule number by sum in color */
arscrrul(y,x) int x, y; {int i, n;
n=QS;
videocursor(0,y,x);
  arscrstr("  ");
  for (i=0;   i<n;   i++) arresch(i);
videocursor(0,y,x);
}

/* display rule number as a hexadecimal */
harscrrul() {int i, k;
  videocursor(0,0,0);
  printf("    ");
  for (i=0,  k=0; i<12;  i++) k=KK*k+sqrule[i]-'0';
  printf("%3.3X",k);
  videocursor(0,0,0);
  }

/* display Wolfram rule number of cross section */
warscrrul() {int i;
  i=sqrule[11]-'0';
  i=KK*i+sqrule[10]-'0';
  i=KK*i+sqrule[3]-'0';
  i=KK*i+sqrule[1] -'0';
  i=KK*i+sqrule[10]-'0';
  i=KK*i+sqrule[9]-'0';
  i=KK*i+sqrule[1] -'0';
  i=KK*i+sqrule[0] -'0';
  videocursor(0,0,32);
  printf("     ");
  videocursor(0,0,32);
  printf("[%2d]",i);
  videocursor(0,0,0);
  }

/* display Wolfram rule number of planar cross section */
/* calculate LCAU21 wru from a VN3D21 argument rule    */
void wxsscrrul(c,ru) char *c, ru[KK][KK][KK][KK][KK][KK][KK]; {
char ur[KK][KK][KK]; int i0, i1, i2;
  for (i0=0; i0<KK; i0++) for (i1=0; i1<KK; i1++) for (i2=0; i2<KK; i2++)
    ur[i0][i1][i2]=ru[i0][i1][i1][i1][i1][i2][i1];
  wrru21(c,ur);
  }

/* display Wolfram rule number of diagonal cross section */
/* calculate LCAU21 wru from a VN3D21 argument rule    */
void wysscrrul(c,ru) char *c, ru[KK][KK][KK][KK][KK][KK][KK]; {
char ur[KK][KK][KK]; int i0, i1, i2;
  for (i0=0; i0<KK; i0++) for (i1=0; i1<KK; i1++) for (i2=0; i2<KK; i2++)
    ur[i0][i1][i2]=ru[i0][i0][i0][i2][i2][i2][i1];
  wrru21(c,ur);
  }

/* calculate wru from an argument rule	*/
void wrru21(c,ru) char *c, ru[KK][KK][KK]; {int  k, i0, i1, i2;
  k=0;
  for (i0=KK-1; i0>=0; i0--)
  for (i1=KK-1; i1>=0; i1--)
  for (i2=KK-1; i2>=0; i2--)
    k=2*k+(ru[i0][i1][i2]-'0');
  sprintf(c,"%3d",k);
}

/* edit the rule */
aredrule() {char c; int i;
  i=0;
  while (0<1) {
//    videocursor(0,TRR,TRC+i+2);
    videoputc(sqrule[i],3);
//    videocursor(0,TRR,TRC+i+2);
    c=kbdin();
      if (c=='\015') break;				/* carriage return */
      switch (c) {
        case '0': case '1': 				/* state = 0,1 */
          sqrule[i]=c;
	  arresch(i);
          i++; if (i==QS) i=0;
	  break;
        case ' ': case '\315':				/* space = advance */
	  arresch(i);
          i++; if (i==QS) i=0;
	  break;
        case '\010': case '\313':			/* backspace */
	  arresch(i);
          if (i>0) i--;
	  break;
	case '<':					/* cursor to front */
	  arresch(i);
	  i=0;
	  break;
	case '>':					/* cursor to end */
	  arresch(i);
	  i=QS-1;
	  break;
	case '\013': case '\310':			/* up arrow */
	  qarule[i]='+';
	  arresch(i);
          i++; if (i==QS) i=0;
	  break;
	case '\012': case '\320':			/* down arrow */
	  qarule[i]=' ';
	  arresch(i);
          i++; if (i==QS) i=0;
	  break;
	default:
	  break;
          };
    };
}

/* restore character in edrule */
arresch(i) int i; {videoputc(sqrule[i],(qarule[i]==' ')?1:2);}

/* edit the plane of cells */
aredline(arr1) int arr1[AL][AL]; {
char c;
int  i, j, k, arr2[AL][AL], argolcl();

for (i=0; i<AL; i++) for (j=0; j<AL; j++) arr2[i][j]=0;

videoscroll(19,0,24,25,0,0);
videocursor(0,19,0);
arscrstr("insert states with 0,1,.");
videocursor(0,20,0);
arscrstr("move cursor using arrows");
videocursor(0,21,0);
arscrstr("NSEW R CGO fgmxz *<>^_");
videocursor(0,22,0);
arscrstr("carriage return exits");

while (0<1) {
  aradot(li,lj,3);
  videocursor(0,0,1);
  videoputc('?',1);
  c=kbdin();
  videocursor(0,0,1);
  videoputc(c,1);
  if (c == '\015') break;
  switch (c) {
    case '0': case '1':	case '.':	/* enter state */
      if (c=='.') c='1';
      i=c-'0';
      arr1[li][lj]=i;
      aradot(li,lj,i);
      break;
    case '\012': case '\320':		/* down    */
      aradot(li,lj,arr1[li][lj]);
      lj++; if (lj==AL) lj=0;
      break;
    case '\013': case '\310':		/* up      */
      aradot(li,lj,arr1[li][lj]);
      if (lj==0) lj=AL; lj--;
      break;
    case '\014': case '\315': case ' ':	/* forward */
      aradot(li,lj,arr1[li][lj]);
      li++; if (li==AL) li=0;
      break;
    case '\010': case '\313':		/* back    */
      aradot(li,lj,arr1[li][lj]);
      if (li==0) li=AL; li--;
      break;
    case '<':				/* left margin  */
      aradot(li,lj,arr1[li][lj]);
      li=0;
      break;
    case '>':				/* right margin */
      aradot(li,lj,arr1[li][lj]);
      li=AL-1;
      break;
    case '^':				/* top margin  */
      aradot(li,lj,arr1[li][lj]);
      lj=0;
      break;
   case '_':				/* bottom margin  */
      aradot(li,lj,arr1[li][lj]);
      lj=AL-1;
      break;
   case '*':				/* bottom margin  */
      aradot(li,lj,arr1[li][lj]);
      li=AL/2; 
      lj=AL/2;
      break;
     case 'f':				/* generate a frame */
//      arframe (2);
      break;
    case 'g':				/* one cycle of evolution */
//      arframe (1);
      cuonegen(arr1);
      arscreen(arr1);
      break;
    case 'm':				/* mark neighborhood */
      qarule[argolcl(li,lj,arr1)]='+';
//      arscrrul(TRR,TRC);
      break;
    case 'M':				/* mark neighborhood over a region */
      if (li<10||li>AL-11) break;
      if (lj<10||lj>AL-11) break;
      for (i=li-10; i<li+10; i++) {
      for (j=lj-10; j<lj+10; j++) {
        qarule[argolcl(i,j,arr1)]='+';
        };};
//      arscrrul(TRR,TRC);
      break;
    case 'x':				/* uncomitted transitions */
      for (k=0; k<QS; k++) qarule[k]=' ';
//      videocursor(0,TRR,TRC);
//      arscrrul(TRR,TRC);
      break;
    case 'z':				/* clear array  */
      arzarry(arr1);
      arscreen(arr1);
      break;
    case 'N':				/* rotate array up */
      for (i=0; i<AL; i++) {
        k=arr1[i][0]; 
        for (j=1; j<AL; j++) arr1[i][j-1]=arr1[i][j];
        arr1[i][AL-1]=k;
	};
      if (lj==0) lj=AL; lj--;
      arscreen(arr1);
      break;
    case 'S':				/* rotate array down */
      for (i=0; i<AL; i++) {
        k=arr1[i][AL-1];
        for (j=1; j<AL; j++) arr1[i][AL-j]=arr1[i][AL-j-1];
        arr1[i][0]=k;
	};
      lj++; if (lj==AL) lj=0;
      arscreen(arr1);
      break;
    case 'E':				/* rotate array right */
      for (j=0; j<AL; j++) {
        k=arr1[AL-1][j];
        for (i=1; i<AL; i++) arr1[AL-i][j]=arr1[AL-i-1][j];
        arr1[0][j]=k;
	};
      li++; if (li==AL) li=0;
      arscreen(arr1);
      break;
    case 'W':				/* rotate array left */
      for (j=0; j<AL; j++) {
        k=arr1[0][j];
        for (i=1; i<AL; i++) arr1[i-1][j]=arr1[i][j];
	arr1[AL-1][j]=k;
	};
      if (li==0) li=AL; li--;
      arscreen(arr1);
      break;
    case 'R':				/* reflect array in diagonal */
      for (j=1; j<AL; j++) {
      for (i=0; i<j;  i++) {
        k=arr1[i][j];
	arr1[i][j]=arr1[j][i];
        arr1[j][i]=k;
        };};
      k=li; li=lj; lj=k;
      arscreen(arr1);
      break;
    case 'C':				/* save cell array */
      for (j=0; j<AL; j++) {
      for (i=0; i<AL; i++) {
	arr2[i][j]=arr1[i][j];
	};};
      break;
    case 'G':				/* recover saved cell array */
      for (j=0; j<AL; j++) {
      for (i=0; i<AL; i++) {
	arr1[i][j]=arr2[i][j];
	};};
      arscreen(arr1);
      break;
    case 'O':				/* or saved cell array */
      for (j=0; j<AL; j++) {
      for (i=0; i<AL; i++) {
	arr1[i][j]|=arr2[i][j];
	};};
      arscreen(arr1);
      break;
    default:
      break;
      };				/* end switch */
  };					/* end while */
aradot(li,lj,arr1[li][lj]);
videoscroll(19,0,24,25,0,0);
videocursor(0,0,1);
videoputc(' ',1);
}				        /* end aredline */

/* generate random cells in arr1 */
arranarr(arr1) int  arr1[AL][AL]; {int i0, i1, i;
for (i=0, i0=0; i0<AL; i0++) for (i1=0; i1<AL; i1++) arr1[i0][i1]=random()%KK;
}

/* generate random cells in the center of arr1 */
arrancen(arr1) int  arr1[AL][AL]; {int i0, i1, ia, ib;
  for (i0=0; i0<AL; i0++) 
  for (i1=0; i1<AL; i1++) arr1[i0][i1]=0;
  ia=AL/3; ib=2*(AL/3);
  for (i0=ia; i0<ib; i0++)
  for (i1=ia; i1<ib; i1++) arr1[i0][i1]=random()%KK;
}

/* generate a random rule */
arranrul() {int i; for (i=0; i<QS; i++) sqrule[i]=random()%KK+'0';}

/* write a string in graphics mode */
arscrstr(s) char *s; {for (; *s != '\0'; s++) videoputc(*s,1); }

/* 		     layout for cell indexing				*/
/*									*/
/*		 |				 |			*/
/*		 z   y				 5   4			*/
/*	       	 | / 			       	 | / 			*/
/*      - - -x - c - x - - -	        - -  2 - 6 - 3 - - -		*/
/*	       / |  			       / |  			*/
/*	    -y  -z			     1   0			*/
/*		 |				 |			*/
/*									*/

/* enumeration of idempotent VN3D rules */
void idempot() {
int na, nb, nc, nd, ne, nf, ng, nh, ni, nj;
int nk, nl, nm, nn, no, np, nq, nr, ns, nt;
int nu, nv, nw, nx, ny;
int jj[RR];
int i, j, kk;
double all, idx[RR];
 for (j=0; j<256; j++) {
  nutoto(j);
  totoasc();
  asctobin();
  all=0.0;
for (i=0; i<RR; i++) idx[i]=0.0;
for (na=0; na<KK; na++) 
for (nb=0; nb<KK; nb++) 
for (nc=0; nc<KK; nc++) 
for (nd=0; nd<KK; nd++) 
for (ne=0; ne<KK; ne++) 
for (nf=0; nf<KK; nf++) 
for (ng=0; ng<KK; ng++) 
for (nh=0; nh<KK; nh++) 
for (ni=0; ni<KK; ni++) 
for (nj=0; nj<KK; nj++) 
for (nk=0; nk<KK; nk++) 
for (nl=0; nl<KK; nl++) 
for (nm=0; nm<KK; nm++) 
for (nn=0; nn<KK; nn++) 
for (no=0; no<KK; no++) 
for (np=0; np<KK; np++) 
for (nq=0; nq<KK; nq++) 
for (nr=0; nr<KK; nr++) 
for (ns=0; ns<KK; ns++) 
for (nt=0; nt<KK; nt++) 
for (nu=0; nu<KK; nu++) 
for (nv=0; nv<KK; nv++) 
for (nw=0; nw<KK; nw++) 
for (nx=0; nx<KK; nx++) 
for (ny=0; ny<KK; ny++) {
//	jj[0]=binrule[00-2][0-1-1][-10-1][10-1][01-1][000][00-1];
//	jj[1]=binrule[0-1-1][0-20][-1-10][1-10][000][0-11][0-10];
//	jj[2]=binrule[-10-1][-1-10][-200][000][-110][-101][-100];
//	jj[3]=binrule[10-1][1-10][000][200][110][101][100];
//	jj[4]=binrule[01-1][000][-110][110][020][011][010];
//	jj[5]=binrule[000][0-11][-101][101][011][002][001];
//	jj[6]=binrule[00-1][0-10][-100][100][010][001][000];
	jj[0]=binrule[na][nm][nn][no][np][ny][ng];
	jj[1]=binrule[nm][nb][nq][nr][ny][ns][nh];
	jj[2]=binrule[nn][nq][nc][ny][nt][nu][ni];
	jj[3]=binrule[no][nr][ny][nd][nv][nw][nj];
	jj[4]=binrule[np][ny][nt][nv][ne][nx][nk];
	jj[5]=binrule[ny][ns][nu][nw][nx][nf][nl];
	jj[6]=binrule[ng][nh][ni][nj][nk][nl][ny];
	      kk=binrule[jj[0]][jj[1]][jj[2]][jj[3]][jj[4]][jj[5]][jj[6]];
	all+=1.0;
	for (i=0; i<RR; i++) if (kk==jj[i]) idx[i]+=1.0;
  }
printf("idempotency indices for SQAR totalistic rule %3d:\n",j);
for (i=0; i<RR; i++) printf("idx %d = %6.3f, ",i,idx[i]/all);
printf("\n");
  }
}

// # include "nxruin21.c"

/* end nxsqar21.c */
